/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// CookieConfigApplyConfiguration represents a declarative configuration of the CookieConfig type for use
// with apply.
//
// CookieConfig defines the configuration for cookie-based session persistence.
type CookieConfigApplyConfiguration struct {
	// LifetimeType specifies whether the cookie has a permanent or
	// session-based lifetime. A permanent cookie persists until its
	// specified expiry time, defined by the Expires or Max-Age cookie
	// attributes, while a session cookie is deleted when the current
	// session ends.
	//
	// When set to "Permanent", AbsoluteTimeout indicates the
	// cookie's lifetime via the Expires or Max-Age cookie attributes
	// and is required.
	//
	// When set to "Session", AbsoluteTimeout indicates the
	// absolute lifetime of the cookie tracked by the gateway and
	// is optional.
	//
	// Defaults to "Session".
	//
	// Support: Core for "Session" type
	//
	// Support: Extended for "Permanent" type
	LifetimeType *apisv1.CookieLifetimeType `json:"lifetimeType,omitempty"`
}

// CookieConfigApplyConfiguration constructs a declarative configuration of the CookieConfig type for use with
// apply.
func CookieConfig() *CookieConfigApplyConfiguration {
	return &CookieConfigApplyConfiguration{}
}

// WithLifetimeType sets the LifetimeType field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the LifetimeType field is set to the value of the last call.
func (b *CookieConfigApplyConfiguration) WithLifetimeType(value apisv1.CookieLifetimeType) *CookieConfigApplyConfiguration {
	b.LifetimeType = &value
	return b
}
