/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

// TLSPortConfigApplyConfiguration represents a declarative configuration of the TLSPortConfig type for use
// with apply.
type TLSPortConfigApplyConfiguration struct {
	// The Port indicates the Port Number to which the TLS configuration will be
	// applied. This configuration will be applied to all Listeners handling HTTPS
	// traffic that match this port.
	//
	// Support: Core
	Port *int32 `json:"port,omitempty"`
	// TLS store the configuration that will be applied to all Listeners handling
	// HTTPS traffic and matching given port.
	//
	// Support: Core
	TLS *TLSConfigApplyConfiguration `json:"tls,omitempty"`
}

// TLSPortConfigApplyConfiguration constructs a declarative configuration of the TLSPortConfig type for use with
// apply.
func TLSPortConfig() *TLSPortConfigApplyConfiguration {
	return &TLSPortConfigApplyConfiguration{}
}

// WithPort sets the Port field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Port field is set to the value of the last call.
func (b *TLSPortConfigApplyConfiguration) WithPort(value int32) *TLSPortConfigApplyConfiguration {
	b.Port = &value
	return b
}

// WithTLS sets the TLS field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the TLS field is set to the value of the last call.
func (b *TLSPortConfigApplyConfiguration) WithTLS(value *TLSConfigApplyConfiguration) *TLSPortConfigApplyConfiguration {
	b.TLS = value
	return b
}
