// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/load_balancing_policies/random_subsetting/v3/random_subsetting.proto

package random_subsettingv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/cluster/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Random Subsetting Load Balancing Policy
//
// This policy selects a subset of endpoints and passes them to the child LB policy.
// It maintains 2 important properties:
// 1. The policy tries to distribute connections among servers as equally as possible. The higher
// “(N_clients*subset_size)/N_servers“ ratio is, the closer the resulting server connection
// distribution is to uniform.
// 2. The policy minimizes the amount of connection churn generated during server scale-ups by
// using rendezvous hashing
//
// See the :ref:`load balancing architecture
// overview<arch_overview_load_balancing_types>` for more information.
//
// [#not-implemented-hide:]
type RandomSubsetting struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// subset_size indicates how many backends every client will be connected to.
	// The value must be greater than 0.
	SubsetSize *wrapperspb.UInt32Value `protobuf:"bytes,1,opt,name=subset_size,json=subsetSize,proto3" json:"subset_size,omitempty"`
	// The config for the child policy.
	// The value is required.
	ChildPolicy   *v3.LoadBalancingPolicy `protobuf:"bytes,2,opt,name=child_policy,json=childPolicy,proto3" json:"child_policy,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RandomSubsetting) Reset() {
	*x = RandomSubsetting{}
	mi := &file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RandomSubsetting) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RandomSubsetting) ProtoMessage() {}

func (x *RandomSubsetting) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RandomSubsetting.ProtoReflect.Descriptor instead.
func (*RandomSubsetting) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescGZIP(), []int{0}
}

func (x *RandomSubsetting) GetSubsetSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.SubsetSize
	}
	return nil
}

func (x *RandomSubsetting) GetChildPolicy() *v3.LoadBalancingPolicy {
	if x != nil {
		return x.ChildPolicy
	}
	return nil
}

var File_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto protoreflect.FileDescriptor

const file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDesc = "" +
	"\n" +
	"Uenvoy/extensions/load_balancing_policies/random_subsetting/v3/random_subsetting.proto\x12=envoy.extensions.load_balancing_policies.random_subsetting.v3\x1a%envoy/config/cluster/v3/cluster.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb5\x01\n" +
	"\x10RandomSubsetting\x12F\n" +
	"\vsubset_size\x18\x01 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02 \x00R\n" +
	"subsetSize\x12Y\n" +
	"\fchild_policy\x18\x02 \x01(\v2,.envoy.config.cluster.v3.LoadBalancingPolicyB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vchildPolicyB\xe8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Kio.envoyproxy.envoy.extensions.load_balancing_policies.random_subsetting.v3B\x15RandomSubsettingProtoP\x01Zxgithub.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/random_subsetting/v3;random_subsettingv3b\x06proto3"

var (
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescOnce sync.Once
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescData []byte
)

func file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescGZIP() []byte {
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDesc), len(file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDesc)))
	})
	return file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDescData
}

var file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_goTypes = []any{
	(*RandomSubsetting)(nil),       // 0: envoy.extensions.load_balancing_policies.random_subsetting.v3.RandomSubsetting
	(*wrapperspb.UInt32Value)(nil), // 1: google.protobuf.UInt32Value
	(*v3.LoadBalancingPolicy)(nil), // 2: envoy.config.cluster.v3.LoadBalancingPolicy
}
var file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.load_balancing_policies.random_subsetting.v3.RandomSubsetting.subset_size:type_name -> google.protobuf.UInt32Value
	2, // 1: envoy.extensions.load_balancing_policies.random_subsetting.v3.RandomSubsetting.child_policy:type_name -> envoy.config.cluster.v3.LoadBalancingPolicy
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_init()
}
func file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_init() {
	if File_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDesc), len(file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_msgTypes,
	}.Build()
	File_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto = out.File
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_goTypes = nil
	file_envoy_extensions_load_balancing_policies_random_subsetting_v3_random_subsetting_proto_depIdxs = nil
}
