// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/connection_limit/v3/connection_limit.proto

package connection_limitv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ConnectionLimit struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The prefix to use when emitting :ref:`statistics
	// <config_network_filters_connection_limit_stats>`.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// The max connections configuration to use for new incoming connections that are processed
	// by the filter's filter chain. When max_connection is reached, the incoming connection
	// will be closed after delay duration.
	MaxConnections *wrapperspb.UInt64Value `protobuf:"bytes,2,opt,name=max_connections,json=maxConnections,proto3" json:"max_connections,omitempty"`
	// The delay configuration to use for rejecting the connection after some specified time duration
	// instead of immediately rejecting the connection. That way, a malicious user is not able to
	// retry as fast as possible which provides a better DoS protection for Envoy. If this is not present,
	// the connection will be closed immediately.
	Delay *durationpb.Duration `protobuf:"bytes,3,opt,name=delay,proto3" json:"delay,omitempty"`
	// Runtime flag that controls whether the filter is enabled or not. If not specified, defaults
	// to enabled.
	RuntimeEnabled *v3.RuntimeFeatureFlag `protobuf:"bytes,4,opt,name=runtime_enabled,json=runtimeEnabled,proto3" json:"runtime_enabled,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ConnectionLimit) Reset() {
	*x = ConnectionLimit{}
	mi := &file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConnectionLimit) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConnectionLimit) ProtoMessage() {}

func (x *ConnectionLimit) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConnectionLimit.ProtoReflect.Descriptor instead.
func (*ConnectionLimit) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescGZIP(), []int{0}
}

func (x *ConnectionLimit) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *ConnectionLimit) GetMaxConnections() *wrapperspb.UInt64Value {
	if x != nil {
		return x.MaxConnections
	}
	return nil
}

func (x *ConnectionLimit) GetDelay() *durationpb.Duration {
	if x != nil {
		return x.Delay
	}
	return nil
}

func (x *ConnectionLimit) GetRuntimeEnabled() *v3.RuntimeFeatureFlag {
	if x != nil {
		return x.RuntimeEnabled
	}
	return nil
}

var File_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDesc = "" +
	"\n" +
	"Kenvoy/extensions/filters/network/connection_limit/v3/connection_limit.proto\x124envoy.extensions.filters.network.connection_limit.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x8f\x02\n" +
	"\x0fConnectionLimit\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12N\n" +
	"\x0fmax_connections\x18\x02 \x01(\v2\x1c.google.protobuf.UInt64ValueB\a\xfaB\x042\x02(\x01R\x0emaxConnections\x12/\n" +
	"\x05delay\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\x05delay\x12Q\n" +
	"\x0fruntime_enabled\x18\x04 \x01(\v2(.envoy.config.core.v3.RuntimeFeatureFlagR\x0eruntimeEnabledB\xd4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Bio.envoyproxy.envoy.extensions.filters.network.connection_limit.v3B\x14ConnectionLimitProtoP\x01Zngithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/connection_limit/v3;connection_limitv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDesc), len(file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDescData
}

var file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_goTypes = []any{
	(*ConnectionLimit)(nil),        // 0: envoy.extensions.filters.network.connection_limit.v3.ConnectionLimit
	(*wrapperspb.UInt64Value)(nil), // 1: google.protobuf.UInt64Value
	(*durationpb.Duration)(nil),    // 2: google.protobuf.Duration
	(*v3.RuntimeFeatureFlag)(nil),  // 3: envoy.config.core.v3.RuntimeFeatureFlag
}
var file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.connection_limit.v3.ConnectionLimit.max_connections:type_name -> google.protobuf.UInt64Value
	2, // 1: envoy.extensions.filters.network.connection_limit.v3.ConnectionLimit.delay:type_name -> google.protobuf.Duration
	3, // 2: envoy.extensions.filters.network.connection_limit.v3.ConnectionLimit.runtime_enabled:type_name -> envoy.config.core.v3.RuntimeFeatureFlag
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_init() }
func file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_init() {
	if File_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDesc), len(file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto = out.File
	file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_goTypes = nil
	file_envoy_extensions_filters_network_connection_limit_v3_connection_limit_proto_depIdxs = nil
}
