// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha3

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for Gateway
func (this *Gateway) MarshalJSON() ([]byte, error) {
	str, err := GatewayMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Gateway
func (this *Gateway) UnmarshalJSON(b []byte) error {
	return GatewayUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for Server
func (this *Server) MarshalJSON() ([]byte, error) {
	str, err := GatewayMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Server
func (this *Server) UnmarshalJSON(b []byte) error {
	return GatewayUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for Port
func (this *Port) MarshalJSON() ([]byte, error) {
	str, err := GatewayMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Port
func (this *Port) UnmarshalJSON(b []byte) error {
	return GatewayUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ServerTLSSettings
func (this *ServerTLSSettings) MarshalJSON() ([]byte, error) {
	str, err := GatewayMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServerTLSSettings
func (this *ServerTLSSettings) UnmarshalJSON(b []byte) error {
	return GatewayUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ServerTLSSettings_TLSCertificate
func (this *ServerTLSSettings_TLSCertificate) MarshalJSON() ([]byte, error) {
	str, err := GatewayMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServerTLSSettings_TLSCertificate
func (this *ServerTLSSettings_TLSCertificate) UnmarshalJSON(b []byte) error {
	return GatewayUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	GatewayMarshaler   = &jsonpb.Marshaler{}
	GatewayUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
