// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/access_loggers/stats/v3/stats.proto

package statsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/cncf/xds/go/xds/type/matcher/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/data/accesslog/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The histogram units. The units are needed for some stat sinks.
type Config_Histogram_Unit int32

const (
	Config_Histogram_Unspecified  Config_Histogram_Unit = 0
	Config_Histogram_Bytes        Config_Histogram_Unit = 1
	Config_Histogram_Microseconds Config_Histogram_Unit = 2
	Config_Histogram_Milliseconds Config_Histogram_Unit = 3
	// Values are scaled to range 0-1.0, indicating 0%-100%. Values can be outside this range,
	// but must be positive. Values extremely far out of this range may overflow.
	Config_Histogram_Percent Config_Histogram_Unit = 4
)

// Enum value maps for Config_Histogram_Unit.
var (
	Config_Histogram_Unit_name = map[int32]string{
		0: "Unspecified",
		1: "Bytes",
		2: "Microseconds",
		3: "Milliseconds",
		4: "Percent",
	}
	Config_Histogram_Unit_value = map[string]int32{
		"Unspecified":  0,
		"Bytes":        1,
		"Microseconds": 2,
		"Milliseconds": 3,
		"Percent":      4,
	}
)

func (x Config_Histogram_Unit) Enum() *Config_Histogram_Unit {
	p := new(Config_Histogram_Unit)
	*p = x
	return p
}

func (x Config_Histogram_Unit) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Config_Histogram_Unit) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_enumTypes[0].Descriptor()
}

func (Config_Histogram_Unit) Type() protoreflect.EnumType {
	return &file_envoy_extensions_access_loggers_stats_v3_stats_proto_enumTypes[0]
}

func (x Config_Histogram_Unit) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Config_Histogram_Unit.Descriptor instead.
func (Config_Histogram_Unit) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 2, 0}
}

// [#next-free-field: 6]
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The stat prefix for the generated stats.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// The histograms this logger will emit.
	Histograms []*Config_Histogram `protobuf:"bytes,3,rep,name=histograms,proto3" json:"histograms,omitempty"`
	// The counters this logger will emit.
	Counters []*Config_Counter `protobuf:"bytes,4,rep,name=counters,proto3" json:"counters,omitempty"`
	// The gauges this logger will emit.
	Gauges        []*Config_Gauge `protobuf:"bytes,5,rep,name=gauges,proto3" json:"gauges,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *Config) GetHistograms() []*Config_Histogram {
	if x != nil {
		return x.Histograms
	}
	return nil
}

func (x *Config) GetCounters() []*Config_Counter {
	if x != nil {
		return x.Counters
	}
	return nil
}

func (x *Config) GetGauges() []*Config_Gauge {
	if x != nil {
		return x.Gauges
	}
	return nil
}

// Defines a tag on a stat.
type Config_Tag struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the tag.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The value of the tag, using :ref:`command operators
	// <config_access_log_command_operators>`.
	ValueFormat string `protobuf:"bytes,2,opt,name=value_format,json=valueFormat,proto3" json:"value_format,omitempty"`
	// The custom rules to generate the stat tags. Currently, the only
	// supported input is
	// :ref:`Stat tag value input <envoy_v3_api_msg_extensions.matching.common_inputs.stats.v3.StatTagValueInput>`.
	// The supported actions are
	// - :ref:`Transform stat action <envoy_v3_api_msg_extensions.matching.actions.transform_stat.v3.TransformStat>`.
	Rules         *v3.Matcher `protobuf:"bytes,3,opt,name=rules,proto3" json:"rules,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Tag) Reset() {
	*x = Config_Tag{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Tag) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Tag) ProtoMessage() {}

func (x *Config_Tag) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Tag.ProtoReflect.Descriptor instead.
func (*Config_Tag) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Config_Tag) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Config_Tag) GetValueFormat() string {
	if x != nil {
		return x.ValueFormat
	}
	return ""
}

func (x *Config_Tag) GetRules() *v3.Matcher {
	if x != nil {
		return x.Rules
	}
	return nil
}

// Defines the name and tags of a stat.
type Config_Stat struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the stat.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The tags for the stat.
	Tags          []*Config_Tag `protobuf:"bytes,2,rep,name=tags,proto3" json:"tags,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Stat) Reset() {
	*x = Config_Stat{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Stat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Stat) ProtoMessage() {}

func (x *Config_Stat) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Stat.ProtoReflect.Descriptor instead.
func (*Config_Stat) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 1}
}

func (x *Config_Stat) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Config_Stat) GetTags() []*Config_Tag {
	if x != nil {
		return x.Tags
	}
	return nil
}

// Configuration for a histogram stat.
type Config_Histogram struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name and tags of this histogram.
	Stat *Config_Stat `protobuf:"bytes,1,opt,name=stat,proto3" json:"stat,omitempty"`
	// The units for this histogram.
	Unit Config_Histogram_Unit `protobuf:"varint,2,opt,name=unit,proto3,enum=envoy.extensions.access_loggers.stats.v3.Config_Histogram_Unit" json:"unit,omitempty"`
	// The format string for the value of this histogram, using :ref:`command operators <config_access_log_command_operators>`.
	// This must evaluate to a positive number.
	ValueFormat   string `protobuf:"bytes,3,opt,name=value_format,json=valueFormat,proto3" json:"value_format,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Histogram) Reset() {
	*x = Config_Histogram{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Histogram) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Histogram) ProtoMessage() {}

func (x *Config_Histogram) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Histogram.ProtoReflect.Descriptor instead.
func (*Config_Histogram) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 2}
}

func (x *Config_Histogram) GetStat() *Config_Stat {
	if x != nil {
		return x.Stat
	}
	return nil
}

func (x *Config_Histogram) GetUnit() Config_Histogram_Unit {
	if x != nil {
		return x.Unit
	}
	return Config_Histogram_Unspecified
}

func (x *Config_Histogram) GetValueFormat() string {
	if x != nil {
		return x.ValueFormat
	}
	return ""
}

// Configuration for a counter stat.
type Config_Counter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name and tags of this counter.
	Stat *Config_Stat `protobuf:"bytes,1,opt,name=stat,proto3" json:"stat,omitempty"`
	// The format string for the value to add to this counter, using :ref:`command operators <config_access_log_command_operators>`.
	// One of “value_format“ or “value_fixed“ must be configured.
	ValueFormat string `protobuf:"bytes,2,opt,name=value_format,json=valueFormat,proto3" json:"value_format,omitempty"`
	// A fixed value to add to this counter.
	// One of “value_format“ or “value_fixed“ must be configured.
	ValueFixed    *wrapperspb.UInt64Value `protobuf:"bytes,3,opt,name=value_fixed,json=valueFixed,proto3" json:"value_fixed,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Counter) Reset() {
	*x = Config_Counter{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Counter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Counter) ProtoMessage() {}

func (x *Config_Counter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Counter.ProtoReflect.Descriptor instead.
func (*Config_Counter) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 3}
}

func (x *Config_Counter) GetStat() *Config_Stat {
	if x != nil {
		return x.Stat
	}
	return nil
}

func (x *Config_Counter) GetValueFormat() string {
	if x != nil {
		return x.ValueFormat
	}
	return ""
}

func (x *Config_Counter) GetValueFixed() *wrapperspb.UInt64Value {
	if x != nil {
		return x.ValueFixed
	}
	return nil
}

// Configuration for a gauge stat. Gauges can be used to add, subtract, or set
// values, and are useful for tracking concurrency or other mutable values
// over time.
// [#next-free-field: 6]
type Config_Gauge struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name and tags of this gauge.
	Stat *Config_Stat `protobuf:"bytes,1,opt,name=stat,proto3" json:"stat,omitempty"`
	// The format string for the value of this gauge, using :ref:`command
	// operators <config_access_log_command_operators>`. This must evaluate to a
	// positive number.
	ValueFormat string `protobuf:"bytes,2,opt,name=value_format,json=valueFormat,proto3" json:"value_format,omitempty"`
	// A fixed value to add/subtract/set to this gauge.
	// One of “value_format“ or “value_fixed“ must be configured.
	ValueFixed *wrapperspb.UInt64Value `protobuf:"bytes,3,opt,name=value_fixed,json=valueFixed,proto3" json:"value_fixed,omitempty"`
	// The PairedAddSubtract operation.
	// Only one of PairedAddSubtract and Set can be defined.
	AddSubtract *Config_Gauge_PairedAddSubtract `protobuf:"bytes,4,opt,name=add_subtract,json=addSubtract,proto3" json:"add_subtract,omitempty"`
	// The Set operation.
	// Only one of PairedAddSubtract and Set can be defined.
	Set           *Config_Gauge_Set `protobuf:"bytes,5,opt,name=set,proto3" json:"set,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Gauge) Reset() {
	*x = Config_Gauge{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Gauge) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Gauge) ProtoMessage() {}

func (x *Config_Gauge) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Gauge.ProtoReflect.Descriptor instead.
func (*Config_Gauge) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 4}
}

func (x *Config_Gauge) GetStat() *Config_Stat {
	if x != nil {
		return x.Stat
	}
	return nil
}

func (x *Config_Gauge) GetValueFormat() string {
	if x != nil {
		return x.ValueFormat
	}
	return ""
}

func (x *Config_Gauge) GetValueFixed() *wrapperspb.UInt64Value {
	if x != nil {
		return x.ValueFixed
	}
	return nil
}

func (x *Config_Gauge) GetAddSubtract() *Config_Gauge_PairedAddSubtract {
	if x != nil {
		return x.AddSubtract
	}
	return nil
}

func (x *Config_Gauge) GetSet() *Config_Gauge_Set {
	if x != nil {
		return x.Set
	}
	return nil
}

// The Set operation config.
type Config_Gauge_Set struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The access log type to trigger the operation.
	LogType       v31.AccessLogType `protobuf:"varint,1,opt,name=log_type,json=logType,proto3,enum=envoy.data.accesslog.v3.AccessLogType" json:"log_type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Gauge_Set) Reset() {
	*x = Config_Gauge_Set{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Gauge_Set) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Gauge_Set) ProtoMessage() {}

func (x *Config_Gauge_Set) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Gauge_Set.ProtoReflect.Descriptor instead.
func (*Config_Gauge_Set) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 4, 0}
}

func (x *Config_Gauge_Set) GetLogType() v31.AccessLogType {
	if x != nil {
		return x.LogType
	}
	return v31.AccessLogType(0)
}

// The PairedAddSubtract operation config.
// Usage restrictions:
//
//  1. We only support add first then subtract logic and we rely on the symmetrical log types
//     (e.g., DownstreamStart/DownstreamEnd) to increment and decrement the gauge.
//  2. During runtime, sub_log_type will execute if and only if add_log_type operation has
//     been done, tracked by inflight counter in filter state.
//  3. If the add_log_type operation was executed, the sub_log_type will happen when the
//     stream/connection is closed, even if the configured log type didn't happen.
type Config_Gauge_PairedAddSubtract struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The access log type to trigger the add operation.
	AddLogType v31.AccessLogType `protobuf:"varint,1,opt,name=add_log_type,json=addLogType,proto3,enum=envoy.data.accesslog.v3.AccessLogType" json:"add_log_type,omitempty"`
	// The access log type to trigger the subtract operation.
	SubLogType    v31.AccessLogType `protobuf:"varint,2,opt,name=sub_log_type,json=subLogType,proto3,enum=envoy.data.accesslog.v3.AccessLogType" json:"sub_log_type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config_Gauge_PairedAddSubtract) Reset() {
	*x = Config_Gauge_PairedAddSubtract{}
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config_Gauge_PairedAddSubtract) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config_Gauge_PairedAddSubtract) ProtoMessage() {}

func (x *Config_Gauge_PairedAddSubtract) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config_Gauge_PairedAddSubtract.ProtoReflect.Descriptor instead.
func (*Config_Gauge_PairedAddSubtract) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP(), []int{0, 4, 1}
}

func (x *Config_Gauge_PairedAddSubtract) GetAddLogType() v31.AccessLogType {
	if x != nil {
		return x.AddLogType
	}
	return v31.AccessLogType(0)
}

func (x *Config_Gauge_PairedAddSubtract) GetSubLogType() v31.AccessLogType {
	if x != nil {
		return x.SubLogType
	}
	return v31.AccessLogType(0)
}

var File_envoy_extensions_access_loggers_stats_v3_stats_proto protoreflect.FileDescriptor

const file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDesc = "" +
	"\n" +
	"4envoy/extensions/access_loggers/stats/v3/stats.proto\x12(envoy.extensions.access_loggers.stats.v3\x1a'envoy/data/accesslog/v3/accesslog.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf4\r\n" +
	"\x06Config\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12Z\n" +
	"\n" +
	"histograms\x18\x03 \x03(\v2:.envoy.extensions.access_loggers.stats.v3.Config.HistogramR\n" +
	"histograms\x12T\n" +
	"\bcounters\x18\x04 \x03(\v28.envoy.extensions.access_loggers.stats.v3.Config.CounterR\bcounters\x12N\n" +
	"\x06gauges\x18\x05 \x03(\v26.envoy.extensions.access_loggers.stats.v3.Config.GaugeR\x06gauges\x1a\x82\x01\n" +
	"\x03Tag\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12*\n" +
	"\fvalue_format\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vvalueFormat\x122\n" +
	"\x05rules\x18\x03 \x01(\v2\x1c.xds.type.matcher.v3.MatcherR\x05rules\x1am\n" +
	"\x04Stat\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12H\n" +
	"\x04tags\x18\x02 \x03(\v24.envoy.extensions.access_loggers.stats.v3.Config.TagR\x04tags\x1a\xc6\x02\n" +
	"\tHistogram\x12S\n" +
	"\x04stat\x18\x01 \x01(\v25.envoy.extensions.access_loggers.stats.v3.Config.StatB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x04stat\x12]\n" +
	"\x04unit\x18\x02 \x01(\x0e2?.envoy.extensions.access_loggers.stats.v3.Config.Histogram.UnitB\b\xfaB\x05\x82\x01\x02\x10\x01R\x04unit\x120\n" +
	"\fvalue_format\x18\x03 \x01(\tB\r\xfaB\n" +
	"r\b\x10\x01:\x01%B\x01%R\vvalueFormat\"S\n" +
	"\x04Unit\x12\x0f\n" +
	"\vUnspecified\x10\x00\x12\t\n" +
	"\x05Bytes\x10\x01\x12\x10\n" +
	"\fMicroseconds\x10\x02\x12\x10\n" +
	"\fMilliseconds\x10\x03\x12\v\n" +
	"\aPercent\x10\x04\x1a\xd9\x01\n" +
	"\aCounter\x12S\n" +
	"\x04stat\x18\x01 \x01(\v25.envoy.extensions.access_loggers.stats.v3.Config.StatB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x04stat\x121\n" +
	"\fvalue_format\x18\x02 \x01(\tB\x0e\xfaB\vr\t:\x01%B\x01%\xd0\x01\x01R\vvalueFormat\x12F\n" +
	"\vvalue_fixed\x18\x03 \x01(\v2\x1c.google.protobuf.UInt64ValueB\a\xfaB\x042\x02 \x00R\n" +
	"valueFixed\x1a\xa4\x05\n" +
	"\x05Gauge\x12S\n" +
	"\x04stat\x18\x01 \x01(\v25.envoy.extensions.access_loggers.stats.v3.Config.StatB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x04stat\x121\n" +
	"\fvalue_format\x18\x02 \x01(\tB\x0e\xfaB\vr\t:\x01%B\x01%\xd0\x01\x01R\vvalueFormat\x12F\n" +
	"\vvalue_fixed\x18\x03 \x01(\v2\x1c.google.protobuf.UInt64ValueB\a\xfaB\x042\x02 \x00R\n" +
	"valueFixed\x12k\n" +
	"\fadd_subtract\x18\x04 \x01(\v2H.envoy.extensions.access_loggers.stats.v3.Config.Gauge.PairedAddSubtractR\vaddSubtract\x12L\n" +
	"\x03set\x18\x05 \x01(\v2:.envoy.extensions.access_loggers.stats.v3.Config.Gauge.SetR\x03set\x1aR\n" +
	"\x03Set\x12K\n" +
	"\blog_type\x18\x01 \x01(\x0e2&.envoy.data.accesslog.v3.AccessLogTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\alogType\x1a\xbb\x01\n" +
	"\x11PairedAddSubtract\x12R\n" +
	"\fadd_log_type\x18\x01 \x01(\x0e2&.envoy.data.accesslog.v3.AccessLogTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\n" +
	"addLogType\x12R\n" +
	"\fsub_log_type\x18\x02 \x01(\x0e2&.envoy.data.accesslog.v3.AccessLogTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\n" +
	"subLogTypeB\xa7\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"6io.envoyproxy.envoy.extensions.access_loggers.stats.v3B\n" +
	"StatsProtoP\x01ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/stats/v3;statsv3b\x06proto3"

var (
	file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescOnce sync.Once
	file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescData []byte
)

func file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescGZIP() []byte {
	file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDesc), len(file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDesc)))
	})
	return file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDescData
}

var file_envoy_extensions_access_loggers_stats_v3_stats_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_envoy_extensions_access_loggers_stats_v3_stats_proto_goTypes = []any{
	(Config_Histogram_Unit)(0),             // 0: envoy.extensions.access_loggers.stats.v3.Config.Histogram.Unit
	(*Config)(nil),                         // 1: envoy.extensions.access_loggers.stats.v3.Config
	(*Config_Tag)(nil),                     // 2: envoy.extensions.access_loggers.stats.v3.Config.Tag
	(*Config_Stat)(nil),                    // 3: envoy.extensions.access_loggers.stats.v3.Config.Stat
	(*Config_Histogram)(nil),               // 4: envoy.extensions.access_loggers.stats.v3.Config.Histogram
	(*Config_Counter)(nil),                 // 5: envoy.extensions.access_loggers.stats.v3.Config.Counter
	(*Config_Gauge)(nil),                   // 6: envoy.extensions.access_loggers.stats.v3.Config.Gauge
	(*Config_Gauge_Set)(nil),               // 7: envoy.extensions.access_loggers.stats.v3.Config.Gauge.Set
	(*Config_Gauge_PairedAddSubtract)(nil), // 8: envoy.extensions.access_loggers.stats.v3.Config.Gauge.PairedAddSubtract
	(*v3.Matcher)(nil),                     // 9: xds.type.matcher.v3.Matcher
	(*wrapperspb.UInt64Value)(nil),         // 10: google.protobuf.UInt64Value
	(v31.AccessLogType)(0),                 // 11: envoy.data.accesslog.v3.AccessLogType
}
var file_envoy_extensions_access_loggers_stats_v3_stats_proto_depIdxs = []int32{
	4,  // 0: envoy.extensions.access_loggers.stats.v3.Config.histograms:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Histogram
	5,  // 1: envoy.extensions.access_loggers.stats.v3.Config.counters:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Counter
	6,  // 2: envoy.extensions.access_loggers.stats.v3.Config.gauges:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Gauge
	9,  // 3: envoy.extensions.access_loggers.stats.v3.Config.Tag.rules:type_name -> xds.type.matcher.v3.Matcher
	2,  // 4: envoy.extensions.access_loggers.stats.v3.Config.Stat.tags:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Tag
	3,  // 5: envoy.extensions.access_loggers.stats.v3.Config.Histogram.stat:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Stat
	0,  // 6: envoy.extensions.access_loggers.stats.v3.Config.Histogram.unit:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Histogram.Unit
	3,  // 7: envoy.extensions.access_loggers.stats.v3.Config.Counter.stat:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Stat
	10, // 8: envoy.extensions.access_loggers.stats.v3.Config.Counter.value_fixed:type_name -> google.protobuf.UInt64Value
	3,  // 9: envoy.extensions.access_loggers.stats.v3.Config.Gauge.stat:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Stat
	10, // 10: envoy.extensions.access_loggers.stats.v3.Config.Gauge.value_fixed:type_name -> google.protobuf.UInt64Value
	8,  // 11: envoy.extensions.access_loggers.stats.v3.Config.Gauge.add_subtract:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Gauge.PairedAddSubtract
	7,  // 12: envoy.extensions.access_loggers.stats.v3.Config.Gauge.set:type_name -> envoy.extensions.access_loggers.stats.v3.Config.Gauge.Set
	11, // 13: envoy.extensions.access_loggers.stats.v3.Config.Gauge.Set.log_type:type_name -> envoy.data.accesslog.v3.AccessLogType
	11, // 14: envoy.extensions.access_loggers.stats.v3.Config.Gauge.PairedAddSubtract.add_log_type:type_name -> envoy.data.accesslog.v3.AccessLogType
	11, // 15: envoy.extensions.access_loggers.stats.v3.Config.Gauge.PairedAddSubtract.sub_log_type:type_name -> envoy.data.accesslog.v3.AccessLogType
	16, // [16:16] is the sub-list for method output_type
	16, // [16:16] is the sub-list for method input_type
	16, // [16:16] is the sub-list for extension type_name
	16, // [16:16] is the sub-list for extension extendee
	0,  // [0:16] is the sub-list for field type_name
}

func init() { file_envoy_extensions_access_loggers_stats_v3_stats_proto_init() }
func file_envoy_extensions_access_loggers_stats_v3_stats_proto_init() {
	if File_envoy_extensions_access_loggers_stats_v3_stats_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDesc), len(file_envoy_extensions_access_loggers_stats_v3_stats_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_access_loggers_stats_v3_stats_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_access_loggers_stats_v3_stats_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_access_loggers_stats_v3_stats_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_access_loggers_stats_v3_stats_proto_msgTypes,
	}.Build()
	File_envoy_extensions_access_loggers_stats_v3_stats_proto = out.File
	file_envoy_extensions_access_loggers_stats_v3_stats_proto_goTypes = nil
	file_envoy_extensions_access_loggers_stats_v3_stats_proto_depIdxs = nil
}
