// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/geoip/v3/geoip.proto

package geoipv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The network geolocation filter performs IP geolocation lookups on incoming connections
// and stores the results in the connection's filter state under the well-known key
// “envoy.geoip“. The stored data is a “GeoipInfo“ object that supports
// serialization for access logging and field-level access.
//
// See :ref:`well known filter state <well_known_filter_state>` for details on accessing
// the geolocation data.
type Geoip struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The prefix to use when emitting statistics. This is useful when there are multiple
	// listeners configured with geoip filters, allowing stats to be grouped per listener.
	// For example, with “stat_prefix: "listener_1."“, stats would be emitted as
	// “listener_1.geoip.total“.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// Geoip driver specific configuration which depends on the driver being instantiated.
	// [#extension-category: envoy.geoip_providers]
	Provider *v3.TypedExtensionConfig `protobuf:"bytes,2,opt,name=provider,proto3" json:"provider,omitempty"`
	// Configuration for dynamically extracting the client IP address used for geolocation lookups.
	//
	// This field accepts the same :ref:`format specifiers <config_access_log_format>` as used for
	// :ref:`HTTP access logging <config_access_log>` to extract the client IP.
	// The formatted result must be a valid IPv4 or IPv6 address string. For example:
	//
	// * “%FILTER_STATE(my.custom.client.ip:PLAIN)%“ - Read from filter state populated by a preceding filter.
	// * “%DYNAMIC_METADATA(namespace:key)%“ - Read from dynamic metadata.
	// * “%REQ(X-Forwarded-For)%“ - Extract from request header (if applicable in context).
	//
	// If not specified, defaults to the downstream connection's remote address.
	// If specified but the result is empty, “-“, or not a valid IP address, the filter
	// falls back to the downstream connection's remote address.
	//
	// Example reading from filter state:
	//
	// .. code-block:: yaml
	//
	//	client_ip: "%FILTER_STATE(my.custom.client.ip:PLAIN)%"
	ClientIp      string `protobuf:"bytes,3,opt,name=client_ip,json=clientIp,proto3" json:"client_ip,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Geoip) Reset() {
	*x = Geoip{}
	mi := &file_envoy_extensions_filters_network_geoip_v3_geoip_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Geoip) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Geoip) ProtoMessage() {}

func (x *Geoip) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_geoip_v3_geoip_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Geoip.ProtoReflect.Descriptor instead.
func (*Geoip) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescGZIP(), []int{0}
}

func (x *Geoip) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *Geoip) GetProvider() *v3.TypedExtensionConfig {
	if x != nil {
		return x.Provider
	}
	return nil
}

func (x *Geoip) GetClientIp() string {
	if x != nil {
		return x.ClientIp
	}
	return ""
}

var File_envoy_extensions_filters_network_geoip_v3_geoip_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDesc = "" +
	"\n" +
	"5envoy/extensions/filters/network/geoip/v3/geoip.proto\x12)envoy.extensions.filters.network.geoip.v3\x1a$envoy/config/core/v3/extension.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x97\x01\n" +
	"\x05Geoip\x12\x1f\n" +
	"\vstat_prefix\x18\x01 \x01(\tR\n" +
	"statPrefix\x12P\n" +
	"\bprovider\x18\x02 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\bprovider\x12\x1b\n" +
	"\tclient_ip\x18\x03 \x01(\tR\bclientIpB\xb1\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"7io.envoyproxy.envoy.extensions.filters.network.geoip.v3B\n" +
	"GeoipProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/geoip/v3;geoipv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDesc), len(file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDescData
}

var file_envoy_extensions_filters_network_geoip_v3_geoip_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_geoip_v3_geoip_proto_goTypes = []any{
	(*Geoip)(nil),                   // 0: envoy.extensions.filters.network.geoip.v3.Geoip
	(*v3.TypedExtensionConfig)(nil), // 1: envoy.config.core.v3.TypedExtensionConfig
}
var file_envoy_extensions_filters_network_geoip_v3_geoip_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.geoip.v3.Geoip.provider:type_name -> envoy.config.core.v3.TypedExtensionConfig
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_geoip_v3_geoip_proto_init() }
func file_envoy_extensions_filters_network_geoip_v3_geoip_proto_init() {
	if File_envoy_extensions_filters_network_geoip_v3_geoip_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDesc), len(file_envoy_extensions_filters_network_geoip_v3_geoip_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_geoip_v3_geoip_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_geoip_v3_geoip_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_geoip_v3_geoip_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_geoip_v3_geoip_proto = out.File
	file_envoy_extensions_filters_network_geoip_v3_geoip_proto_goTypes = nil
	file_envoy_extensions_filters_network_geoip_v3_geoip_proto_depIdxs = nil
}
