// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha3

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for ServiceEntry
func (this *ServiceEntry) MarshalJSON() ([]byte, error) {
	str, err := ServiceEntryMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServiceEntry
func (this *ServiceEntry) UnmarshalJSON(b []byte) error {
	return ServiceEntryUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ServicePort
func (this *ServicePort) MarshalJSON() ([]byte, error) {
	str, err := ServiceEntryMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServicePort
func (this *ServicePort) UnmarshalJSON(b []byte) error {
	return ServiceEntryUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ServiceEntryStatus
func (this *ServiceEntryStatus) MarshalJSON() ([]byte, error) {
	str, err := ServiceEntryMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServiceEntryStatus
func (this *ServiceEntryStatus) UnmarshalJSON(b []byte) error {
	return ServiceEntryUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for ServiceEntryAddress
func (this *ServiceEntryAddress) MarshalJSON() ([]byte, error) {
	str, err := ServiceEntryMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for ServiceEntryAddress
func (this *ServiceEntryAddress) UnmarshalJSON(b []byte) error {
	return ServiceEntryUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	ServiceEntryMarshaler   = &jsonpb.Marshaler{}
	ServiceEntryUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
