// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/tracers/opentelemetry/samplers/v3/dynatrace_sampler.proto

package samplersv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Dynatrace Sampler extension.
// [#extension: envoy.tracers.opentelemetry.samplers.dynatrace]
type DynatraceSamplerConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The Dynatrace tenant.
	//
	// The value can be obtained from the Envoy deployment page in Dynatrace.
	Tenant string `protobuf:"bytes,1,opt,name=tenant,proto3" json:"tenant,omitempty"`
	// The id of the Dynatrace cluster id.
	//
	// The value can be obtained from the Envoy deployment page in Dynatrace.
	ClusterId int32 `protobuf:"varint,2,opt,name=cluster_id,json=clusterId,proto3" json:"cluster_id,omitempty"`
	// The HTTP service to fetch the sampler configuration from the Dynatrace API (root spans per minute). For example:
	//
	// .. code-block:: yaml
	//
	//	http_service:
	//	  http_uri:
	//	    cluster: dynatrace
	//	    uri: <tenant>.dev.dynatracelabs.com/api/v2/samplingConfiguration
	//	    timeout: 10s
	//	  request_headers_to_add:
	//	  - header:
	//	      key : "authorization"
	//	      value: "Api-Token dt..."
	HttpService *v3.HttpService `protobuf:"bytes,3,opt,name=http_service,json=httpService,proto3" json:"http_service,omitempty"`
	// Default number of root spans per minute, used when the value can't be obtained from the Dynatrace API.
	//
	// A default value of “1000“ is used when:
	//
	// - “root_spans_per_minute“ is unset
	// - “root_spans_per_minute“ is set to 0
	RootSpansPerMinute uint32 `protobuf:"varint,4,opt,name=root_spans_per_minute,json=rootSpansPerMinute,proto3" json:"root_spans_per_minute,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *DynatraceSamplerConfig) Reset() {
	*x = DynatraceSamplerConfig{}
	mi := &file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynatraceSamplerConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynatraceSamplerConfig) ProtoMessage() {}

func (x *DynatraceSamplerConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynatraceSamplerConfig.ProtoReflect.Descriptor instead.
func (*DynatraceSamplerConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescGZIP(), []int{0}
}

func (x *DynatraceSamplerConfig) GetTenant() string {
	if x != nil {
		return x.Tenant
	}
	return ""
}

func (x *DynatraceSamplerConfig) GetClusterId() int32 {
	if x != nil {
		return x.ClusterId
	}
	return 0
}

func (x *DynatraceSamplerConfig) GetHttpService() *v3.HttpService {
	if x != nil {
		return x.HttpService
	}
	return nil
}

func (x *DynatraceSamplerConfig) GetRootSpansPerMinute() uint32 {
	if x != nil {
		return x.RootSpansPerMinute
	}
	return 0
}

var File_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto protoreflect.FileDescriptor

const file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDesc = "" +
	"\n" +
	"Jenvoy/extensions/tracers/opentelemetry/samplers/v3/dynatrace_sampler.proto\x122envoy.extensions.tracers.opentelemetry.samplers.v3\x1a'envoy/config/core/v3/http_service.proto\x1a\x1dudpa/annotations/status.proto\"\xc8\x01\n" +
	"\x16DynatraceSamplerConfig\x12\x16\n" +
	"\x06tenant\x18\x01 \x01(\tR\x06tenant\x12\x1d\n" +
	"\n" +
	"cluster_id\x18\x02 \x01(\x05R\tclusterId\x12D\n" +
	"\fhttp_service\x18\x03 \x01(\v2!.envoy.config.core.v3.HttpServiceR\vhttpService\x121\n" +
	"\x15root_spans_per_minute\x18\x04 \x01(\rR\x12rootSpansPerMinuteB\xc9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.tracers.opentelemetry.samplers.v3B\x15DynatraceSamplerProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/tracers/opentelemetry/samplers/v3;samplersv3b\x06proto3"

var (
	file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescOnce sync.Once
	file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescData []byte
)

func file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescGZIP() []byte {
	file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDesc), len(file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDesc)))
	})
	return file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDescData
}

var file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_goTypes = []any{
	(*DynatraceSamplerConfig)(nil), // 0: envoy.extensions.tracers.opentelemetry.samplers.v3.DynatraceSamplerConfig
	(*v3.HttpService)(nil),         // 1: envoy.config.core.v3.HttpService
}
var file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.tracers.opentelemetry.samplers.v3.DynatraceSamplerConfig.http_service:type_name -> envoy.config.core.v3.HttpService
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_init() }
func file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_init() {
	if File_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDesc), len(file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_msgTypes,
	}.Build()
	File_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto = out.File
	file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_goTypes = nil
	file_envoy_extensions_tracers_opentelemetry_samplers_v3_dynatrace_sampler_proto_depIdxs = nil
}
