// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto

package save_processing_responsev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Extension to save the
// :ref:`ProcessingResponse <envoy_v3_api_msg_service.ext_proc.v3.ProcessingResponse>` from the
// external processor as filter state with name
// “envoy.http.ext_proc.response_processors.save_processing_response“. If
// :ref:`filter_state_name_suffix <envoy_v3_api_field_extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.filter_state_name_suffix>`
// is defined, it is appended to this name.
//
// This extension supports saving of request and response headers, request and response trailers,
// and immediate response.
//
// .. note::
//
//	Response processors are currently in alpha.
//
// [#next-free-field: 7]
type SaveProcessingResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The default filter state name is
	// “envoy.http.ext_proc.response_processors.save_processing_response“.
	// If defined, “filter_state_name_suffix“ is appended to this name.
	//
	// For example, setting “filter_state_name_suffix“ to “xyz“ will set the filter state name
	// to “envoy.http.ext_proc.response_processors.save_processing_response.xyz“.
	FilterStateNameSuffix string `protobuf:"bytes,1,opt,name=filter_state_name_suffix,json=filterStateNameSuffix,proto3" json:"filter_state_name_suffix,omitempty"`
	// Save the response to filter state when
	// :ref:`request_headers <envoy_v3_api_field_service.ext_proc.v3.ProcessingResponse.request_headers>`
	// is set.
	SaveRequestHeaders *SaveProcessingResponse_SaveOptions `protobuf:"bytes,2,opt,name=save_request_headers,json=saveRequestHeaders,proto3" json:"save_request_headers,omitempty"`
	// Save the response to filter state when
	// :ref:`response_headers <envoy_v3_api_field_service.ext_proc.v3.ProcessingResponse.response_headers>`
	// is set.
	SaveResponseHeaders *SaveProcessingResponse_SaveOptions `protobuf:"bytes,3,opt,name=save_response_headers,json=saveResponseHeaders,proto3" json:"save_response_headers,omitempty"`
	// Save the response to filter state when
	// :ref:`request_trailers <envoy_v3_api_field_service.ext_proc.v3.ProcessingResponse.request_trailers>`
	// is set.
	SaveRequestTrailers *SaveProcessingResponse_SaveOptions `protobuf:"bytes,4,opt,name=save_request_trailers,json=saveRequestTrailers,proto3" json:"save_request_trailers,omitempty"`
	// Save the response to filter state when
	// :ref:`response_trailers <envoy_v3_api_field_service.ext_proc.v3.ProcessingResponse.response_trailers>`
	// is set.
	SaveResponseTrailers *SaveProcessingResponse_SaveOptions `protobuf:"bytes,5,opt,name=save_response_trailers,json=saveResponseTrailers,proto3" json:"save_response_trailers,omitempty"`
	// Save the response to filter state when
	// :ref:`immediate_response <envoy_v3_api_field_service.ext_proc.v3.ProcessingResponse.immediate_response>`
	// is set.
	SaveImmediateResponse *SaveProcessingResponse_SaveOptions `protobuf:"bytes,6,opt,name=save_immediate_response,json=saveImmediateResponse,proto3" json:"save_immediate_response,omitempty"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *SaveProcessingResponse) Reset() {
	*x = SaveProcessingResponse{}
	mi := &file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SaveProcessingResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SaveProcessingResponse) ProtoMessage() {}

func (x *SaveProcessingResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SaveProcessingResponse.ProtoReflect.Descriptor instead.
func (*SaveProcessingResponse) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescGZIP(), []int{0}
}

func (x *SaveProcessingResponse) GetFilterStateNameSuffix() string {
	if x != nil {
		return x.FilterStateNameSuffix
	}
	return ""
}

func (x *SaveProcessingResponse) GetSaveRequestHeaders() *SaveProcessingResponse_SaveOptions {
	if x != nil {
		return x.SaveRequestHeaders
	}
	return nil
}

func (x *SaveProcessingResponse) GetSaveResponseHeaders() *SaveProcessingResponse_SaveOptions {
	if x != nil {
		return x.SaveResponseHeaders
	}
	return nil
}

func (x *SaveProcessingResponse) GetSaveRequestTrailers() *SaveProcessingResponse_SaveOptions {
	if x != nil {
		return x.SaveRequestTrailers
	}
	return nil
}

func (x *SaveProcessingResponse) GetSaveResponseTrailers() *SaveProcessingResponse_SaveOptions {
	if x != nil {
		return x.SaveResponseTrailers
	}
	return nil
}

func (x *SaveProcessingResponse) GetSaveImmediateResponse() *SaveProcessingResponse_SaveOptions {
	if x != nil {
		return x.SaveImmediateResponse
	}
	return nil
}

// Options for saving the processing response.
type SaveProcessingResponse_SaveOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// When set to “true“, saves the response for the corresponding response type.
	//
	// Defaults to “false“.
	SaveResponse bool `protobuf:"varint,1,opt,name=save_response,json=saveResponse,proto3" json:"save_response,omitempty"`
	// When set to “true“, saves the response if there was an error when processing the response
	// from the external processor.
	//
	// Defaults to “false“.
	SaveOnError   bool `protobuf:"varint,2,opt,name=save_on_error,json=saveOnError,proto3" json:"save_on_error,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SaveProcessingResponse_SaveOptions) Reset() {
	*x = SaveProcessingResponse_SaveOptions{}
	mi := &file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SaveProcessingResponse_SaveOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SaveProcessingResponse_SaveOptions) ProtoMessage() {}

func (x *SaveProcessingResponse_SaveOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SaveProcessingResponse_SaveOptions.ProtoReflect.Descriptor instead.
func (*SaveProcessingResponse_SaveOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescGZIP(), []int{0, 0}
}

func (x *SaveProcessingResponse_SaveOptions) GetSaveResponse() bool {
	if x != nil {
		return x.SaveResponse
	}
	return false
}

func (x *SaveProcessingResponse_SaveOptions) GetSaveOnError() bool {
	if x != nil {
		return x.SaveOnError
	}
	return false
}

var File_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto protoreflect.FileDescriptor

const file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDesc = "" +
	"\n" +
	"menvoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto\x12Nenvoy.extensions.http.ext_proc.response_processors.save_processing_response.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\xfa\a\n" +
	"\x16SaveProcessingResponse\x127\n" +
	"\x18filter_state_name_suffix\x18\x01 \x01(\tR\x15filterStateNameSuffix\x12\xa4\x01\n" +
	"\x14save_request_headers\x18\x02 \x01(\v2r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptionsR\x12saveRequestHeaders\x12\xa6\x01\n" +
	"\x15save_response_headers\x18\x03 \x01(\v2r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptionsR\x13saveResponseHeaders\x12\xa6\x01\n" +
	"\x15save_request_trailers\x18\x04 \x01(\v2r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptionsR\x13saveRequestTrailers\x12\xa8\x01\n" +
	"\x16save_response_trailers\x18\x05 \x01(\v2r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptionsR\x14saveResponseTrailers\x12\xaa\x01\n" +
	"\x17save_immediate_response\x18\x06 \x01(\v2r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptionsR\x15saveImmediateResponse\x1aV\n" +
	"\vSaveOptions\x12#\n" +
	"\rsave_response\x18\x01 \x01(\bR\fsaveResponse\x12\"\n" +
	"\rsave_on_error\x18\x02 \x01(\bR\vsaveOnErrorB\xa0\x02\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"\\io.envoyproxy.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3B\x1bSaveProcessingResponseProtoP\x01Z\x90\x01github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3;save_processing_responsev3b\x06proto3"

var (
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescOnce sync.Once
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescData []byte
)

func file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescGZIP() []byte {
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDesc), len(file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDesc)))
	})
	return file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDescData
}

var file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_goTypes = []any{
	(*SaveProcessingResponse)(nil),             // 0: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse
	(*SaveProcessingResponse_SaveOptions)(nil), // 1: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
}
var file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.save_request_headers:type_name -> envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
	1, // 1: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.save_response_headers:type_name -> envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
	1, // 2: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.save_request_trailers:type_name -> envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
	1, // 3: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.save_response_trailers:type_name -> envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
	1, // 4: envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.save_immediate_response:type_name -> envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_init()
}
func file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_init() {
	if File_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDesc), len(file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_msgTypes,
	}.Build()
	File_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto = out.File
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_goTypes = nil
	file_envoy_extensions_http_ext_proc_response_processors_save_processing_response_v3_save_processing_response_proto_depIdxs = nil
}
