// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/access_loggers/grpc/v3/als.proto

package grpcv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/type/tracing/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the built-in “envoy.access_loggers.http_grpc“
// :ref:`AccessLog <envoy_v3_api_msg_config.accesslog.v3.AccessLog>`. This configuration will
// populate :ref:`StreamAccessLogsMessage.http_logs
// <envoy_v3_api_field_service.accesslog.v3.StreamAccessLogsMessage.http_logs>`.
// [#extension: envoy.access_loggers.http_grpc]
type HttpGrpcAccessLogConfig struct {
	state        protoimpl.MessageState     `protogen:"open.v1"`
	CommonConfig *CommonGrpcAccessLogConfig `protobuf:"bytes,1,opt,name=common_config,json=commonConfig,proto3" json:"common_config,omitempty"`
	// Additional request headers to log in :ref:`HTTPRequestProperties.request_headers
	// <envoy_v3_api_field_data.accesslog.v3.HTTPRequestProperties.request_headers>`.
	AdditionalRequestHeadersToLog []string `protobuf:"bytes,2,rep,name=additional_request_headers_to_log,json=additionalRequestHeadersToLog,proto3" json:"additional_request_headers_to_log,omitempty"`
	// Additional response headers to log in :ref:`HTTPResponseProperties.response_headers
	// <envoy_v3_api_field_data.accesslog.v3.HTTPResponseProperties.response_headers>`.
	AdditionalResponseHeadersToLog []string `protobuf:"bytes,3,rep,name=additional_response_headers_to_log,json=additionalResponseHeadersToLog,proto3" json:"additional_response_headers_to_log,omitempty"`
	// Additional response trailers to log in :ref:`HTTPResponseProperties.response_trailers
	// <envoy_v3_api_field_data.accesslog.v3.HTTPResponseProperties.response_trailers>`.
	AdditionalResponseTrailersToLog []string `protobuf:"bytes,4,rep,name=additional_response_trailers_to_log,json=additionalResponseTrailersToLog,proto3" json:"additional_response_trailers_to_log,omitempty"`
	unknownFields                   protoimpl.UnknownFields
	sizeCache                       protoimpl.SizeCache
}

func (x *HttpGrpcAccessLogConfig) Reset() {
	*x = HttpGrpcAccessLogConfig{}
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpGrpcAccessLogConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpGrpcAccessLogConfig) ProtoMessage() {}

func (x *HttpGrpcAccessLogConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpGrpcAccessLogConfig.ProtoReflect.Descriptor instead.
func (*HttpGrpcAccessLogConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescGZIP(), []int{0}
}

func (x *HttpGrpcAccessLogConfig) GetCommonConfig() *CommonGrpcAccessLogConfig {
	if x != nil {
		return x.CommonConfig
	}
	return nil
}

func (x *HttpGrpcAccessLogConfig) GetAdditionalRequestHeadersToLog() []string {
	if x != nil {
		return x.AdditionalRequestHeadersToLog
	}
	return nil
}

func (x *HttpGrpcAccessLogConfig) GetAdditionalResponseHeadersToLog() []string {
	if x != nil {
		return x.AdditionalResponseHeadersToLog
	}
	return nil
}

func (x *HttpGrpcAccessLogConfig) GetAdditionalResponseTrailersToLog() []string {
	if x != nil {
		return x.AdditionalResponseTrailersToLog
	}
	return nil
}

// Configuration for the built-in “envoy.access_loggers.tcp_grpc“ type. This configuration will
// populate “StreamAccessLogsMessage.tcp_logs“.
// [#extension: envoy.access_loggers.tcp_grpc]
type TcpGrpcAccessLogConfig struct {
	state         protoimpl.MessageState     `protogen:"open.v1"`
	CommonConfig  *CommonGrpcAccessLogConfig `protobuf:"bytes,1,opt,name=common_config,json=commonConfig,proto3" json:"common_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TcpGrpcAccessLogConfig) Reset() {
	*x = TcpGrpcAccessLogConfig{}
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TcpGrpcAccessLogConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TcpGrpcAccessLogConfig) ProtoMessage() {}

func (x *TcpGrpcAccessLogConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TcpGrpcAccessLogConfig.ProtoReflect.Descriptor instead.
func (*TcpGrpcAccessLogConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescGZIP(), []int{1}
}

func (x *TcpGrpcAccessLogConfig) GetCommonConfig() *CommonGrpcAccessLogConfig {
	if x != nil {
		return x.CommonConfig
	}
	return nil
}

// Common configuration for gRPC access logs.
// [#next-free-field: 9]
type CommonGrpcAccessLogConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The friendly name of the access log to be returned in :ref:`StreamAccessLogsMessage.Identifier
	// <envoy_v3_api_msg_service.accesslog.v3.StreamAccessLogsMessage.Identifier>`. This allows the
	// access log server to differentiate between different access logs coming from the same Envoy.
	LogName string `protobuf:"bytes,1,opt,name=log_name,json=logName,proto3" json:"log_name,omitempty"`
	// The gRPC service for the access log service.
	GrpcService *v3.GrpcService `protobuf:"bytes,2,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	// API version for access logs service transport protocol. This describes the access logs service
	// gRPC endpoint and version of messages used on the wire.
	TransportApiVersion v3.ApiVersion `protobuf:"varint,6,opt,name=transport_api_version,json=transportApiVersion,proto3,enum=envoy.config.core.v3.ApiVersion" json:"transport_api_version,omitempty"`
	// Interval for flushing access logs to the gRPC stream. Logger will flush requests every time
	// this interval is elapsed, or when batch size limit is hit, whichever comes first. Defaults to
	// 1 second.
	BufferFlushInterval *durationpb.Duration `protobuf:"bytes,3,opt,name=buffer_flush_interval,json=bufferFlushInterval,proto3" json:"buffer_flush_interval,omitempty"`
	// Soft size limit in bytes for access log entries buffer. Logger will buffer requests until
	// this limit it hit, or every time flush interval is elapsed, whichever comes first. Setting it
	// to zero effectively disables the batching. Defaults to 16384.
	BufferSizeBytes *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=buffer_size_bytes,json=bufferSizeBytes,proto3" json:"buffer_size_bytes,omitempty"`
	// Additional filter state objects to log in :ref:`filter_state_objects
	// <envoy_v3_api_field_data.accesslog.v3.AccessLogCommon.filter_state_objects>`.
	// Logger will call “FilterState::Object::serializeAsProto“ to serialize the filter state object.
	FilterStateObjectsToLog []string `protobuf:"bytes,5,rep,name=filter_state_objects_to_log,json=filterStateObjectsToLog,proto3" json:"filter_state_objects_to_log,omitempty"`
	// Sets the retry policy when the establishment of a gRPC stream fails.
	// If the stream succeeds at least once in establishing itself,
	// no retry will be performed no matter what gRPC status is received.
	// Note that only :ref:`num_retries <envoy_v3_api_field_config.core.v3.RetryPolicy.num_retries>`
	// will be used in this configuration. This feature is used only when you are using
	// :ref:`Envoy gRPC client <envoy_v3_api_field_config.core.v3.GrpcService.envoy_grpc>`.
	GrpcStreamRetryPolicy *v3.RetryPolicy `protobuf:"bytes,7,opt,name=grpc_stream_retry_policy,json=grpcStreamRetryPolicy,proto3" json:"grpc_stream_retry_policy,omitempty"`
	// A list of custom tags with unique tag name to create tags for the logs.
	CustomTags    []*v31.CustomTag `protobuf:"bytes,8,rep,name=custom_tags,json=customTags,proto3" json:"custom_tags,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CommonGrpcAccessLogConfig) Reset() {
	*x = CommonGrpcAccessLogConfig{}
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CommonGrpcAccessLogConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CommonGrpcAccessLogConfig) ProtoMessage() {}

func (x *CommonGrpcAccessLogConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CommonGrpcAccessLogConfig.ProtoReflect.Descriptor instead.
func (*CommonGrpcAccessLogConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescGZIP(), []int{2}
}

func (x *CommonGrpcAccessLogConfig) GetLogName() string {
	if x != nil {
		return x.LogName
	}
	return ""
}

func (x *CommonGrpcAccessLogConfig) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

func (x *CommonGrpcAccessLogConfig) GetTransportApiVersion() v3.ApiVersion {
	if x != nil {
		return x.TransportApiVersion
	}
	return v3.ApiVersion(0)
}

func (x *CommonGrpcAccessLogConfig) GetBufferFlushInterval() *durationpb.Duration {
	if x != nil {
		return x.BufferFlushInterval
	}
	return nil
}

func (x *CommonGrpcAccessLogConfig) GetBufferSizeBytes() *wrapperspb.UInt32Value {
	if x != nil {
		return x.BufferSizeBytes
	}
	return nil
}

func (x *CommonGrpcAccessLogConfig) GetFilterStateObjectsToLog() []string {
	if x != nil {
		return x.FilterStateObjectsToLog
	}
	return nil
}

func (x *CommonGrpcAccessLogConfig) GetGrpcStreamRetryPolicy() *v3.RetryPolicy {
	if x != nil {
		return x.GrpcStreamRetryPolicy
	}
	return nil
}

func (x *CommonGrpcAccessLogConfig) GetCustomTags() []*v31.CustomTag {
	if x != nil {
		return x.CustomTags
	}
	return nil
}

var File_envoy_extensions_access_loggers_grpc_v3_als_proto protoreflect.FileDescriptor

const file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDesc = "" +
	"\n" +
	"1envoy/extensions/access_loggers/grpc/v3/als.proto\x12'envoy.extensions.access_loggers.grpc.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a(envoy/config/core/v3/config_source.proto\x1a'envoy/config/core/v3/grpc_service.proto\x1a&envoy/type/tracing/v3/custom_tag.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xaa\x03\n" +
	"\x17HttpGrpcAccessLogConfig\x12q\n" +
	"\rcommon_config\x18\x01 \x01(\v2B.envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fcommonConfig\x12H\n" +
	"!additional_request_headers_to_log\x18\x02 \x03(\tR\x1dadditionalRequestHeadersToLog\x12J\n" +
	"\"additional_response_headers_to_log\x18\x03 \x03(\tR\x1eadditionalResponseHeadersToLog\x12L\n" +
	"#additional_response_trailers_to_log\x18\x04 \x03(\tR\x1fadditionalResponseTrailersToLog:8\x9aň\x1e3\n" +
	"1envoy.config.accesslog.v2.HttpGrpcAccessLogConfig\"\xc4\x01\n" +
	"\x16TcpGrpcAccessLogConfig\x12q\n" +
	"\rcommon_config\x18\x01 \x01(\v2B.envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fcommonConfig:7\x9aň\x1e2\n" +
	"0envoy.config.accesslog.v2.TcpGrpcAccessLogConfig\"\xab\x05\n" +
	"\x19CommonGrpcAccessLogConfig\x12\"\n" +
	"\blog_name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\alogName\x12N\n" +
	"\fgrpc_service\x18\x02 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vgrpcService\x12^\n" +
	"\x15transport_api_version\x18\x06 \x01(\x0e2 .envoy.config.core.v3.ApiVersionB\b\xfaB\x05\x82\x01\x02\x10\x01R\x13transportApiVersion\x12W\n" +
	"\x15buffer_flush_interval\x18\x03 \x01(\v2\x19.google.protobuf.DurationB\b\xfaB\x05\xaa\x01\x02*\x00R\x13bufferFlushInterval\x12H\n" +
	"\x11buffer_size_bytes\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x0fbufferSizeBytes\x12<\n" +
	"\x1bfilter_state_objects_to_log\x18\x05 \x03(\tR\x17filterStateObjectsToLog\x12Z\n" +
	"\x18grpc_stream_retry_policy\x18\a \x01(\v2!.envoy.config.core.v3.RetryPolicyR\x15grpcStreamRetryPolicy\x12A\n" +
	"\vcustom_tags\x18\b \x03(\v2 .envoy.type.tracing.v3.CustomTagR\n" +
	"customTags::\x9aň\x1e5\n" +
	"3envoy.config.accesslog.v2.CommonGrpcAccessLogConfigB\xa2\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"5io.envoyproxy.envoy.extensions.access_loggers.grpc.v3B\bAlsProtoP\x01ZUgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/grpc/v3;grpcv3b\x06proto3"

var (
	file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescOnce sync.Once
	file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescData []byte
)

func file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescGZIP() []byte {
	file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDesc), len(file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDesc)))
	})
	return file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDescData
}

var file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_extensions_access_loggers_grpc_v3_als_proto_goTypes = []any{
	(*HttpGrpcAccessLogConfig)(nil),   // 0: envoy.extensions.access_loggers.grpc.v3.HttpGrpcAccessLogConfig
	(*TcpGrpcAccessLogConfig)(nil),    // 1: envoy.extensions.access_loggers.grpc.v3.TcpGrpcAccessLogConfig
	(*CommonGrpcAccessLogConfig)(nil), // 2: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig
	(*v3.GrpcService)(nil),            // 3: envoy.config.core.v3.GrpcService
	(v3.ApiVersion)(0),                // 4: envoy.config.core.v3.ApiVersion
	(*durationpb.Duration)(nil),       // 5: google.protobuf.Duration
	(*wrapperspb.UInt32Value)(nil),    // 6: google.protobuf.UInt32Value
	(*v3.RetryPolicy)(nil),            // 7: envoy.config.core.v3.RetryPolicy
	(*v31.CustomTag)(nil),             // 8: envoy.type.tracing.v3.CustomTag
}
var file_envoy_extensions_access_loggers_grpc_v3_als_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.access_loggers.grpc.v3.HttpGrpcAccessLogConfig.common_config:type_name -> envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig
	2, // 1: envoy.extensions.access_loggers.grpc.v3.TcpGrpcAccessLogConfig.common_config:type_name -> envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig
	3, // 2: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	4, // 3: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.transport_api_version:type_name -> envoy.config.core.v3.ApiVersion
	5, // 4: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.buffer_flush_interval:type_name -> google.protobuf.Duration
	6, // 5: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.buffer_size_bytes:type_name -> google.protobuf.UInt32Value
	7, // 6: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.grpc_stream_retry_policy:type_name -> envoy.config.core.v3.RetryPolicy
	8, // 7: envoy.extensions.access_loggers.grpc.v3.CommonGrpcAccessLogConfig.custom_tags:type_name -> envoy.type.tracing.v3.CustomTag
	8, // [8:8] is the sub-list for method output_type
	8, // [8:8] is the sub-list for method input_type
	8, // [8:8] is the sub-list for extension type_name
	8, // [8:8] is the sub-list for extension extendee
	0, // [0:8] is the sub-list for field type_name
}

func init() { file_envoy_extensions_access_loggers_grpc_v3_als_proto_init() }
func file_envoy_extensions_access_loggers_grpc_v3_als_proto_init() {
	if File_envoy_extensions_access_loggers_grpc_v3_als_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDesc), len(file_envoy_extensions_access_loggers_grpc_v3_als_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_access_loggers_grpc_v3_als_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_access_loggers_grpc_v3_als_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_access_loggers_grpc_v3_als_proto_msgTypes,
	}.Build()
	File_envoy_extensions_access_loggers_grpc_v3_als_proto = out.File
	file_envoy_extensions_access_loggers_grpc_v3_als_proto_goTypes = nil
	file_envoy_extensions_access_loggers_grpc_v3_als_proto_depIdxs = nil
}
