// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/ext_proc/v3/ext_proc.proto

package ext_procv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Defines how traffic should be handled by the external processor.
type ProcessingMode_DataSendMode int32

const (
	// Send the data to the external processor for processing whenever the data is ready.
	ProcessingMode_STREAMED ProcessingMode_DataSendMode = 0
	// Skip sending the data to the external processor.
	ProcessingMode_SKIP ProcessingMode_DataSendMode = 1
)

// Enum value maps for ProcessingMode_DataSendMode.
var (
	ProcessingMode_DataSendMode_name = map[int32]string{
		0: "STREAMED",
		1: "SKIP",
	}
	ProcessingMode_DataSendMode_value = map[string]int32{
		"STREAMED": 0,
		"SKIP":     1,
	}
)

func (x ProcessingMode_DataSendMode) Enum() *ProcessingMode_DataSendMode {
	p := new(ProcessingMode_DataSendMode)
	*p = x
	return p
}

func (x ProcessingMode_DataSendMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ProcessingMode_DataSendMode) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_enumTypes[0].Descriptor()
}

func (ProcessingMode_DataSendMode) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_enumTypes[0]
}

func (x ProcessingMode_DataSendMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ProcessingMode_DataSendMode.Descriptor instead.
func (ProcessingMode_DataSendMode) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP(), []int{1, 0}
}

// The Network External Processing filter allows an external service to process raw TCP/UDP traffic
// in a flexible way using a bidirectional gRPC stream. Unlike the HTTP External Processing filter,
// this filter operates at the L4 (transport) layer, giving access to raw network traffic.
//
// The filter communicates with an external gRPC service that can:
//
// 1. Inspect traffic in both directions
// 2. Modify the network traffic
// 3. Control connection lifecycle (continue, close, or reset)
//
// By using the filter's processing mode, you can selectively choose which data
// directions to process (read, write or both), allowing for efficient processing.
// [#next-free-field: 7]
type NetworkExternalProcessor struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The gRPC service that will process network traffic.
	// This service must implement the NetworkExternalProcessor service
	// defined in the proto file /envoy/service/network_ext_proc/v3/external_processor.proto.
	GrpcService *v3.GrpcService `protobuf:"bytes,1,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	// By default, if the gRPC stream cannot be established, or if it is closed
	// prematurely with an error, the filter will fail, leading to the close of connection.
	// With this parameter set to true, however, then if the gRPC stream is prematurely closed
	// or could not be opened, processing continues without error.
	FailureModeAllow bool `protobuf:"varint,2,opt,name=failure_mode_allow,json=failureModeAllow,proto3" json:"failure_mode_allow,omitempty"`
	// Options for controlling processing behavior.
	ProcessingMode *ProcessingMode `protobuf:"bytes,3,opt,name=processing_mode,json=processingMode,proto3" json:"processing_mode,omitempty"`
	// Specifies the timeout for each individual message sent on the stream and
	// when the filter is running in synchronous mode. Whenever
	// the proxy sends a message on the stream that requires a response, it will
	// reset this timer, and will stop processing and return an error (subject
	// to the processing mode) if the timer expires. Default is 200 ms.
	MessageTimeout *durationpb.Duration `protobuf:"bytes,4,opt,name=message_timeout,json=messageTimeout,proto3" json:"message_timeout,omitempty"`
	StatPrefix     string               `protobuf:"bytes,5,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// Options related to the sending and receiving of dynamic metadata.
	MetadataOptions *MetadataOptions `protobuf:"bytes,6,opt,name=metadata_options,json=metadataOptions,proto3" json:"metadata_options,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *NetworkExternalProcessor) Reset() {
	*x = NetworkExternalProcessor{}
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkExternalProcessor) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkExternalProcessor) ProtoMessage() {}

func (x *NetworkExternalProcessor) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkExternalProcessor.ProtoReflect.Descriptor instead.
func (*NetworkExternalProcessor) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP(), []int{0}
}

func (x *NetworkExternalProcessor) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

func (x *NetworkExternalProcessor) GetFailureModeAllow() bool {
	if x != nil {
		return x.FailureModeAllow
	}
	return false
}

func (x *NetworkExternalProcessor) GetProcessingMode() *ProcessingMode {
	if x != nil {
		return x.ProcessingMode
	}
	return nil
}

func (x *NetworkExternalProcessor) GetMessageTimeout() *durationpb.Duration {
	if x != nil {
		return x.MessageTimeout
	}
	return nil
}

func (x *NetworkExternalProcessor) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *NetworkExternalProcessor) GetMetadataOptions() *MetadataOptions {
	if x != nil {
		return x.MetadataOptions
	}
	return nil
}

// Options for controlling processing behavior.
// Filter will reject the config if both read and write are SKIP mode.
type ProcessingMode struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Controls whether inbound (read) data from the client is sent to the external processor.
	// Default: STREAMED
	ProcessRead ProcessingMode_DataSendMode `protobuf:"varint,1,opt,name=process_read,json=processRead,proto3,enum=envoy.extensions.filters.network.ext_proc.v3.ProcessingMode_DataSendMode" json:"process_read,omitempty"`
	// Controls whether outbound (write) data to the client is sent to the external processor.
	// Default: STREAMED
	ProcessWrite  ProcessingMode_DataSendMode `protobuf:"varint,2,opt,name=process_write,json=processWrite,proto3,enum=envoy.extensions.filters.network.ext_proc.v3.ProcessingMode_DataSendMode" json:"process_write,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ProcessingMode) Reset() {
	*x = ProcessingMode{}
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProcessingMode) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProcessingMode) ProtoMessage() {}

func (x *ProcessingMode) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProcessingMode.ProtoReflect.Descriptor instead.
func (*ProcessingMode) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP(), []int{1}
}

func (x *ProcessingMode) GetProcessRead() ProcessingMode_DataSendMode {
	if x != nil {
		return x.ProcessRead
	}
	return ProcessingMode_STREAMED
}

func (x *ProcessingMode) GetProcessWrite() ProcessingMode_DataSendMode {
	if x != nil {
		return x.ProcessWrite
	}
	return ProcessingMode_STREAMED
}

// The MetadataOptions structure defines options for sending dynamic metadata. Specifically,
// which namespaces to send to the server.
type MetadataOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Describes which typed or untyped dynamic metadata namespaces to forward to
	// the external processing server.
	ForwardingNamespaces *MetadataOptions_MetadataNamespaces `protobuf:"bytes,1,opt,name=forwarding_namespaces,json=forwardingNamespaces,proto3" json:"forwarding_namespaces,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *MetadataOptions) Reset() {
	*x = MetadataOptions{}
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetadataOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetadataOptions) ProtoMessage() {}

func (x *MetadataOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetadataOptions.ProtoReflect.Descriptor instead.
func (*MetadataOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP(), []int{2}
}

func (x *MetadataOptions) GetForwardingNamespaces() *MetadataOptions_MetadataNamespaces {
	if x != nil {
		return x.ForwardingNamespaces
	}
	return nil
}

type MetadataOptions_MetadataNamespaces struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies a list of metadata namespaces whose values, if present,
	// will be passed to the ext_proc service as an opaque *protobuf::Struct*.
	Untyped []string `protobuf:"bytes,1,rep,name=untyped,proto3" json:"untyped,omitempty"`
	// Specifies a list of metadata namespaces whose values, if present,
	// will be passed to the ext_proc service as a *protobuf::Any*. This allows
	// envoy and the external processing server to share the protobuf message
	// definition for safe parsing.
	Typed         []string `protobuf:"bytes,2,rep,name=typed,proto3" json:"typed,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetadataOptions_MetadataNamespaces) Reset() {
	*x = MetadataOptions_MetadataNamespaces{}
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetadataOptions_MetadataNamespaces) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetadataOptions_MetadataNamespaces) ProtoMessage() {}

func (x *MetadataOptions_MetadataNamespaces) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetadataOptions_MetadataNamespaces.ProtoReflect.Descriptor instead.
func (*MetadataOptions_MetadataNamespaces) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP(), []int{2, 0}
}

func (x *MetadataOptions_MetadataNamespaces) GetUntyped() []string {
	if x != nil {
		return x.Untyped
	}
	return nil
}

func (x *MetadataOptions_MetadataNamespaces) GetTyped() []string {
	if x != nil {
		return x.Typed
	}
	return nil
}

var File_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDesc = "" +
	"\n" +
	";envoy/extensions/filters/network/ext_proc/v3/ext_proc.proto\x12,envoy.extensions.filters.network.ext_proc.v3\x1a'envoy/config/core/v3/grpc_service.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xdc\x03\n" +
	"\x18NetworkExternalProcessor\x12D\n" +
	"\fgrpc_service\x18\x01 \x01(\v2!.envoy.config.core.v3.GrpcServiceR\vgrpcService\x12,\n" +
	"\x12failure_mode_allow\x18\x02 \x01(\bR\x10failureModeAllow\x12e\n" +
	"\x0fprocessing_mode\x18\x03 \x01(\v2<.envoy.extensions.filters.network.ext_proc.v3.ProcessingModeR\x0eprocessingMode\x12Q\n" +
	"\x0fmessage_timeout\x18\x04 \x01(\v2\x19.google.protobuf.DurationB\r\xfaB\n" +
	"\xaa\x01\a\"\x03\b\x90\x1c2\x00R\x0emessageTimeout\x12(\n" +
	"\vstat_prefix\x18\x05 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12h\n" +
	"\x10metadata_options\x18\x06 \x01(\v2=.envoy.extensions.filters.network.ext_proc.v3.MetadataOptionsR\x0fmetadataOptions\"\x96\x02\n" +
	"\x0eProcessingMode\x12l\n" +
	"\fprocess_read\x18\x01 \x01(\x0e2I.envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.DataSendModeR\vprocessRead\x12n\n" +
	"\rprocess_write\x18\x02 \x01(\x0e2I.envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.DataSendModeR\fprocessWrite\"&\n" +
	"\fDataSendMode\x12\f\n" +
	"\bSTREAMED\x10\x00\x12\b\n" +
	"\x04SKIP\x10\x01\"\xdf\x01\n" +
	"\x0fMetadataOptions\x12\x85\x01\n" +
	"\x15forwarding_namespaces\x18\x01 \x01(\v2P.envoy.extensions.filters.network.ext_proc.v3.MetadataOptions.MetadataNamespacesR\x14forwardingNamespaces\x1aD\n" +
	"\x12MetadataNamespaces\x12\x18\n" +
	"\auntyped\x18\x01 \x03(\tR\auntyped\x12\x14\n" +
	"\x05typed\x18\x02 \x03(\tR\x05typedB\xbc\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	":io.envoyproxy.envoy.extensions.filters.network.ext_proc.v3B\fExtProcProtoP\x01Z^github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/ext_proc/v3;ext_procv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDesc), len(file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDescData
}

var file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_goTypes = []any{
	(ProcessingMode_DataSendMode)(0),           // 0: envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.DataSendMode
	(*NetworkExternalProcessor)(nil),           // 1: envoy.extensions.filters.network.ext_proc.v3.NetworkExternalProcessor
	(*ProcessingMode)(nil),                     // 2: envoy.extensions.filters.network.ext_proc.v3.ProcessingMode
	(*MetadataOptions)(nil),                    // 3: envoy.extensions.filters.network.ext_proc.v3.MetadataOptions
	(*MetadataOptions_MetadataNamespaces)(nil), // 4: envoy.extensions.filters.network.ext_proc.v3.MetadataOptions.MetadataNamespaces
	(*v3.GrpcService)(nil),                     // 5: envoy.config.core.v3.GrpcService
	(*durationpb.Duration)(nil),                // 6: google.protobuf.Duration
}
var file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_depIdxs = []int32{
	5, // 0: envoy.extensions.filters.network.ext_proc.v3.NetworkExternalProcessor.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	2, // 1: envoy.extensions.filters.network.ext_proc.v3.NetworkExternalProcessor.processing_mode:type_name -> envoy.extensions.filters.network.ext_proc.v3.ProcessingMode
	6, // 2: envoy.extensions.filters.network.ext_proc.v3.NetworkExternalProcessor.message_timeout:type_name -> google.protobuf.Duration
	3, // 3: envoy.extensions.filters.network.ext_proc.v3.NetworkExternalProcessor.metadata_options:type_name -> envoy.extensions.filters.network.ext_proc.v3.MetadataOptions
	0, // 4: envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.process_read:type_name -> envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.DataSendMode
	0, // 5: envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.process_write:type_name -> envoy.extensions.filters.network.ext_proc.v3.ProcessingMode.DataSendMode
	4, // 6: envoy.extensions.filters.network.ext_proc.v3.MetadataOptions.forwarding_namespaces:type_name -> envoy.extensions.filters.network.ext_proc.v3.MetadataOptions.MetadataNamespaces
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_init() }
func file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_init() {
	if File_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDesc), len(file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto = out.File
	file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_goTypes = nil
	file_envoy_extensions_filters_network_ext_proc_v3_ext_proc_proto_depIdxs = nil
}
