/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// HTTPRouteRetryApplyConfiguration represents a declarative configuration of the HTTPRouteRetry type for use
// with apply.
//
// HTTPRouteRetry defines retry configuration for an HTTPRoute.
//
// Implementations SHOULD retry on connection errors (disconnect, reset, timeout,
// TCP failure) if a retry stanza is configured.
type HTTPRouteRetryApplyConfiguration struct {
	// Codes defines the HTTP response status codes for which a backend request
	// should be retried.
	//
	// Support: Extended
	Codes []apisv1.HTTPRouteRetryStatusCode `json:"codes,omitempty"`
	// Attempts specifies the maximum number of times an individual request
	// from the gateway to a backend should be retried.
	//
	// If the maximum number of retries has been attempted without a successful
	// response from the backend, the Gateway MUST return an error.
	//
	// When this field is unspecified, the number of times to attempt to retry
	// a backend request is implementation-specific.
	//
	// Support: Extended
	Attempts *int `json:"attempts,omitempty"`
	// Backoff specifies the minimum duration a Gateway should wait between
	// retry attempts and is represented in Gateway API Duration formatting.
	//
	// For example, setting the `rules[].retry.backoff` field to the value
	// `100ms` will cause a backend request to first be retried approximately
	// 100 milliseconds after timing out or receiving a response code configured
	// to be retriable.
	//
	// An implementation MAY use an exponential or alternative backoff strategy
	// for subsequent retry attempts, MAY cap the maximum backoff duration to
	// some amount greater than the specified minimum, and MAY add arbitrary
	// jitter to stagger requests, as long as unsuccessful backend requests are
	// not retried before the configured minimum duration.
	//
	// If a Request timeout (`rules[].timeouts.request`) is configured on the
	// route, the entire duration of the initial request and any retry attempts
	// MUST not exceed the Request timeout duration. If any retry attempts are
	// still in progress when the Request timeout duration has been reached,
	// these SHOULD be canceled if possible and the Gateway MUST immediately
	// return a timeout error.
	//
	// If a BackendRequest timeout (`rules[].timeouts.backendRequest`) is
	// configured on the route, any retry attempts which reach the configured
	// BackendRequest timeout duration without a response SHOULD be canceled if
	// possible and the Gateway should wait for at least the specified backoff
	// duration before attempting to retry the backend request again.
	//
	// If a BackendRequest timeout is _not_ configured on the route, retry
	// attempts MAY time out after an implementation default duration, or MAY
	// remain pending until a configured Request timeout or implementation
	// default duration for total request time is reached.
	//
	// When this field is unspecified, the time to wait between retry attempts
	// is implementation-specific.
	//
	// Support: Extended
	Backoff *apisv1.Duration `json:"backoff,omitempty"`
}

// HTTPRouteRetryApplyConfiguration constructs a declarative configuration of the HTTPRouteRetry type for use with
// apply.
func HTTPRouteRetry() *HTTPRouteRetryApplyConfiguration {
	return &HTTPRouteRetryApplyConfiguration{}
}

// WithCodes adds the given value to the Codes field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Codes field.
func (b *HTTPRouteRetryApplyConfiguration) WithCodes(values ...apisv1.HTTPRouteRetryStatusCode) *HTTPRouteRetryApplyConfiguration {
	for i := range values {
		b.Codes = append(b.Codes, values[i])
	}
	return b
}

// WithAttempts sets the Attempts field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Attempts field is set to the value of the last call.
func (b *HTTPRouteRetryApplyConfiguration) WithAttempts(value int) *HTTPRouteRetryApplyConfiguration {
	b.Attempts = &value
	return b
}

// WithBackoff sets the Backoff field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Backoff field is set to the value of the last call.
func (b *HTTPRouteRetryApplyConfiguration) WithBackoff(value apisv1.Duration) *HTTPRouteRetryApplyConfiguration {
	b.Backoff = &value
	return b
}
