// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/tap/v3/tap.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/common/tap/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for tap transport socket. This wraps another transport socket, providing the
// ability to interpose and record in plain text any traffic that is surfaced to Envoy.
type Tap struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Common configuration for the tap transport socket.
	CommonConfig *v3.CommonExtensionConfig `protobuf:"bytes,1,opt,name=common_config,json=commonConfig,proto3" json:"common_config,omitempty"`
	// The underlying transport socket being wrapped.
	TransportSocket *v31.TransportSocket `protobuf:"bytes,2,opt,name=transport_socket,json=transportSocket,proto3" json:"transport_socket,omitempty"`
	// Additional configurations for the transport socket tap
	SocketTapConfig *SocketTapConfig `protobuf:"bytes,3,opt,name=socket_tap_config,json=socketTapConfig,proto3" json:"socket_tap_config,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Tap) Reset() {
	*x = Tap{}
	mi := &file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Tap) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Tap) ProtoMessage() {}

func (x *Tap) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Tap.ProtoReflect.Descriptor instead.
func (*Tap) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescGZIP(), []int{0}
}

func (x *Tap) GetCommonConfig() *v3.CommonExtensionConfig {
	if x != nil {
		return x.CommonConfig
	}
	return nil
}

func (x *Tap) GetTransportSocket() *v31.TransportSocket {
	if x != nil {
		return x.TransportSocket
	}
	return nil
}

func (x *Tap) GetSocketTapConfig() *SocketTapConfig {
	if x != nil {
		return x.SocketTapConfig
	}
	return nil
}

// Additional configurations for the transport socket tap
type SocketTapConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Indicates to whether output the connection information per event
	// This is only applicable if the streamed trace is enabled
	SetConnectionPerEvent bool `protobuf:"varint,1,opt,name=set_connection_per_event,json=setConnectionPerEvent,proto3" json:"set_connection_per_event,omitempty"`
	// The contents of the transport tap's statistics prefix.
	StatsPrefix   string `protobuf:"bytes,2,opt,name=stats_prefix,json=statsPrefix,proto3" json:"stats_prefix,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SocketTapConfig) Reset() {
	*x = SocketTapConfig{}
	mi := &file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SocketTapConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SocketTapConfig) ProtoMessage() {}

func (x *SocketTapConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SocketTapConfig.ProtoReflect.Descriptor instead.
func (*SocketTapConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescGZIP(), []int{1}
}

func (x *SocketTapConfig) GetSetConnectionPerEvent() bool {
	if x != nil {
		return x.SetConnectionPerEvent
	}
	return false
}

func (x *SocketTapConfig) GetStatsPrefix() string {
	if x != nil {
		return x.StatsPrefix
	}
	return ""
}

var File_envoy_extensions_transport_sockets_tap_v3_tap_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDesc = "" +
	"\n" +
	"3envoy/extensions/transport_sockets/tap/v3/tap.proto\x12)envoy.extensions.transport_sockets.tap.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a+envoy/extensions/common/tap/v3/common.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xe5\x02\n" +
	"\x03Tap\x12d\n" +
	"\rcommon_config\x18\x01 \x01(\v25.envoy.extensions.common.tap.v3.CommonExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fcommonConfig\x12Z\n" +
	"\x10transport_socket\x18\x02 \x01(\v2%.envoy.config.core.v3.TransportSocketB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0ftransportSocket\x12f\n" +
	"\x11socket_tap_config\x18\x03 \x01(\v2:.envoy.extensions.transport_sockets.tap.v3.SocketTapConfigR\x0fsocketTapConfig:4\x9aň\x1e/\n" +
	"-envoy.config.transport_socket.tap.v2alpha.Tap\"m\n" +
	"\x0fSocketTapConfig\x127\n" +
	"\x18set_connection_per_event\x18\x01 \x01(\bR\x15setConnectionPerEvent\x12!\n" +
	"\fstats_prefix\x18\x02 \x01(\tR\vstatsPrefixB\xa5\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"7io.envoyproxy.envoy.extensions.transport_sockets.tap.v3B\bTapProtoP\x01ZVgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_transport_sockets_tap_v3_tap_proto_goTypes = []any{
	(*Tap)(nil),                      // 0: envoy.extensions.transport_sockets.tap.v3.Tap
	(*SocketTapConfig)(nil),          // 1: envoy.extensions.transport_sockets.tap.v3.SocketTapConfig
	(*v3.CommonExtensionConfig)(nil), // 2: envoy.extensions.common.tap.v3.CommonExtensionConfig
	(*v31.TransportSocket)(nil),      // 3: envoy.config.core.v3.TransportSocket
}
var file_envoy_extensions_transport_sockets_tap_v3_tap_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.transport_sockets.tap.v3.Tap.common_config:type_name -> envoy.extensions.common.tap.v3.CommonExtensionConfig
	3, // 1: envoy.extensions.transport_sockets.tap.v3.Tap.transport_socket:type_name -> envoy.config.core.v3.TransportSocket
	1, // 2: envoy.extensions.transport_sockets.tap.v3.Tap.socket_tap_config:type_name -> envoy.extensions.transport_sockets.tap.v3.SocketTapConfig
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_transport_sockets_tap_v3_tap_proto_init() }
func file_envoy_extensions_transport_sockets_tap_v3_tap_proto_init() {
	if File_envoy_extensions_transport_sockets_tap_v3_tap_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tap_v3_tap_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_tap_v3_tap_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_tap_v3_tap_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_tap_v3_tap_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_tap_v3_tap_proto = out.File
	file_envoy_extensions_transport_sockets_tap_v3_tap_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_tap_v3_tap_proto_depIdxs = nil
}
