// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/starttls/v3/starttls.proto

package starttlsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/raw_buffer/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for a downstream StartTls transport socket.
// StartTls transport socket wraps two sockets:
// * raw_buffer socket which is used at the beginning of the session
// * TLS socket used when a protocol negotiates a switch to encrypted traffic.
type StartTlsConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// (optional) Configuration for clear-text socket used at the beginning of the session.
	CleartextSocketConfig *v3.RawBuffer `protobuf:"bytes,1,opt,name=cleartext_socket_config,json=cleartextSocketConfig,proto3" json:"cleartext_socket_config,omitempty"`
	// Configuration for a downstream TLS socket.
	TlsSocketConfig *v31.DownstreamTlsContext `protobuf:"bytes,2,opt,name=tls_socket_config,json=tlsSocketConfig,proto3" json:"tls_socket_config,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *StartTlsConfig) Reset() {
	*x = StartTlsConfig{}
	mi := &file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StartTlsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StartTlsConfig) ProtoMessage() {}

func (x *StartTlsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StartTlsConfig.ProtoReflect.Descriptor instead.
func (*StartTlsConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescGZIP(), []int{0}
}

func (x *StartTlsConfig) GetCleartextSocketConfig() *v3.RawBuffer {
	if x != nil {
		return x.CleartextSocketConfig
	}
	return nil
}

func (x *StartTlsConfig) GetTlsSocketConfig() *v31.DownstreamTlsContext {
	if x != nil {
		return x.TlsSocketConfig
	}
	return nil
}

// Configuration for an upstream StartTls transport socket.
// StartTls transport socket wraps two sockets:
// * raw_buffer socket which is used at the beginning of the session
// * TLS socket used when a protocol negotiates a switch to encrypted traffic.
type UpstreamStartTlsConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// (optional) Configuration for clear-text socket used at the beginning of the session.
	CleartextSocketConfig *v3.RawBuffer `protobuf:"bytes,1,opt,name=cleartext_socket_config,json=cleartextSocketConfig,proto3" json:"cleartext_socket_config,omitempty"`
	// Configuration for an upstream TLS socket.
	TlsSocketConfig *v31.UpstreamTlsContext `protobuf:"bytes,2,opt,name=tls_socket_config,json=tlsSocketConfig,proto3" json:"tls_socket_config,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *UpstreamStartTlsConfig) Reset() {
	*x = UpstreamStartTlsConfig{}
	mi := &file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpstreamStartTlsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpstreamStartTlsConfig) ProtoMessage() {}

func (x *UpstreamStartTlsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpstreamStartTlsConfig.ProtoReflect.Descriptor instead.
func (*UpstreamStartTlsConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescGZIP(), []int{1}
}

func (x *UpstreamStartTlsConfig) GetCleartextSocketConfig() *v3.RawBuffer {
	if x != nil {
		return x.CleartextSocketConfig
	}
	return nil
}

func (x *UpstreamStartTlsConfig) GetTlsSocketConfig() *v31.UpstreamTlsContext {
	if x != nil {
		return x.TlsSocketConfig
	}
	return nil
}

var File_envoy_extensions_transport_sockets_starttls_v3_starttls_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDesc = "" +
	"\n" +
	"=envoy/extensions/transport_sockets/starttls/v3/starttls.proto\x12.envoy.extensions.transport_sockets.starttls.v3\x1aAenvoy/extensions/transport_sockets/raw_buffer/v3/raw_buffer.proto\x1a3envoy/extensions/transport_sockets/tls/v3/tls.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xfc\x01\n" +
	"\x0eStartTlsConfig\x12s\n" +
	"\x17cleartext_socket_config\x18\x01 \x01(\v2;.envoy.extensions.transport_sockets.raw_buffer.v3.RawBufferR\x15cleartextSocketConfig\x12u\n" +
	"\x11tls_socket_config\x18\x02 \x01(\v2?.envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContextB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0ftlsSocketConfig\"\x82\x02\n" +
	"\x16UpstreamStartTlsConfig\x12s\n" +
	"\x17cleartext_socket_config\x18\x01 \x01(\v2;.envoy.extensions.transport_sockets.raw_buffer.v3.RawBufferR\x15cleartextSocketConfig\x12s\n" +
	"\x11tls_socket_config\x18\x02 \x01(\v2=.envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContextB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0ftlsSocketConfigB\xb9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"<io.envoyproxy.envoy.extensions.transport_sockets.starttls.v3B\rStarttlsProtoP\x01Z`github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/starttls/v3;starttlsv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDesc), len(file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_goTypes = []any{
	(*StartTlsConfig)(nil),           // 0: envoy.extensions.transport_sockets.starttls.v3.StartTlsConfig
	(*UpstreamStartTlsConfig)(nil),   // 1: envoy.extensions.transport_sockets.starttls.v3.UpstreamStartTlsConfig
	(*v3.RawBuffer)(nil),             // 2: envoy.extensions.transport_sockets.raw_buffer.v3.RawBuffer
	(*v31.DownstreamTlsContext)(nil), // 3: envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContext
	(*v31.UpstreamTlsContext)(nil),   // 4: envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext
}
var file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.transport_sockets.starttls.v3.StartTlsConfig.cleartext_socket_config:type_name -> envoy.extensions.transport_sockets.raw_buffer.v3.RawBuffer
	3, // 1: envoy.extensions.transport_sockets.starttls.v3.StartTlsConfig.tls_socket_config:type_name -> envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContext
	2, // 2: envoy.extensions.transport_sockets.starttls.v3.UpstreamStartTlsConfig.cleartext_socket_config:type_name -> envoy.extensions.transport_sockets.raw_buffer.v3.RawBuffer
	4, // 3: envoy.extensions.transport_sockets.starttls.v3.UpstreamStartTlsConfig.tls_socket_config:type_name -> envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_init() }
func file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_init() {
	if File_envoy_extensions_transport_sockets_starttls_v3_starttls_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDesc), len(file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_starttls_v3_starttls_proto = out.File
	file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_starttls_v3_starttls_proto_depIdxs = nil
}
