/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha2

// TargetModelApplyConfiguration represents a declarative configuration of the TargetModel type for use
// with apply.
//
// TargetModel defines a weighted model destination for traffic distribution.
type TargetModelApplyConfiguration struct {
	// (The following comment is copied from the original targetModel)
	// Weight is used to determine the proportion of traffic that should be
	// sent to this model when multiple target models are specified.
	//
	// Weight defines the proportion of requests forwarded to the specified
	// model. This is computed as weight/(sum of all weights in this
	// TargetModels list). For non-zero values, there may be some epsilon from
	// the exact proportion defined here depending on the precision an
	// implementation supports. Weight is not a percentage and the sum of
	// weights does not need to equal 100.
	//
	// If a weight is set for any targetModel, it must be set for all targetModels.
	// Conversely weights are optional, so long as ALL targetModels do not specify a weight.
	Weight *int32 `json:"weight,omitempty"`
	// --- Destination Types ---
	// ModelRewrite specifies a static model name destination.
	ModelRewrite *string `json:"modelRewrite,omitempty"`
}

// TargetModelApplyConfiguration constructs a declarative configuration of the TargetModel type for use with
// apply.
func TargetModel() *TargetModelApplyConfiguration {
	return &TargetModelApplyConfiguration{}
}

// WithWeight sets the Weight field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Weight field is set to the value of the last call.
func (b *TargetModelApplyConfiguration) WithWeight(value int32) *TargetModelApplyConfiguration {
	b.Weight = &value
	return b
}

// WithModelRewrite sets the ModelRewrite field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ModelRewrite field is set to the value of the last call.
func (b *TargetModelApplyConfiguration) WithModelRewrite(value string) *TargetModelApplyConfiguration {
	b.ModelRewrite = &value
	return b
}
