// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha1

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for IstioStatus
func (this *IstioStatus) MarshalJSON() ([]byte, error) {
	str, err := StatusMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for IstioStatus
func (this *IstioStatus) UnmarshalJSON(b []byte) error {
	return StatusUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

// MarshalJSON is a custom marshaler for IstioCondition
func (this *IstioCondition) MarshalJSON() ([]byte, error) {
	str, err := StatusMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for IstioCondition
func (this *IstioCondition) UnmarshalJSON(b []byte) error {
	return StatusUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	StatusMarshaler   = &jsonpb.Marshaler{}
	StatusUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
