// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/network/peer_metadata/v3/peer_metadata.proto

package peer_metadatav3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the downstream network filter installed in the
// connect_originate or inner_connect_originate filter chains. It extracts
// peer metadata from TCP Proxy filter state (baggage header from HBONE CONNECT
// response) and injects it as a data preamble to be consumed by the upstream
// filter.
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Filter state key under which the baggage value encoding the proxy workload
	// is stored. The upstream filter that populates the baggage header in the
	// HBONE request should use the same key.
	BaggageKey    string `protobuf:"bytes,1,opt,name=baggage_key,json=baggageKey,proto3" json:"baggage_key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetBaggageKey() string {
	if x != nil {
		return x.BaggageKey
	}
	return ""
}

// Configuration for the upstream network filter installed in service clusters
// that communicate over HBONE. It consumes and removes the metadata preamble
// injected by the downstream filter and populates filter state for use by
// Istio telemetry filters.
type UpstreamConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpstreamConfig) Reset() {
	*x = UpstreamConfig{}
	mi := &file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpstreamConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpstreamConfig) ProtoMessage() {}

func (x *UpstreamConfig) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpstreamConfig.ProtoReflect.Descriptor instead.
func (*UpstreamConfig) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescGZIP(), []int{1}
}

var File_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDesc = "" +
	"\n" +
	"Mcontrib/envoy/extensions/filters/network/peer_metadata/v3/peer_metadata.proto\x121envoy.extensions.filters.network.peer_metadata.v3\x1a\x1dudpa/annotations/status.proto\")\n" +
	"\x06Config\x12\x1f\n" +
	"\vbaggage_key\x18\x01 \x01(\tR\n" +
	"baggageKey\"\x10\n" +
	"\x0eUpstreamConfigB\xd0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"?io.envoyproxy.envoy.extensions.filters.network.peer_metadata.v3B\x11PeerMetadataProtoP\x01Zpgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/peer_metadata/v3;peer_metadatav3b\x06proto3"

var (
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_goTypes = []any{
	(*Config)(nil),         // 0: envoy.extensions.filters.network.peer_metadata.v3.Config
	(*UpstreamConfig)(nil), // 1: envoy.extensions.filters.network.peer_metadata.v3.UpstreamConfig
}
var file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_init()
}
func file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_init() {
	if File_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto = out.File
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_network_peer_metadata_v3_peer_metadata_proto_depIdxs = nil
}
