// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/rbac/v3/rbac.proto

package rbacv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v31 "github.com/cncf/xds/go/xds/type/matcher/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/rbac/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RBAC_EnforcementType int32

const (
	// Apply RBAC policies when the first byte of data arrives on the connection.
	RBAC_ONE_TIME_ON_FIRST_BYTE RBAC_EnforcementType = 0
	// Continuously apply RBAC policies as data arrives. Use this mode when
	// using RBAC with message oriented protocols such as Mongo, MySQL, Kafka,
	// etc. when the protocol decoders emit dynamic metadata such as the
	// resources being accessed and the operations on the resources.
	RBAC_CONTINUOUS RBAC_EnforcementType = 1
)

// Enum value maps for RBAC_EnforcementType.
var (
	RBAC_EnforcementType_name = map[int32]string{
		0: "ONE_TIME_ON_FIRST_BYTE",
		1: "CONTINUOUS",
	}
	RBAC_EnforcementType_value = map[string]int32{
		"ONE_TIME_ON_FIRST_BYTE": 0,
		"CONTINUOUS":             1,
	}
)

func (x RBAC_EnforcementType) Enum() *RBAC_EnforcementType {
	p := new(RBAC_EnforcementType)
	*p = x
	return p
}

func (x RBAC_EnforcementType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RBAC_EnforcementType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_rbac_v3_rbac_proto_enumTypes[0].Descriptor()
}

func (RBAC_EnforcementType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_rbac_v3_rbac_proto_enumTypes[0]
}

func (x RBAC_EnforcementType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use RBAC_EnforcementType.Descriptor instead.
func (RBAC_EnforcementType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescGZIP(), []int{0, 0}
}

// RBAC network filter config.
//
// Header should not be used in rules/shadow_rules in RBAC network filter as
// this information is only available in :ref:`RBAC http filter <config_http_filters_rbac>`.
// [#next-free-field: 9]
type RBAC struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specify the RBAC rules to be applied globally.
	// If absent, no enforcing RBAC policy will be applied.
	// If present and empty, DENY.
	// If both rules and matcher are configured, rules will be ignored.
	Rules *v3.RBAC `protobuf:"bytes,1,opt,name=rules,proto3" json:"rules,omitempty"`
	// The match tree to use when resolving RBAC action for incoming connections. Connections do
	// not match any matcher will be denied.
	// If absent, no enforcing RBAC matcher will be applied.
	// If present and empty, deny all connections.
	Matcher *v31.Matcher `protobuf:"bytes,6,opt,name=matcher,proto3" json:"matcher,omitempty"`
	// Shadow rules are not enforced by the filter but will emit stats and logs
	// and can be used for rule testing.
	// If absent, no shadow RBAC policy will be applied.
	// If both shadow rules and shadow matcher are configured, shadow rules will be ignored.
	ShadowRules *v3.RBAC `protobuf:"bytes,2,opt,name=shadow_rules,json=shadowRules,proto3" json:"shadow_rules,omitempty"`
	// The match tree to use for emitting stats and logs which can be used for rule testing for
	// incoming connections.
	// If absent, no shadow matcher will be applied.
	ShadowMatcher *v31.Matcher `protobuf:"bytes,7,opt,name=shadow_matcher,json=shadowMatcher,proto3" json:"shadow_matcher,omitempty"`
	// If specified, shadow rules will emit stats with the given prefix.
	// This is useful to distinguish the stat when there are more than 1 RBAC filter configured with
	// shadow rules.
	ShadowRulesStatPrefix string `protobuf:"bytes,5,opt,name=shadow_rules_stat_prefix,json=shadowRulesStatPrefix,proto3" json:"shadow_rules_stat_prefix,omitempty"`
	// The prefix to use when emitting statistics.
	StatPrefix string `protobuf:"bytes,3,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// RBAC enforcement strategy. By default RBAC will be enforced only once
	// when the first byte of data arrives from the downstream. When used in
	// conjunction with filters that emit dynamic metadata after decoding
	// every payload (e.g., Mongo, MySQL, Kafka) set the enforcement type to
	// CONTINUOUS to enforce RBAC policies on every message boundary.
	EnforcementType RBAC_EnforcementType `protobuf:"varint,4,opt,name=enforcement_type,json=enforcementType,proto3,enum=envoy.extensions.filters.network.rbac.v3.RBAC_EnforcementType" json:"enforcement_type,omitempty"`
	// Delay the specified duration before closing the connection when the policy evaluation
	// result is “DENY“. If this is not present, the connection will be closed immediately.
	// This is useful to provide a better protection for Envoy against clients that retries
	// aggressively when the connection is rejected by the RBAC filter.
	DelayDeny     *durationpb.Duration `protobuf:"bytes,8,opt,name=delay_deny,json=delayDeny,proto3" json:"delay_deny,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RBAC) Reset() {
	*x = RBAC{}
	mi := &file_envoy_extensions_filters_network_rbac_v3_rbac_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RBAC) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RBAC) ProtoMessage() {}

func (x *RBAC) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_rbac_v3_rbac_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RBAC.ProtoReflect.Descriptor instead.
func (*RBAC) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescGZIP(), []int{0}
}

func (x *RBAC) GetRules() *v3.RBAC {
	if x != nil {
		return x.Rules
	}
	return nil
}

func (x *RBAC) GetMatcher() *v31.Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

func (x *RBAC) GetShadowRules() *v3.RBAC {
	if x != nil {
		return x.ShadowRules
	}
	return nil
}

func (x *RBAC) GetShadowMatcher() *v31.Matcher {
	if x != nil {
		return x.ShadowMatcher
	}
	return nil
}

func (x *RBAC) GetShadowRulesStatPrefix() string {
	if x != nil {
		return x.ShadowRulesStatPrefix
	}
	return ""
}

func (x *RBAC) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *RBAC) GetEnforcementType() RBAC_EnforcementType {
	if x != nil {
		return x.EnforcementType
	}
	return RBAC_ONE_TIME_ON_FIRST_BYTE
}

func (x *RBAC) GetDelayDeny() *durationpb.Duration {
	if x != nil {
		return x.DelayDeny
	}
	return nil
}

var File_envoy_extensions_filters_network_rbac_v3_rbac_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDesc = "" +
	"\n" +
	"3envoy/extensions/filters/network/rbac/v3/rbac.proto\x12(envoy.extensions.filters.network.rbac.v3\x1a\x1fenvoy/config/rbac/v3/rbac.proto\x1a\x1egoogle/protobuf/duration.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xde\x05\n" +
	"\x04RBAC\x12I\n" +
	"\x05rules\x18\x01 \x01(\v2\x1a.envoy.config.rbac.v3.RBACB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0frules_specifierR\x05rules\x12O\n" +
	"\amatcher\x18\x06 \x01(\v2\x1c.xds.type.matcher.v3.MatcherB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0frules_specifierR\amatcher\x12]\n" +
	"\fshadow_rules\x18\x02 \x01(\v2\x1a.envoy.config.rbac.v3.RBACB\x1e\xf2\x98\xfe\x8f\x05\x18\x12\x16shadow_rules_specifierR\vshadowRules\x12c\n" +
	"\x0eshadow_matcher\x18\a \x01(\v2\x1c.xds.type.matcher.v3.MatcherB\x1e\xf2\x98\xfe\x8f\x05\x18\x12\x16shadow_rules_specifierR\rshadowMatcher\x127\n" +
	"\x18shadow_rules_stat_prefix\x18\x05 \x01(\tR\x15shadowRulesStatPrefix\x12(\n" +
	"\vstat_prefix\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12i\n" +
	"\x10enforcement_type\x18\x04 \x01(\x0e2>.envoy.extensions.filters.network.rbac.v3.RBAC.EnforcementTypeR\x0fenforcementType\x128\n" +
	"\n" +
	"delay_deny\x18\b \x01(\v2\x19.google.protobuf.DurationR\tdelayDeny\"=\n" +
	"\x0fEnforcementType\x12\x1a\n" +
	"\x16ONE_TIME_ON_FIRST_BYTE\x10\x00\x12\x0e\n" +
	"\n" +
	"CONTINUOUS\x10\x01:/\x9aň\x1e*\n" +
	"(envoy.config.filter.network.rbac.v2.RBACB\xa5\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"6io.envoyproxy.envoy.extensions.filters.network.rbac.v3B\tRbacProtoP\x01ZVgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/rbac/v3;rbacv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDesc), len(file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDescData
}

var file_envoy_extensions_filters_network_rbac_v3_rbac_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_filters_network_rbac_v3_rbac_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_rbac_v3_rbac_proto_goTypes = []any{
	(RBAC_EnforcementType)(0),   // 0: envoy.extensions.filters.network.rbac.v3.RBAC.EnforcementType
	(*RBAC)(nil),                // 1: envoy.extensions.filters.network.rbac.v3.RBAC
	(*v3.RBAC)(nil),             // 2: envoy.config.rbac.v3.RBAC
	(*v31.Matcher)(nil),         // 3: xds.type.matcher.v3.Matcher
	(*durationpb.Duration)(nil), // 4: google.protobuf.Duration
}
var file_envoy_extensions_filters_network_rbac_v3_rbac_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.network.rbac.v3.RBAC.rules:type_name -> envoy.config.rbac.v3.RBAC
	3, // 1: envoy.extensions.filters.network.rbac.v3.RBAC.matcher:type_name -> xds.type.matcher.v3.Matcher
	2, // 2: envoy.extensions.filters.network.rbac.v3.RBAC.shadow_rules:type_name -> envoy.config.rbac.v3.RBAC
	3, // 3: envoy.extensions.filters.network.rbac.v3.RBAC.shadow_matcher:type_name -> xds.type.matcher.v3.Matcher
	0, // 4: envoy.extensions.filters.network.rbac.v3.RBAC.enforcement_type:type_name -> envoy.extensions.filters.network.rbac.v3.RBAC.EnforcementType
	4, // 5: envoy.extensions.filters.network.rbac.v3.RBAC.delay_deny:type_name -> google.protobuf.Duration
	6, // [6:6] is the sub-list for method output_type
	6, // [6:6] is the sub-list for method input_type
	6, // [6:6] is the sub-list for extension type_name
	6, // [6:6] is the sub-list for extension extendee
	0, // [0:6] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_rbac_v3_rbac_proto_init() }
func file_envoy_extensions_filters_network_rbac_v3_rbac_proto_init() {
	if File_envoy_extensions_filters_network_rbac_v3_rbac_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDesc), len(file_envoy_extensions_filters_network_rbac_v3_rbac_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_rbac_v3_rbac_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_rbac_v3_rbac_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_network_rbac_v3_rbac_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_network_rbac_v3_rbac_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_rbac_v3_rbac_proto = out.File
	file_envoy_extensions_filters_network_rbac_v3_rbac_proto_goTypes = nil
	file_envoy_extensions_filters_network_rbac_v3_rbac_proto_depIdxs = nil
}
