// Code generated by protoc-gen-deepcopy. DO NOT EDIT.
package v1alpha1

import (
	proto "google.golang.org/protobuf/proto"
)

// DeepCopyInto supports using TrafficExtension within kubernetes types, where deepcopy-gen is used.
func (in *TrafficExtension) DeepCopyInto(out *TrafficExtension) {
	p := proto.Clone(in).(*TrafficExtension)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new TrafficExtension. Required by controller-gen.
func (in *TrafficExtension) DeepCopy() *TrafficExtension {
	if in == nil {
		return nil
	}
	out := new(TrafficExtension)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new TrafficExtension. Required by controller-gen.
func (in *TrafficExtension) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using WasmConfig within kubernetes types, where deepcopy-gen is used.
func (in *WasmConfig) DeepCopyInto(out *WasmConfig) {
	p := proto.Clone(in).(*WasmConfig)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new WasmConfig. Required by controller-gen.
func (in *WasmConfig) DeepCopy() *WasmConfig {
	if in == nil {
		return nil
	}
	out := new(WasmConfig)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new WasmConfig. Required by controller-gen.
func (in *WasmConfig) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using LuaConfig within kubernetes types, where deepcopy-gen is used.
func (in *LuaConfig) DeepCopyInto(out *LuaConfig) {
	p := proto.Clone(in).(*LuaConfig)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new LuaConfig. Required by controller-gen.
func (in *LuaConfig) DeepCopy() *LuaConfig {
	if in == nil {
		return nil
	}
	out := new(LuaConfig)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new LuaConfig. Required by controller-gen.
func (in *LuaConfig) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using TrafficSelector within kubernetes types, where deepcopy-gen is used.
func (in *TrafficSelector) DeepCopyInto(out *TrafficSelector) {
	p := proto.Clone(in).(*TrafficSelector)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new TrafficSelector. Required by controller-gen.
func (in *TrafficSelector) DeepCopy() *TrafficSelector {
	if in == nil {
		return nil
	}
	out := new(TrafficSelector)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new TrafficSelector. Required by controller-gen.
func (in *TrafficSelector) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}
