// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/http/golang/v3alpha/golang.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The meanings are as follows:
//
// :“MERGE_VIRTUALHOST_ROUTER_FILTER“: Pass all configuration into Go plugin.
// :“MERGE_VIRTUALHOST_ROUTER“: Pass merged Virtual host and Router configuration into Go plugin.
// :“OVERRIDE“: Pass merged Virtual host, Router, and plugin configuration into Go plugin.
//
// [#not-implemented-hide:]
type Config_MergePolicy int32

const (
	Config_MERGE_VIRTUALHOST_ROUTER_FILTER Config_MergePolicy = 0
	Config_MERGE_VIRTUALHOST_ROUTER        Config_MergePolicy = 1
	Config_OVERRIDE                        Config_MergePolicy = 3
)

// Enum value maps for Config_MergePolicy.
var (
	Config_MergePolicy_name = map[int32]string{
		0: "MERGE_VIRTUALHOST_ROUTER_FILTER",
		1: "MERGE_VIRTUALHOST_ROUTER",
		3: "OVERRIDE",
	}
	Config_MergePolicy_value = map[string]int32{
		"MERGE_VIRTUALHOST_ROUTER_FILTER": 0,
		"MERGE_VIRTUALHOST_ROUTER":        1,
		"OVERRIDE":                        3,
	}
)

func (x Config_MergePolicy) Enum() *Config_MergePolicy {
	p := new(Config_MergePolicy)
	*p = x
	return p
}

func (x Config_MergePolicy) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Config_MergePolicy) Descriptor() protoreflect.EnumDescriptor {
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_enumTypes[0].Descriptor()
}

func (Config_MergePolicy) Type() protoreflect.EnumType {
	return &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_enumTypes[0]
}

func (x Config_MergePolicy) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Config_MergePolicy.Descriptor instead.
func (Config_MergePolicy) EnumDescriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescGZIP(), []int{0, 0}
}

// [#next-free-field: 7]
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Globally unique ID for a dynamic library file.
	LibraryId string `protobuf:"bytes,1,opt,name=library_id,json=libraryId,proto3" json:"library_id,omitempty"`
	// Path to a dynamic library implementing the
	// :repo:`StreamFilter API <contrib/golang/common/go/api.StreamFilter>`
	// interface.
	// [#comment:TODO(wangfakang): Support for downloading libraries from remote repositories.]
	LibraryPath string `protobuf:"bytes,2,opt,name=library_path,json=libraryPath,proto3" json:"library_path,omitempty"`
	// Globally unique name of the Go plugin.
	//
	// This name **must** be consistent with the name registered in “http::RegisterHttpFilterConfigFactory“,
	// and can be used to associate :ref:`route and virtualHost plugin configuration
	// <envoy_v3_api_field_extensions.filters.http.golang.v3alpha.ConfigsPerRoute.plugins_config>`.
	PluginName string `protobuf:"bytes,3,opt,name=plugin_name,json=pluginName,proto3" json:"plugin_name,omitempty"`
	// Configuration for the Go plugin.
	//
	// .. note::
	//
	//	This configuration is only parsed in the go plugin, and is therefore not validated
	//	by Envoy.
	//
	//	See the :repo:`StreamFilter API <contrib/golang/common/go/api/filter.go>`
	//	for more information about how the plugin's configuration data can be accessed.
	PluginConfig *anypb.Any `protobuf:"bytes,4,opt,name=plugin_config,json=pluginConfig,proto3" json:"plugin_config,omitempty"`
	// Merge policy for plugin configuration.
	//
	// The Go plugin configuration supports three dimensions:
	//
	// * Virtual host’s :ref:`typed_per_filter_config <envoy_v3_api_field_config.route.v3.VirtualHost.typed_per_filter_config>`
	// * Route’s :ref:`typed_per_filter_config <envoy_v3_api_field_config.route.v3.Route.typed_per_filter_config>`
	// * The filter's :ref:`plugin_config <envoy_v3_api_field_extensions.filters.http.golang.v3alpha.Config.plugin_config>`
	//
	// [#not-implemented-hide:]
	MergePolicy Config_MergePolicy `protobuf:"varint,5,opt,name=merge_policy,json=mergePolicy,proto3,enum=envoy.extensions.filters.http.golang.v3alpha.Config_MergePolicy" json:"merge_policy,omitempty"`
	// Generic secret list available to the plugin.
	// Looks into SDS or static bootstrap configuration.
	//
	// See :repo:`StreamFilter API <contrib/golang/common/go/api/filter.go>`
	// for more information about how to access secrets from Go.
	GenericSecrets []*v3.SdsSecretConfig `protobuf:"bytes,6,rep,name=generic_secrets,json=genericSecrets,proto3" json:"generic_secrets,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetLibraryId() string {
	if x != nil {
		return x.LibraryId
	}
	return ""
}

func (x *Config) GetLibraryPath() string {
	if x != nil {
		return x.LibraryPath
	}
	return ""
}

func (x *Config) GetPluginName() string {
	if x != nil {
		return x.PluginName
	}
	return ""
}

func (x *Config) GetPluginConfig() *anypb.Any {
	if x != nil {
		return x.PluginConfig
	}
	return nil
}

func (x *Config) GetMergePolicy() Config_MergePolicy {
	if x != nil {
		return x.MergePolicy
	}
	return Config_MERGE_VIRTUALHOST_ROUTER_FILTER
}

func (x *Config) GetGenericSecrets() []*v3.SdsSecretConfig {
	if x != nil {
		return x.GenericSecrets
	}
	return nil
}

type RouterPlugin struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Override:
	//
	//	*RouterPlugin_Disabled
	//	*RouterPlugin_Config
	Override      isRouterPlugin_Override `protobuf_oneof:"override"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RouterPlugin) Reset() {
	*x = RouterPlugin{}
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RouterPlugin) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RouterPlugin) ProtoMessage() {}

func (x *RouterPlugin) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RouterPlugin.ProtoReflect.Descriptor instead.
func (*RouterPlugin) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescGZIP(), []int{1}
}

func (x *RouterPlugin) GetOverride() isRouterPlugin_Override {
	if x != nil {
		return x.Override
	}
	return nil
}

func (x *RouterPlugin) GetDisabled() bool {
	if x != nil {
		if x, ok := x.Override.(*RouterPlugin_Disabled); ok {
			return x.Disabled
		}
	}
	return false
}

func (x *RouterPlugin) GetConfig() *anypb.Any {
	if x != nil {
		if x, ok := x.Override.(*RouterPlugin_Config); ok {
			return x.Config
		}
	}
	return nil
}

type isRouterPlugin_Override interface {
	isRouterPlugin_Override()
}

type RouterPlugin_Disabled struct {
	// [#not-implemented-hide:]
	// Disable the filter for this particular vhost or route.
	// If disabled is specified in multiple per-filter-configs, the most specific one will be used.
	Disabled bool `protobuf:"varint,1,opt,name=disabled,proto3,oneof"`
}

type RouterPlugin_Config struct {
	// The config field is used for setting per-route and per-virtualhost plugin config.
	Config *anypb.Any `protobuf:"bytes,2,opt,name=config,proto3,oneof"`
}

func (*RouterPlugin_Disabled) isRouterPlugin_Override() {}

func (*RouterPlugin_Config) isRouterPlugin_Override() {}

type ConfigsPerRoute struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Configuration of the Go plugin at the per-router or per-virtualhost level,
	// keyed on the :ref:`plugin_name <envoy_v3_api_field_extensions.filters.http.golang.v3alpha.Config.plugin_name>`
	// of the Go plugin.
	PluginsConfig map[string]*RouterPlugin `protobuf:"bytes,1,rep,name=plugins_config,json=pluginsConfig,proto3" json:"plugins_config,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConfigsPerRoute) Reset() {
	*x = ConfigsPerRoute{}
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConfigsPerRoute) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConfigsPerRoute) ProtoMessage() {}

func (x *ConfigsPerRoute) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConfigsPerRoute.ProtoReflect.Descriptor instead.
func (*ConfigsPerRoute) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescGZIP(), []int{2}
}

func (x *ConfigsPerRoute) GetPluginsConfig() map[string]*RouterPlugin {
	if x != nil {
		return x.PluginsConfig
	}
	return nil
}

var File_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDesc = "" +
	"\n" +
	"Acontrib/envoy/extensions/filters/http/golang/v3alpha/golang.proto\x12,envoy.extensions.filters.http.golang.v3alpha\x1a6envoy/extensions/transport_sockets/tls/v3/secret.proto\x1a\x19google/protobuf/any.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf5\x03\n" +
	"\x06Config\x12&\n" +
	"\n" +
	"library_id\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\tlibraryId\x12*\n" +
	"\flibrary_path\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vlibraryPath\x12(\n" +
	"\vplugin_name\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"pluginName\x129\n" +
	"\rplugin_config\x18\x04 \x01(\v2\x14.google.protobuf.AnyR\fpluginConfig\x12m\n" +
	"\fmerge_policy\x18\x05 \x01(\x0e2@.envoy.extensions.filters.http.golang.v3alpha.Config.MergePolicyB\b\xfaB\x05\x82\x01\x02\x10\x01R\vmergePolicy\x12c\n" +
	"\x0fgeneric_secrets\x18\x06 \x03(\v2:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfigR\x0egenericSecrets\"^\n" +
	"\vMergePolicy\x12#\n" +
	"\x1fMERGE_VIRTUALHOST_ROUTER_FILTER\x10\x00\x12\x1c\n" +
	"\x18MERGE_VIRTUALHOST_ROUTER\x10\x01\x12\f\n" +
	"\bOVERRIDE\x10\x03\"v\n" +
	"\fRouterPlugin\x12%\n" +
	"\bdisabled\x18\x01 \x01(\bB\a\xfaB\x04j\x02\b\x01H\x00R\bdisabled\x12.\n" +
	"\x06config\x18\x02 \x01(\v2\x14.google.protobuf.AnyH\x00R\x06configB\x0f\n" +
	"\boverride\x12\x03\xf8B\x01\"\x88\x02\n" +
	"\x0fConfigsPerRoute\x12w\n" +
	"\x0eplugins_config\x18\x01 \x03(\v2P.envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute.PluginsConfigEntryR\rpluginsConfig\x1a|\n" +
	"\x12PluginsConfigEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12P\n" +
	"\x05value\x18\x02 \x01(\v2:.envoy.extensions.filters.http.golang.v3alpha.RouterPluginR\x05value:\x028\x01B\xb8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	":io.envoyproxy.envoy.extensions.filters.http.golang.v3alphaB\vGolangProtoP\x01Z[github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/golang/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDesc), len(file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_goTypes = []any{
	(Config_MergePolicy)(0),    // 0: envoy.extensions.filters.http.golang.v3alpha.Config.MergePolicy
	(*Config)(nil),             // 1: envoy.extensions.filters.http.golang.v3alpha.Config
	(*RouterPlugin)(nil),       // 2: envoy.extensions.filters.http.golang.v3alpha.RouterPlugin
	(*ConfigsPerRoute)(nil),    // 3: envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute
	nil,                        // 4: envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute.PluginsConfigEntry
	(*anypb.Any)(nil),          // 5: google.protobuf.Any
	(*v3.SdsSecretConfig)(nil), // 6: envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig
}
var file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_depIdxs = []int32{
	5, // 0: envoy.extensions.filters.http.golang.v3alpha.Config.plugin_config:type_name -> google.protobuf.Any
	0, // 1: envoy.extensions.filters.http.golang.v3alpha.Config.merge_policy:type_name -> envoy.extensions.filters.http.golang.v3alpha.Config.MergePolicy
	6, // 2: envoy.extensions.filters.http.golang.v3alpha.Config.generic_secrets:type_name -> envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig
	5, // 3: envoy.extensions.filters.http.golang.v3alpha.RouterPlugin.config:type_name -> google.protobuf.Any
	4, // 4: envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute.plugins_config:type_name -> envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute.PluginsConfigEntry
	2, // 5: envoy.extensions.filters.http.golang.v3alpha.ConfigsPerRoute.PluginsConfigEntry.value:type_name -> envoy.extensions.filters.http.golang.v3alpha.RouterPlugin
	6, // [6:6] is the sub-list for method output_type
	6, // [6:6] is the sub-list for method input_type
	6, // [6:6] is the sub-list for extension type_name
	6, // [6:6] is the sub-list for extension extendee
	0, // [0:6] is the sub-list for field type_name
}

func init() { file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_init() }
func file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_init() {
	if File_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto != nil {
		return
	}
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes[1].OneofWrappers = []any{
		(*RouterPlugin_Disabled)(nil),
		(*RouterPlugin_Config)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDesc), len(file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_depIdxs,
		EnumInfos:         file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_enumTypes,
		MessageInfos:      file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto = out.File
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_http_golang_v3alpha_golang_proto_depIdxs = nil
}
