// Code generated by protoc-gen-deepcopy. DO NOT EDIT.
package v1alpha1

import (
	proto "google.golang.org/protobuf/proto"
)

// DeepCopyInto supports using Telemetry within kubernetes types, where deepcopy-gen is used.
func (in *Telemetry) DeepCopyInto(out *Telemetry) {
	p := proto.Clone(in).(*Telemetry)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Telemetry. Required by controller-gen.
func (in *Telemetry) DeepCopy() *Telemetry {
	if in == nil {
		return nil
	}
	out := new(Telemetry)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Telemetry. Required by controller-gen.
func (in *Telemetry) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing within kubernetes types, where deepcopy-gen is used.
func (in *Tracing) DeepCopyInto(out *Tracing) {
	p := proto.Clone(in).(*Tracing)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing. Required by controller-gen.
func (in *Tracing) DeepCopy() *Tracing {
	if in == nil {
		return nil
	}
	out := new(Tracing)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing. Required by controller-gen.
func (in *Tracing) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_TracingSelector within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_TracingSelector) DeepCopyInto(out *Tracing_TracingSelector) {
	p := proto.Clone(in).(*Tracing_TracingSelector)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_TracingSelector. Required by controller-gen.
func (in *Tracing_TracingSelector) DeepCopy() *Tracing_TracingSelector {
	if in == nil {
		return nil
	}
	out := new(Tracing_TracingSelector)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_TracingSelector. Required by controller-gen.
func (in *Tracing_TracingSelector) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_CustomTag within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_CustomTag) DeepCopyInto(out *Tracing_CustomTag) {
	p := proto.Clone(in).(*Tracing_CustomTag)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_CustomTag. Required by controller-gen.
func (in *Tracing_CustomTag) DeepCopy() *Tracing_CustomTag {
	if in == nil {
		return nil
	}
	out := new(Tracing_CustomTag)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_CustomTag. Required by controller-gen.
func (in *Tracing_CustomTag) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_Literal within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_Literal) DeepCopyInto(out *Tracing_Literal) {
	p := proto.Clone(in).(*Tracing_Literal)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Literal. Required by controller-gen.
func (in *Tracing_Literal) DeepCopy() *Tracing_Literal {
	if in == nil {
		return nil
	}
	out := new(Tracing_Literal)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Literal. Required by controller-gen.
func (in *Tracing_Literal) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_Environment within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_Environment) DeepCopyInto(out *Tracing_Environment) {
	p := proto.Clone(in).(*Tracing_Environment)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Environment. Required by controller-gen.
func (in *Tracing_Environment) DeepCopy() *Tracing_Environment {
	if in == nil {
		return nil
	}
	out := new(Tracing_Environment)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Environment. Required by controller-gen.
func (in *Tracing_Environment) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_RequestHeader within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_RequestHeader) DeepCopyInto(out *Tracing_RequestHeader) {
	p := proto.Clone(in).(*Tracing_RequestHeader)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_RequestHeader. Required by controller-gen.
func (in *Tracing_RequestHeader) DeepCopy() *Tracing_RequestHeader {
	if in == nil {
		return nil
	}
	out := new(Tracing_RequestHeader)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_RequestHeader. Required by controller-gen.
func (in *Tracing_RequestHeader) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Tracing_Formatter within kubernetes types, where deepcopy-gen is used.
func (in *Tracing_Formatter) DeepCopyInto(out *Tracing_Formatter) {
	p := proto.Clone(in).(*Tracing_Formatter)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Formatter. Required by controller-gen.
func (in *Tracing_Formatter) DeepCopy() *Tracing_Formatter {
	if in == nil {
		return nil
	}
	out := new(Tracing_Formatter)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Tracing_Formatter. Required by controller-gen.
func (in *Tracing_Formatter) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using ProviderRef within kubernetes types, where deepcopy-gen is used.
func (in *ProviderRef) DeepCopyInto(out *ProviderRef) {
	p := proto.Clone(in).(*ProviderRef)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ProviderRef. Required by controller-gen.
func (in *ProviderRef) DeepCopy() *ProviderRef {
	if in == nil {
		return nil
	}
	out := new(ProviderRef)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new ProviderRef. Required by controller-gen.
func (in *ProviderRef) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using Metrics within kubernetes types, where deepcopy-gen is used.
func (in *Metrics) DeepCopyInto(out *Metrics) {
	p := proto.Clone(in).(*Metrics)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Metrics. Required by controller-gen.
func (in *Metrics) DeepCopy() *Metrics {
	if in == nil {
		return nil
	}
	out := new(Metrics)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Metrics. Required by controller-gen.
func (in *Metrics) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using MetricSelector within kubernetes types, where deepcopy-gen is used.
func (in *MetricSelector) DeepCopyInto(out *MetricSelector) {
	p := proto.Clone(in).(*MetricSelector)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new MetricSelector. Required by controller-gen.
func (in *MetricSelector) DeepCopy() *MetricSelector {
	if in == nil {
		return nil
	}
	out := new(MetricSelector)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new MetricSelector. Required by controller-gen.
func (in *MetricSelector) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using MetricsOverrides within kubernetes types, where deepcopy-gen is used.
func (in *MetricsOverrides) DeepCopyInto(out *MetricsOverrides) {
	p := proto.Clone(in).(*MetricsOverrides)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new MetricsOverrides. Required by controller-gen.
func (in *MetricsOverrides) DeepCopy() *MetricsOverrides {
	if in == nil {
		return nil
	}
	out := new(MetricsOverrides)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new MetricsOverrides. Required by controller-gen.
func (in *MetricsOverrides) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using MetricsOverrides_TagOverride within kubernetes types, where deepcopy-gen is used.
func (in *MetricsOverrides_TagOverride) DeepCopyInto(out *MetricsOverrides_TagOverride) {
	p := proto.Clone(in).(*MetricsOverrides_TagOverride)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new MetricsOverrides_TagOverride. Required by controller-gen.
func (in *MetricsOverrides_TagOverride) DeepCopy() *MetricsOverrides_TagOverride {
	if in == nil {
		return nil
	}
	out := new(MetricsOverrides_TagOverride)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new MetricsOverrides_TagOverride. Required by controller-gen.
func (in *MetricsOverrides_TagOverride) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using AccessLogging within kubernetes types, where deepcopy-gen is used.
func (in *AccessLogging) DeepCopyInto(out *AccessLogging) {
	p := proto.Clone(in).(*AccessLogging)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging. Required by controller-gen.
func (in *AccessLogging) DeepCopy() *AccessLogging {
	if in == nil {
		return nil
	}
	out := new(AccessLogging)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging. Required by controller-gen.
func (in *AccessLogging) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using AccessLogging_LogSelector within kubernetes types, where deepcopy-gen is used.
func (in *AccessLogging_LogSelector) DeepCopyInto(out *AccessLogging_LogSelector) {
	p := proto.Clone(in).(*AccessLogging_LogSelector)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging_LogSelector. Required by controller-gen.
func (in *AccessLogging_LogSelector) DeepCopy() *AccessLogging_LogSelector {
	if in == nil {
		return nil
	}
	out := new(AccessLogging_LogSelector)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging_LogSelector. Required by controller-gen.
func (in *AccessLogging_LogSelector) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using AccessLogging_Filter within kubernetes types, where deepcopy-gen is used.
func (in *AccessLogging_Filter) DeepCopyInto(out *AccessLogging_Filter) {
	p := proto.Clone(in).(*AccessLogging_Filter)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging_Filter. Required by controller-gen.
func (in *AccessLogging_Filter) DeepCopy() *AccessLogging_Filter {
	if in == nil {
		return nil
	}
	out := new(AccessLogging_Filter)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new AccessLogging_Filter. Required by controller-gen.
func (in *AccessLogging_Filter) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}
