// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/redis_auth/v3/redis_external_auth.proto

package redis_authv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	status "google.golang.org/genproto/googleapis/rpc/status"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RedisProxyExternalAuthRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Username, if applicable. Otherwise, empty.
	Username string `protobuf:"bytes,1,opt,name=username,proto3" json:"username,omitempty"`
	// Password sent with the AUTH command.
	Password      string `protobuf:"bytes,2,opt,name=password,proto3" json:"password,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RedisProxyExternalAuthRequest) Reset() {
	*x = RedisProxyExternalAuthRequest{}
	mi := &file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RedisProxyExternalAuthRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RedisProxyExternalAuthRequest) ProtoMessage() {}

func (x *RedisProxyExternalAuthRequest) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RedisProxyExternalAuthRequest.ProtoReflect.Descriptor instead.
func (*RedisProxyExternalAuthRequest) Descriptor() ([]byte, []int) {
	return file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescGZIP(), []int{0}
}

func (x *RedisProxyExternalAuthRequest) GetUsername() string {
	if x != nil {
		return x.Username
	}
	return ""
}

func (x *RedisProxyExternalAuthRequest) GetPassword() string {
	if x != nil {
		return x.Password
	}
	return ""
}

type RedisProxyExternalAuthResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Status of the authentication check.
	Status *status.Status `protobuf:"bytes,1,opt,name=status,proto3" json:"status,omitempty"`
	// Optional expiration time for the authentication.
	// If set, the authentication will be valid until this time.
	// If not set, the authentication will be valid indefinitely.
	Expiration *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=expiration,proto3" json:"expiration,omitempty"`
	// Optional message to be sent back to the client.
	Message       string `protobuf:"bytes,3,opt,name=message,proto3" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RedisProxyExternalAuthResponse) Reset() {
	*x = RedisProxyExternalAuthResponse{}
	mi := &file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RedisProxyExternalAuthResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RedisProxyExternalAuthResponse) ProtoMessage() {}

func (x *RedisProxyExternalAuthResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RedisProxyExternalAuthResponse.ProtoReflect.Descriptor instead.
func (*RedisProxyExternalAuthResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescGZIP(), []int{1}
}

func (x *RedisProxyExternalAuthResponse) GetStatus() *status.Status {
	if x != nil {
		return x.Status
	}
	return nil
}

func (x *RedisProxyExternalAuthResponse) GetExpiration() *timestamppb.Timestamp {
	if x != nil {
		return x.Expiration
	}
	return nil
}

func (x *RedisProxyExternalAuthResponse) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

var File_envoy_service_redis_auth_v3_redis_external_auth_proto protoreflect.FileDescriptor

const file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDesc = "" +
	"\n" +
	"5envoy/service/redis_auth/v3/redis_external_auth.proto\x12\x1benvoy.service.redis_auth.v3\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17google/rpc/status.proto\x1a\x1dudpa/annotations/status.proto\"W\n" +
	"\x1dRedisProxyExternalAuthRequest\x12\x1a\n" +
	"\busername\x18\x01 \x01(\tR\busername\x12\x1a\n" +
	"\bpassword\x18\x02 \x01(\tR\bpassword\"\xa2\x01\n" +
	"\x1eRedisProxyExternalAuthResponse\x12*\n" +
	"\x06status\x18\x01 \x01(\v2\x12.google.rpc.StatusR\x06status\x12:\n" +
	"\n" +
	"expiration\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"expiration\x12\x18\n" +
	"\amessage\x18\x03 \x01(\tR\amessage2\xa4\x01\n" +
	"\x16RedisProxyExternalAuth\x12\x89\x01\n" +
	"\fAuthenticate\x12:.envoy.service.redis_auth.v3.RedisProxyExternalAuthRequest\x1a;.envoy.service.redis_auth.v3.RedisProxyExternalAuthResponse\"\x00B\x9e\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	")io.envoyproxy.envoy.service.redis_auth.v3B\x16RedisExternalAuthProtoP\x01ZOgithub.com/envoyproxy/go-control-plane/envoy/service/redis_auth/v3;redis_authv3b\x06proto3"

var (
	file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescOnce sync.Once
	file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescData []byte
)

func file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescGZIP() []byte {
	file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescOnce.Do(func() {
		file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDesc), len(file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDesc)))
	})
	return file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDescData
}

var file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_service_redis_auth_v3_redis_external_auth_proto_goTypes = []any{
	(*RedisProxyExternalAuthRequest)(nil),  // 0: envoy.service.redis_auth.v3.RedisProxyExternalAuthRequest
	(*RedisProxyExternalAuthResponse)(nil), // 1: envoy.service.redis_auth.v3.RedisProxyExternalAuthResponse
	(*status.Status)(nil),                  // 2: google.rpc.Status
	(*timestamppb.Timestamp)(nil),          // 3: google.protobuf.Timestamp
}
var file_envoy_service_redis_auth_v3_redis_external_auth_proto_depIdxs = []int32{
	2, // 0: envoy.service.redis_auth.v3.RedisProxyExternalAuthResponse.status:type_name -> google.rpc.Status
	3, // 1: envoy.service.redis_auth.v3.RedisProxyExternalAuthResponse.expiration:type_name -> google.protobuf.Timestamp
	0, // 2: envoy.service.redis_auth.v3.RedisProxyExternalAuth.Authenticate:input_type -> envoy.service.redis_auth.v3.RedisProxyExternalAuthRequest
	1, // 3: envoy.service.redis_auth.v3.RedisProxyExternalAuth.Authenticate:output_type -> envoy.service.redis_auth.v3.RedisProxyExternalAuthResponse
	3, // [3:4] is the sub-list for method output_type
	2, // [2:3] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_service_redis_auth_v3_redis_external_auth_proto_init() }
func file_envoy_service_redis_auth_v3_redis_external_auth_proto_init() {
	if File_envoy_service_redis_auth_v3_redis_external_auth_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDesc), len(file_envoy_service_redis_auth_v3_redis_external_auth_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_redis_auth_v3_redis_external_auth_proto_goTypes,
		DependencyIndexes: file_envoy_service_redis_auth_v3_redis_external_auth_proto_depIdxs,
		MessageInfos:      file_envoy_service_redis_auth_v3_redis_external_auth_proto_msgTypes,
	}.Build()
	File_envoy_service_redis_auth_v3_redis_external_auth_proto = out.File
	file_envoy_service_redis_auth_v3_redis_external_auth_proto_goTypes = nil
	file_envoy_service_redis_auth_v3_redis_external_auth_proto_depIdxs = nil
}
