// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/matching/actions/transform_stat/v3/transform_stat.proto

package transform_statv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Transform action for the stat matched by the tag.
type TransformStat struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// If set, the stat will be dropped.
	DropStat *TransformStat_DropStat `protobuf:"bytes,1,opt,name=drop_stat,json=dropStat,proto3" json:"drop_stat,omitempty"`
	// If set, the tag ill be dropped.
	// This removes the tag from the stat entirely.
	DropTag *TransformStat_DropTag `protobuf:"bytes,2,opt,name=drop_tag,json=dropTag,proto3" json:"drop_tag,omitempty"`
	// If set, the tag will be updated.
	UpdateTag     *TransformStat_UpdateTag `protobuf:"bytes,3,opt,name=update_tag,json=updateTag,proto3" json:"update_tag,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TransformStat) Reset() {
	*x = TransformStat{}
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TransformStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TransformStat) ProtoMessage() {}

func (x *TransformStat) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TransformStat.ProtoReflect.Descriptor instead.
func (*TransformStat) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescGZIP(), []int{0}
}

func (x *TransformStat) GetDropStat() *TransformStat_DropStat {
	if x != nil {
		return x.DropStat
	}
	return nil
}

func (x *TransformStat) GetDropTag() *TransformStat_DropTag {
	if x != nil {
		return x.DropTag
	}
	return nil
}

func (x *TransformStat) GetUpdateTag() *TransformStat_UpdateTag {
	if x != nil {
		return x.UpdateTag
	}
	return nil
}

// Action that drops the stat.
type TransformStat_DropStat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TransformStat_DropStat) Reset() {
	*x = TransformStat_DropStat{}
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TransformStat_DropStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TransformStat_DropStat) ProtoMessage() {}

func (x *TransformStat_DropStat) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TransformStat_DropStat.ProtoReflect.Descriptor instead.
func (*TransformStat_DropStat) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescGZIP(), []int{0, 0}
}

// Action that drops the tag.
// This removes the tag from the stat entirely. This is different from updating the
// tag to an empty value, which keeps the tag key with an empty value.
type TransformStat_DropTag struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TransformStat_DropTag) Reset() {
	*x = TransformStat_DropTag{}
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TransformStat_DropTag) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TransformStat_DropTag) ProtoMessage() {}

func (x *TransformStat_DropTag) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TransformStat_DropTag.ProtoReflect.Descriptor instead.
func (*TransformStat_DropTag) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescGZIP(), []int{0, 1}
}

// Action that updates the tag.
type TransformStat_UpdateTag struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The new tag value.
	NewTagValue   string `protobuf:"bytes,2,opt,name=new_tag_value,json=newTagValue,proto3" json:"new_tag_value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TransformStat_UpdateTag) Reset() {
	*x = TransformStat_UpdateTag{}
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TransformStat_UpdateTag) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TransformStat_UpdateTag) ProtoMessage() {}

func (x *TransformStat_UpdateTag) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TransformStat_UpdateTag.ProtoReflect.Descriptor instead.
func (*TransformStat_UpdateTag) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescGZIP(), []int{0, 2}
}

func (x *TransformStat_UpdateTag) GetNewTagValue() string {
	if x != nil {
		return x.NewTagValue
	}
	return ""
}

var File_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto protoreflect.FileDescriptor

const file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDesc = "" +
	"\n" +
	"Henvoy/extensions/matching/actions/transform_stat/v3/transform_stat.proto\x123envoy.extensions.matching.actions.transform_stat.v3\x1a\x1dudpa/annotations/status.proto\"\x95\x03\n" +
	"\rTransformStat\x12h\n" +
	"\tdrop_stat\x18\x01 \x01(\v2K.envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropStatR\bdropStat\x12e\n" +
	"\bdrop_tag\x18\x02 \x01(\v2J.envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropTagR\adropTag\x12k\n" +
	"\n" +
	"update_tag\x18\x03 \x01(\v2L.envoy.extensions.matching.actions.transform_stat.v3.TransformStat.UpdateTagR\tupdateTag\x1a\n" +
	"\n" +
	"\bDropStat\x1a\t\n" +
	"\aDropTag\x1a/\n" +
	"\tUpdateTag\x12\"\n" +
	"\rnew_tag_value\x18\x02 \x01(\tR\vnewTagValueB\xce\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Aio.envoyproxy.envoy.extensions.matching.actions.transform_stat.v3B\x12TransformStatProtoP\x01Zkgithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/actions/transform_stat/v3;transform_statv3b\x06proto3"

var (
	file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescOnce sync.Once
	file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescData []byte
)

func file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescGZIP() []byte {
	file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDesc), len(file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDesc)))
	})
	return file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDescData
}

var file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_goTypes = []any{
	(*TransformStat)(nil),           // 0: envoy.extensions.matching.actions.transform_stat.v3.TransformStat
	(*TransformStat_DropStat)(nil),  // 1: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropStat
	(*TransformStat_DropTag)(nil),   // 2: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropTag
	(*TransformStat_UpdateTag)(nil), // 3: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.UpdateTag
}
var file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.drop_stat:type_name -> envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropStat
	2, // 1: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.drop_tag:type_name -> envoy.extensions.matching.actions.transform_stat.v3.TransformStat.DropTag
	3, // 2: envoy.extensions.matching.actions.transform_stat.v3.TransformStat.update_tag:type_name -> envoy.extensions.matching.actions.transform_stat.v3.TransformStat.UpdateTag
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_init() }
func file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_init() {
	if File_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDesc), len(file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_msgTypes,
	}.Build()
	File_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto = out.File
	file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_goTypes = nil
	file_envoy_extensions_matching_actions_transform_stat_v3_transform_stat_proto_depIdxs = nil
}
