// Code generated by protoc-gen-deepcopy. DO NOT EDIT.
package v1alpha3

import (
	proto "google.golang.org/protobuf/proto"
)

// DeepCopyInto supports using Sidecar within kubernetes types, where deepcopy-gen is used.
func (in *Sidecar) DeepCopyInto(out *Sidecar) {
	p := proto.Clone(in).(*Sidecar)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Sidecar. Required by controller-gen.
func (in *Sidecar) DeepCopy() *Sidecar {
	if in == nil {
		return nil
	}
	out := new(Sidecar)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new Sidecar. Required by controller-gen.
func (in *Sidecar) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using IstioIngressListener within kubernetes types, where deepcopy-gen is used.
func (in *IstioIngressListener) DeepCopyInto(out *IstioIngressListener) {
	p := proto.Clone(in).(*IstioIngressListener)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new IstioIngressListener. Required by controller-gen.
func (in *IstioIngressListener) DeepCopy() *IstioIngressListener {
	if in == nil {
		return nil
	}
	out := new(IstioIngressListener)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new IstioIngressListener. Required by controller-gen.
func (in *IstioIngressListener) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using IstioEgressListener within kubernetes types, where deepcopy-gen is used.
func (in *IstioEgressListener) DeepCopyInto(out *IstioEgressListener) {
	p := proto.Clone(in).(*IstioEgressListener)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new IstioEgressListener. Required by controller-gen.
func (in *IstioEgressListener) DeepCopy() *IstioEgressListener {
	if in == nil {
		return nil
	}
	out := new(IstioEgressListener)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new IstioEgressListener. Required by controller-gen.
func (in *IstioEgressListener) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using WorkloadSelector within kubernetes types, where deepcopy-gen is used.
func (in *WorkloadSelector) DeepCopyInto(out *WorkloadSelector) {
	p := proto.Clone(in).(*WorkloadSelector)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new WorkloadSelector. Required by controller-gen.
func (in *WorkloadSelector) DeepCopy() *WorkloadSelector {
	if in == nil {
		return nil
	}
	out := new(WorkloadSelector)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new WorkloadSelector. Required by controller-gen.
func (in *WorkloadSelector) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using OutboundTrafficPolicy within kubernetes types, where deepcopy-gen is used.
func (in *OutboundTrafficPolicy) DeepCopyInto(out *OutboundTrafficPolicy) {
	p := proto.Clone(in).(*OutboundTrafficPolicy)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new OutboundTrafficPolicy. Required by controller-gen.
func (in *OutboundTrafficPolicy) DeepCopy() *OutboundTrafficPolicy {
	if in == nil {
		return nil
	}
	out := new(OutboundTrafficPolicy)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new OutboundTrafficPolicy. Required by controller-gen.
func (in *OutboundTrafficPolicy) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}

// DeepCopyInto supports using SidecarPort within kubernetes types, where deepcopy-gen is used.
func (in *SidecarPort) DeepCopyInto(out *SidecarPort) {
	p := proto.Clone(in).(*SidecarPort)
	*out = *p
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SidecarPort. Required by controller-gen.
func (in *SidecarPort) DeepCopy() *SidecarPort {
	if in == nil {
		return nil
	}
	out := new(SidecarPort)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInterface is an autogenerated deepcopy function, copying the receiver, creating a new SidecarPort. Required by controller-gen.
func (in *SidecarPort) DeepCopyInterface() interface{} {
	return in.DeepCopy()
}
