// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha3

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for WorkloadEntry
func (this *WorkloadEntry) MarshalJSON() ([]byte, error) {
	str, err := WorkloadEntryMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for WorkloadEntry
func (this *WorkloadEntry) UnmarshalJSON(b []byte) error {
	return WorkloadEntryUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	WorkloadEntryMarshaler   = &jsonpb.Marshaler{}
	WorkloadEntryUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
