/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// RouteGroupKindApplyConfiguration represents a declarative configuration of the RouteGroupKind type for use
// with apply.
//
// RouteGroupKind indicates the group and kind of a Route resource.
type RouteGroupKindApplyConfiguration struct {
	// Group is the group of the Route.
	Group *apisv1.Group `json:"group,omitempty"`
	// Kind is the kind of the Route.
	Kind *apisv1.Kind `json:"kind,omitempty"`
}

// RouteGroupKindApplyConfiguration constructs a declarative configuration of the RouteGroupKind type for use with
// apply.
func RouteGroupKind() *RouteGroupKindApplyConfiguration {
	return &RouteGroupKindApplyConfiguration{}
}

// WithGroup sets the Group field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Group field is set to the value of the last call.
func (b *RouteGroupKindApplyConfiguration) WithGroup(value apisv1.Group) *RouteGroupKindApplyConfiguration {
	b.Group = &value
	return b
}

// WithKind sets the Kind field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Kind field is set to the value of the last call.
func (b *RouteGroupKindApplyConfiguration) WithKind(value apisv1.Kind) *RouteGroupKindApplyConfiguration {
	b.Kind = &value
	return b
}
