// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/mcp_json_rest_bridge/v3/mcp_json_rest_bridge.proto

package mcp_json_rest_bridgev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the MCP MCP JSON REST Bridge.
//
// This extension translates Model Context Protocol (MCP) JSON-RPC requests into standard JSON-REST
// HTTP requests. This enables existing REST backends to function as MCP servers without native MCP
// support.
//
// Main functionalities:
//
//  1. Transcoding: Converts JSON-RPC request payload to HTTP REST request, and maps JSON response
//     back to JSON-RPC.
//  2. Session negotiation: Handles MCP connection prerequisites.
//
// The core logic transforms "tools/call" request into HTTP request following the “HttpRule“
// specification.
//
// Example 1: GET request with path and query parameters
//
// .. code-block:: text
//
//	tools: {
//	  name: "getResource"
//	  http_rule: {
//	    get: "/v1/projects/{project_id}/resources/{resource_id}"
//	    // body is omitted for GET
//	  }
//	}
//	If tools/call params are:
//	{ "name": "getResource", "arguments": {"project_id": "foo", "resource_id": "res-789", "view": "FULL"} }
//	Translation:
//	- Method: GET
//	- URL: /v1/projects/foo/resources/res-789?view=FULL
//	  (Arguments not matching path templates become query parameters.)
//
// Example 2: POST request with wildcard body
//
// .. code-block:: text
//
//	tools: {
//	  name: "createResource"
//	  http_rule: {
//	    post: "/v1/projects/{project_id}/resources"
//	    body: "*"
//	  }
//	}
//	If tools/call params are:
//	{ "name": "createResource", "arguments": {"project_id": "foo", "resource_id": "res-456", "payload": { "data": "some value" }} }
//	Translation:
//	- Method: POST
//	- URL: /v1/projects/foo/resources
//	- Body: {"resource_id": "res-456", "payload": { "data": "some value" }}
//	  (Arguments not used in the path form the body, as per body: "*".)
//
// Example 3: PUT request with a specific field as body
//
// .. code-block:: text
//
//	tools: {
//	  name: "updateResource"
//	  http_rule: {
//	    put: "/v1/projects/{project_id}"
//	    body: "payload"
//	  }
//	}
//	If tools/call params are:
//	{ "name": "updateResource", "arguments": {"project_id": "foo", "resource_id": "res-456", "payload": { "data": "updated value" }} }
//	Translation:
//	- Method: PUT
//	- URL: /v1/projects/foo?resource_id=res-456
//	- Body: {"data": "updated value"}
//	  (Only the "payload" field from arguments is used as the body. Other arguments not in the
//	  path, like 'resource_id', become query parameters.)
type McpJsonRestBridge struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// General server information.
	ServerInfo *ServerInfo `protobuf:"bytes,1,opt,name=server_info,json=serverInfo,proto3" json:"server_info,omitempty"`
	// Configuration for the MCP tools.
	ToolConfig    *ServerToolConfig `protobuf:"bytes,2,opt,name=tool_config,json=toolConfig,proto3" json:"tool_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *McpJsonRestBridge) Reset() {
	*x = McpJsonRestBridge{}
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *McpJsonRestBridge) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*McpJsonRestBridge) ProtoMessage() {}

func (x *McpJsonRestBridge) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use McpJsonRestBridge.ProtoReflect.Descriptor instead.
func (*McpJsonRestBridge) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP(), []int{0}
}

func (x *McpJsonRestBridge) GetServerInfo() *ServerInfo {
	if x != nil {
		return x.ServerInfo
	}
	return nil
}

func (x *McpJsonRestBridge) GetToolConfig() *ServerToolConfig {
	if x != nil {
		return x.ToolConfig
	}
	return nil
}

// Configuration for the server metadata.
type ServerInfo struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// [#not-implemented-hide:]
	// [#comment:TODO(guoyilin42): Implement supported_protocol_versions]
	// Lists the MCP protocol versions supported by this MCP endpoint.
	//
	//   - If provided: The extension enforces version negotiation according to the MCP specification:
	//     https://modelcontextprotocol.io/specification/2025-11-25/basic/lifecycle#version-negotiation
	//   - If not provided: The extension accepts any version sent by the client during negotiation and
	//     skips validation of the mcp-protocol-version header on subsequent requests.
	//
	// Example values: ["2025-11-25", "2025-06-18"]
	SupportedProtocolVersions []string `protobuf:"bytes,1,rep,name=supported_protocol_versions,json=supportedProtocolVersions,proto3" json:"supported_protocol_versions,omitempty"`
	// [#not-implemented-hide:]
	// [#comment:TODO(guoyilin42): Implement description]
	// Optional description of the server.
	Description string `protobuf:"bytes,2,opt,name=description,proto3" json:"description,omitempty"`
	// The fallback protocol version to use if the client does not provide the “mcp-protocol-version“ header.
	//
	//   - If provided: The extension uses this version as the fallback protocol version.
	//   - If not provided: The extension uses the fallback protocol version defined in the latest MCP
	//     specification. For example, the current latest 2025-11-25 specification designates "2025-03-26"
	//     as the fallback protocol version.
	//     See https://modelcontextprotocol.io/specification/2025-11-25/basic/transports#protocol-version-header
	FallbackProtocolVersion *wrapperspb.StringValue `protobuf:"bytes,3,opt,name=fallback_protocol_version,json=fallbackProtocolVersion,proto3" json:"fallback_protocol_version,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *ServerInfo) Reset() {
	*x = ServerInfo{}
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServerInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServerInfo) ProtoMessage() {}

func (x *ServerInfo) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServerInfo.ProtoReflect.Descriptor instead.
func (*ServerInfo) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP(), []int{1}
}

func (x *ServerInfo) GetSupportedProtocolVersions() []string {
	if x != nil {
		return x.SupportedProtocolVersions
	}
	return nil
}

func (x *ServerInfo) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *ServerInfo) GetFallbackProtocolVersion() *wrapperspb.StringValue {
	if x != nil {
		return x.FallbackProtocolVersion
	}
	return nil
}

// Configuration for the MCP tool capability of the server.
type ServerToolConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of MCP tools configurations.
	Tools []*ToolConfig `protobuf:"bytes,1,rep,name=tools,proto3" json:"tools,omitempty"`
	// [#not-implemented-hide:]
	// [#comment:TODO(guoyilin42): Implement list_changed]
	// Whether this server supports notifications for changes to the tool list.
	ListChanged bool `protobuf:"varint,2,opt,name=list_changed,json=listChanged,proto3" json:"list_changed,omitempty"`
	// Optional configuration to transcode the tools/list requests to a standard HTTP request.
	//
	// Note: tools/list should be mapped to a GET request with an empty body.
	//
	//   - If provided: The extension transcodes the request and forwards it down the filter chain.
	//     The response (whether from an upstream backend, a configured “direct_response“, or another
	//     extension) MUST be a JSON body strictly matching the MCP “ListToolsResult“ schema.
	//     Ref: https://modelcontextprotocol.io/specification/2025-11-25/schema#listtoolsresult
	//   - If not provided: The “tools/list“ request is passed through. This allows subsequent
	//     extension or the backend itself to handle the tools/list request if they support it.
	ToolListHttpRule *HttpRule `protobuf:"bytes,3,opt,name=tool_list_http_rule,json=toolListHttpRule,proto3" json:"tool_list_http_rule,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *ServerToolConfig) Reset() {
	*x = ServerToolConfig{}
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServerToolConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServerToolConfig) ProtoMessage() {}

func (x *ServerToolConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServerToolConfig.ProtoReflect.Descriptor instead.
func (*ServerToolConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP(), []int{2}
}

func (x *ServerToolConfig) GetTools() []*ToolConfig {
	if x != nil {
		return x.Tools
	}
	return nil
}

func (x *ServerToolConfig) GetListChanged() bool {
	if x != nil {
		return x.ListChanged
	}
	return false
}

func (x *ServerToolConfig) GetToolListHttpRule() *HttpRule {
	if x != nil {
		return x.ToolListHttpRule
	}
	return nil
}

// Configuration for a specific MCP tool.
type ToolConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the tool.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The HTTP configuration rules that apply to the normal backend.
	HttpRule      *HttpRule `protobuf:"bytes,2,opt,name=http_rule,json=httpRule,proto3" json:"http_rule,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ToolConfig) Reset() {
	*x = ToolConfig{}
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ToolConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ToolConfig) ProtoMessage() {}

func (x *ToolConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ToolConfig.ProtoReflect.Descriptor instead.
func (*ToolConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP(), []int{3}
}

func (x *ToolConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ToolConfig) GetHttpRule() *HttpRule {
	if x != nil {
		return x.HttpRule
	}
	return nil
}

// Defines the schema of the JSON-RPC to REST mapping. It specifies how the "arguments"
// in a tools/call request are mapped to the URL path, query parameters, and HTTP request body.
//
// Mapping Rules:
//
//  1. Path: Fields defined in the path template (e.g., “/v1/resources/{id}“) are extracted from
//     arguments and placed in the URL.
//  2. Body: Determined by the “body“ field.
//     - If "*": All arguments not used in the path become the HTTP JSON body.
//     - If specify a field: Only that specific argument becomes the HTTP JSON body.
//     - If empty: No body is sent.
//  3. Query: Any leaf arguments not mapped to Path or Body are added as URL query parameters.
//
// [#next-free-field: 7]
type HttpRule struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Determines the HTTP method and the URL path template.
	//
	// Path templating uses curly braces “{}“ to mark a section of the URL path as replaceable.
	// Each template variable MUST correspond to a field in the JSON-RPC "arguments".
	// Use dot-notation to access fields within nested objects (e.g., "user.id" maps the value of the
	// "id" field inside "user").
	//
	// To support backward compatibility with future methods, these are defined as individual fields
	// rather than a "oneof". If multiple fields are present, the one with the highest field number
	// highest priority) is the effective method.
	//
	// Maps to HTTP GET.
	Get string `protobuf:"bytes,1,opt,name=get,proto3" json:"get,omitempty"`
	// Maps to HTTP PUT.
	Put string `protobuf:"bytes,2,opt,name=put,proto3" json:"put,omitempty"`
	// Maps to HTTP POST.
	Post string `protobuf:"bytes,3,opt,name=post,proto3" json:"post,omitempty"`
	// Maps to HTTP DELETE.
	Delete string `protobuf:"bytes,4,opt,name=delete,proto3" json:"delete,omitempty"`
	// Maps to HTTP PATCH.
	Patch string `protobuf:"bytes,5,opt,name=patch,proto3" json:"patch,omitempty"`
	// The name of the request field whose value is mapped to the HTTP request body.
	//
	//   - If "*": All fields not bound by the path template are mapped to the request body.
	//   - If specify a field: This specific field is mapped to the body. Uses dot-notation for nested
	//     fields (e.g., "user.data" maps the value of the "data" field inside "user").
	//   - If omitted: There is no HTTP request body; fields not in the path become query parameters.
	Body          string `protobuf:"bytes,6,opt,name=body,proto3" json:"body,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpRule) Reset() {
	*x = HttpRule{}
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpRule) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpRule) ProtoMessage() {}

func (x *HttpRule) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpRule.ProtoReflect.Descriptor instead.
func (*HttpRule) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP(), []int{4}
}

func (x *HttpRule) GetGet() string {
	if x != nil {
		return x.Get
	}
	return ""
}

func (x *HttpRule) GetPut() string {
	if x != nil {
		return x.Put
	}
	return ""
}

func (x *HttpRule) GetPost() string {
	if x != nil {
		return x.Post
	}
	return ""
}

func (x *HttpRule) GetDelete() string {
	if x != nil {
		return x.Delete
	}
	return ""
}

func (x *HttpRule) GetPatch() string {
	if x != nil {
		return x.Patch
	}
	return ""
}

func (x *HttpRule) GetBody() string {
	if x != nil {
		return x.Body
	}
	return ""
}

var File_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDesc = "" +
	"\n" +
	"Penvoy/extensions/filters/http/mcp_json_rest_bridge/v3/mcp_json_rest_bridge.proto\x125envoy.extensions.filters.http.mcp_json_rest_bridge.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe1\x01\n" +
	"\x11McpJsonRestBridge\x12b\n" +
	"\vserver_info\x18\x01 \x01(\v2A.envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerInfoR\n" +
	"serverInfo\x12h\n" +
	"\vtool_config\x18\x02 \x01(\v2G.envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerToolConfigR\n" +
	"toolConfig\"\xc8\x01\n" +
	"\n" +
	"ServerInfo\x12>\n" +
	"\x1bsupported_protocol_versions\x18\x01 \x03(\tR\x19supportedProtocolVersions\x12 \n" +
	"\vdescription\x18\x02 \x01(\tR\vdescription\x12X\n" +
	"\x19fallback_protocol_version\x18\x03 \x01(\v2\x1c.google.protobuf.StringValueR\x17fallbackProtocolVersion\"\xfe\x01\n" +
	"\x10ServerToolConfig\x12W\n" +
	"\x05tools\x18\x01 \x03(\v2A.envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ToolConfigR\x05tools\x12!\n" +
	"\flist_changed\x18\x02 \x01(\bR\vlistChanged\x12n\n" +
	"\x13tool_list_http_rule\x18\x03 \x01(\v2?.envoy.extensions.filters.http.mcp_json_rest_bridge.v3.HttpRuleR\x10toolListHttpRule\"\x87\x01\n" +
	"\n" +
	"ToolConfig\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12\\\n" +
	"\thttp_rule\x18\x02 \x01(\v2?.envoy.extensions.filters.http.mcp_json_rest_bridge.v3.HttpRuleR\bhttpRule\"\x84\x01\n" +
	"\bHttpRule\x12\x10\n" +
	"\x03get\x18\x01 \x01(\tR\x03get\x12\x10\n" +
	"\x03put\x18\x02 \x01(\tR\x03put\x12\x12\n" +
	"\x04post\x18\x03 \x01(\tR\x04post\x12\x16\n" +
	"\x06delete\x18\x04 \x01(\tR\x06delete\x12\x14\n" +
	"\x05patch\x18\x05 \x01(\tR\x05patch\x12\x12\n" +
	"\x04body\x18\x06 \x01(\tR\x04bodyB\xe4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"Cio.envoyproxy.envoy.extensions.filters.http.mcp_json_rest_bridge.v3B\x16McpJsonRestBridgeProtoP\x01Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_json_rest_bridge/v3;mcp_json_rest_bridgev3b\x06proto3"

var (
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDesc), len(file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDescData
}

var file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_goTypes = []any{
	(*McpJsonRestBridge)(nil),      // 0: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.McpJsonRestBridge
	(*ServerInfo)(nil),             // 1: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerInfo
	(*ServerToolConfig)(nil),       // 2: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerToolConfig
	(*ToolConfig)(nil),             // 3: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ToolConfig
	(*HttpRule)(nil),               // 4: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.HttpRule
	(*wrapperspb.StringValue)(nil), // 5: google.protobuf.StringValue
}
var file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.McpJsonRestBridge.server_info:type_name -> envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerInfo
	2, // 1: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.McpJsonRestBridge.tool_config:type_name -> envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerToolConfig
	5, // 2: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerInfo.fallback_protocol_version:type_name -> google.protobuf.StringValue
	3, // 3: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerToolConfig.tools:type_name -> envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ToolConfig
	4, // 4: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ServerToolConfig.tool_list_http_rule:type_name -> envoy.extensions.filters.http.mcp_json_rest_bridge.v3.HttpRule
	4, // 5: envoy.extensions.filters.http.mcp_json_rest_bridge.v3.ToolConfig.http_rule:type_name -> envoy.extensions.filters.http.mcp_json_rest_bridge.v3.HttpRule
	6, // [6:6] is the sub-list for method output_type
	6, // [6:6] is the sub-list for method input_type
	6, // [6:6] is the sub-list for extension type_name
	6, // [6:6] is the sub-list for extension extendee
	0, // [0:6] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_init()
}
func file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_init() {
	if File_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDesc), len(file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto = out.File
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_goTypes = nil
	file_envoy_extensions_filters_http_mcp_json_rest_bridge_v3_mcp_json_rest_bridge_proto_depIdxs = nil
}
