// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/health_checkers/redis/v3/redis.proto

package redisv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/redis_proxy/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Redis struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// If set, optionally perform “EXISTS <key>“ instead of “PING“. A return value
	// from Redis of 0 (does not exist) is considered a passing healthcheck. A return value other
	// than 0 is considered a failure. This allows the user to mark a Redis instance for maintenance
	// by setting the specified key to any value and waiting for traffic to drain.
	Key string `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	// Use AWS IAM for health checker authentication
	AwsIam        *v3.AwsIam `protobuf:"bytes,2,opt,name=aws_iam,json=awsIam,proto3" json:"aws_iam,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Redis) Reset() {
	*x = Redis{}
	mi := &file_envoy_extensions_health_checkers_redis_v3_redis_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Redis) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Redis) ProtoMessage() {}

func (x *Redis) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_health_checkers_redis_v3_redis_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Redis.ProtoReflect.Descriptor instead.
func (*Redis) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescGZIP(), []int{0}
}

func (x *Redis) GetKey() string {
	if x != nil {
		return x.Key
	}
	return ""
}

func (x *Redis) GetAwsIam() *v3.AwsIam {
	if x != nil {
		return x.AwsIam
	}
	return nil
}

var File_envoy_extensions_health_checkers_redis_v3_redis_proto protoreflect.FileDescriptor

const file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDesc = "" +
	"\n" +
	"5envoy/extensions/health_checkers/redis/v3/redis.proto\x12)envoy.extensions.health_checkers.redis.v3\x1aAenvoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x9e\x01\n" +
	"\x05Redis\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12P\n" +
	"\aaws_iam\x18\x02 \x01(\v27.envoy.extensions.filters.network.redis_proxy.v3.AwsIamR\x06awsIam:1\x9aň\x1e,\n" +
	"*envoy.config.health_checker.redis.v2.RedisB\xa9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"7io.envoyproxy.envoy.extensions.health_checkers.redis.v3B\n" +
	"RedisProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/extensions/health_checkers/redis/v3;redisv3b\x06proto3"

var (
	file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescOnce sync.Once
	file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescData []byte
)

func file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescGZIP() []byte {
	file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDesc), len(file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDesc)))
	})
	return file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDescData
}

var file_envoy_extensions_health_checkers_redis_v3_redis_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_health_checkers_redis_v3_redis_proto_goTypes = []any{
	(*Redis)(nil),     // 0: envoy.extensions.health_checkers.redis.v3.Redis
	(*v3.AwsIam)(nil), // 1: envoy.extensions.filters.network.redis_proxy.v3.AwsIam
}
var file_envoy_extensions_health_checkers_redis_v3_redis_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.health_checkers.redis.v3.Redis.aws_iam:type_name -> envoy.extensions.filters.network.redis_proxy.v3.AwsIam
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_health_checkers_redis_v3_redis_proto_init() }
func file_envoy_extensions_health_checkers_redis_v3_redis_proto_init() {
	if File_envoy_extensions_health_checkers_redis_v3_redis_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDesc), len(file_envoy_extensions_health_checkers_redis_v3_redis_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_health_checkers_redis_v3_redis_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_health_checkers_redis_v3_redis_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_health_checkers_redis_v3_redis_proto_msgTypes,
	}.Build()
	File_envoy_extensions_health_checkers_redis_v3_redis_proto = out.File
	file_envoy_extensions_health_checkers_redis_v3_redis_proto_goTypes = nil
	file_envoy_extensions_health_checkers_redis_v3_redis_proto_depIdxs = nil
}
