// Code generated by protoc-gen-jsonshim. DO NOT EDIT.
package v1alpha1

import (
	bytes "bytes"
	jsonpb "github.com/golang/protobuf/jsonpb"
)

// MarshalJSON is a custom marshaler for Resource
func (this *Resource) MarshalJSON() ([]byte, error) {
	str, err := ResourceMarshaler.MarshalToString(this)
	return []byte(str), err
}

// UnmarshalJSON is a custom unmarshaler for Resource
func (this *Resource) UnmarshalJSON(b []byte) error {
	return ResourceUnmarshaler.Unmarshal(bytes.NewReader(b), this)
}

var (
	ResourceMarshaler   = &jsonpb.Marshaler{}
	ResourceUnmarshaler = &jsonpb.Unmarshaler{AllowUnknownFields: true}
)
