// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/alts/v3/alts.proto

package altsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for ALTS transport socket. This provides Google's ALTS protocol to Envoy.
// Store the peer identity in dynamic metadata, namespace is "envoy.transport_socket.peer_information", key is "peer_identity".
// https://cloud.google.com/security/encryption-in-transit/application-layer-transport-security/
type Alts struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The location of a handshaker service, this is usually 169.254.169.254:8080
	// on GCE.
	HandshakerService string `protobuf:"bytes,1,opt,name=handshaker_service,json=handshakerService,proto3" json:"handshaker_service,omitempty"`
	// The acceptable service accounts from peer, peers not in the list will be rejected in the
	// handshake validation step. If empty, no validation will be performed.
	PeerServiceAccounts []string `protobuf:"bytes,2,rep,name=peer_service_accounts,json=peerServiceAccounts,proto3" json:"peer_service_accounts,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *Alts) Reset() {
	*x = Alts{}
	mi := &file_envoy_extensions_transport_sockets_alts_v3_alts_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Alts) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Alts) ProtoMessage() {}

func (x *Alts) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_alts_v3_alts_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Alts.ProtoReflect.Descriptor instead.
func (*Alts) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescGZIP(), []int{0}
}

func (x *Alts) GetHandshakerService() string {
	if x != nil {
		return x.HandshakerService
	}
	return ""
}

func (x *Alts) GetPeerServiceAccounts() []string {
	if x != nil {
		return x.PeerServiceAccounts
	}
	return nil
}

var File_envoy_extensions_transport_sockets_alts_v3_alts_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDesc = "" +
	"\n" +
	"5envoy/extensions/transport_sockets/alts/v3/alts.proto\x12*envoy.extensions.transport_sockets.alts.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xaa\x01\n" +
	"\x04Alts\x126\n" +
	"\x12handshaker_service\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x11handshakerService\x122\n" +
	"\x15peer_service_accounts\x18\x02 \x03(\tR\x13peerServiceAccounts:6\x9aň\x1e1\n" +
	"/envoy.config.transport_socket.alts.v2alpha.AltsB\xa9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"8io.envoyproxy.envoy.extensions.transport_sockets.alts.v3B\tAltsProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/alts/v3;altsv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDesc), len(file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_alts_v3_alts_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_transport_sockets_alts_v3_alts_proto_goTypes = []any{
	(*Alts)(nil), // 0: envoy.extensions.transport_sockets.alts.v3.Alts
}
var file_envoy_extensions_transport_sockets_alts_v3_alts_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_transport_sockets_alts_v3_alts_proto_init() }
func file_envoy_extensions_transport_sockets_alts_v3_alts_proto_init() {
	if File_envoy_extensions_transport_sockets_alts_v3_alts_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDesc), len(file_envoy_extensions_transport_sockets_alts_v3_alts_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_alts_v3_alts_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_alts_v3_alts_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_alts_v3_alts_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_alts_v3_alts_proto = out.File
	file_envoy_extensions_transport_sockets_alts_v3_alts_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_alts_v3_alts_proto_depIdxs = nil
}
