// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/clusters/mcp_multicluster/v3/cluster.proto

package mcp_multiclusterv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the MCP multi cluster. See the :ref:`architecture overview
// <arch_overview_mcp_multicluster>` for more information. This cluster type allows aggregation of
// multiple clusters into one, providing metadata with the list of aggregated clusters.
// Use the “attemptCount“ property of the request “StreamInfo“ object to select host in a specific subcluster.
// If the “attemptCount“ value is greater than the number of aggregated clusters, the host selection will fail.
//
// The primary purpose of this cluster extension is to provide the list of servers for the MCP router
// configured for tool and resource aggregation. For details of how tools and resource are aggregated see
// :ref:`MCP router documentation<config_http_filters_mcp_router>`.
//
// Example configuration:
//
// .. code-block:: yaml
//
//	name: mcp_multicluster
//	connect_timeout: 0.25s
//	lb_policy: CLUSTER_PROVIDED
//	cluster_type:
//	  name: envoy.clusters.mcp_multicluster
//	  typed_config:
//	    "@type": type.googleapis.com/envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig
//	    servers:
//	    - name: build_tools
//	      mcp_cluster:
//	        cluster: build_tools
//	    - name: review_tools
//	       mcp_cluster:
//	         cluster: review_tools
//	         host_rewrite_literal: "mcp.review_tools.acme.com"
//
// [#extension: envoy.clusters.mcp_multicluster]
type ClusterConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A list of remote MCP servers.
	// Based on the MCP multi cluster configuration the MCP router aggregates capabilities, tools and resources from remote MCP servers
	// and presents itself as single MCP server to the client. All remote MCP servers are sent the same capabilities
	// that the client presented to Envoy. MCP router prefixes tool names and resource path with the server name to resolve
	// naming collisions.
	Servers       []*ClusterConfig_McpBackend `protobuf:"bytes,1,rep,name=servers,proto3" json:"servers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterConfig) Reset() {
	*x = ClusterConfig{}
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig) ProtoMessage() {}

func (x *ClusterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig.ProtoReflect.Descriptor instead.
func (*ClusterConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescGZIP(), []int{0}
}

func (x *ClusterConfig) GetServers() []*ClusterConfig_McpBackend {
	if x != nil {
		return x.Servers
	}
	return nil
}

// Cluster-based backend configuration.
type ClusterConfig_McpCluster struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Cluster name to route requests to.
	Cluster string `protobuf:"bytes,1,opt,name=cluster,proto3" json:"cluster,omitempty"`
	// Path to use for MCP requests. Defaults to "/mcp".
	Path string `protobuf:"bytes,2,opt,name=path,proto3" json:"path,omitempty"`
	// Request timeout.
	// If not set, uses cluster's timeout configuration.
	Timeout *durationpb.Duration `protobuf:"bytes,3,opt,name=timeout,proto3" json:"timeout,omitempty"`
	// Indicates that during forwarding, the host header will be swapped with
	// this value.
	HostRewriteLiteral string `protobuf:"bytes,4,opt,name=host_rewrite_literal,json=hostRewriteLiteral,proto3" json:"host_rewrite_literal,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *ClusterConfig_McpCluster) Reset() {
	*x = ClusterConfig_McpCluster{}
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig_McpCluster) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig_McpCluster) ProtoMessage() {}

func (x *ClusterConfig_McpCluster) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig_McpCluster.ProtoReflect.Descriptor instead.
func (*ClusterConfig_McpCluster) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescGZIP(), []int{0, 0}
}

func (x *ClusterConfig_McpCluster) GetCluster() string {
	if x != nil {
		return x.Cluster
	}
	return ""
}

func (x *ClusterConfig_McpCluster) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *ClusterConfig_McpCluster) GetTimeout() *durationpb.Duration {
	if x != nil {
		return x.Timeout
	}
	return nil
}

func (x *ClusterConfig_McpCluster) GetHostRewriteLiteral() string {
	if x != nil {
		return x.HostRewriteLiteral
	}
	return ""
}

// Specification of the MCP server.
type ClusterConfig_McpBackend struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Unique name for this backend. Used for:
	// - Tool name prefixing (e.g., "time__get_current_time")
	// - Session ID composition
	// - Logging and error messages.
	// Default will be the cluster name if not specified.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Backend target specification.
	McpCluster    *ClusterConfig_McpCluster `protobuf:"bytes,2,opt,name=mcp_cluster,json=mcpCluster,proto3" json:"mcp_cluster,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterConfig_McpBackend) Reset() {
	*x = ClusterConfig_McpBackend{}
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig_McpBackend) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig_McpBackend) ProtoMessage() {}

func (x *ClusterConfig_McpBackend) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig_McpBackend.ProtoReflect.Descriptor instead.
func (*ClusterConfig_McpBackend) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescGZIP(), []int{0, 1}
}

func (x *ClusterConfig_McpBackend) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ClusterConfig_McpBackend) GetMcpCluster() *ClusterConfig_McpCluster {
	if x != nil {
		return x.McpCluster
	}
	return nil
}

var File_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto protoreflect.FileDescriptor

const file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDesc = "" +
	"\n" +
	";envoy/extensions/clusters/mcp_multicluster/v3/cluster.proto\x12-envoy.extensions.clusters.mcp_multicluster.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb6\x03\n" +
	"\rClusterConfig\x12k\n" +
	"\aservers\x18\x01 \x03(\v2G.envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpBackendB\b\xfaB\x05\x92\x01\x02\b\x01R\aservers\x1a\xaa\x01\n" +
	"\n" +
	"McpCluster\x12!\n" +
	"\acluster\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\acluster\x12\x12\n" +
	"\x04path\x18\x02 \x01(\tR\x04path\x123\n" +
	"\atimeout\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\atimeout\x120\n" +
	"\x14host_rewrite_literal\x18\x04 \x01(\tR\x12hostRewriteLiteral\x1a\x8a\x01\n" +
	"\n" +
	"McpBackend\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12h\n" +
	"\vmcp_cluster\x18\x02 \x01(\v2G.envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpClusterR\n" +
	"mcpClusterB\xc6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	";io.envoyproxy.envoy.extensions.clusters.mcp_multicluster.v3B\fClusterProtoP\x01Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/clusters/mcp_multicluster/v3;mcp_multiclusterv3b\x06proto3"

var (
	file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescOnce sync.Once
	file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescData []byte
)

func file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescGZIP() []byte {
	file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDesc)))
	})
	return file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDescData
}

var file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_goTypes = []any{
	(*ClusterConfig)(nil),            // 0: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig
	(*ClusterConfig_McpCluster)(nil), // 1: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpCluster
	(*ClusterConfig_McpBackend)(nil), // 2: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpBackend
	(*durationpb.Duration)(nil),      // 3: google.protobuf.Duration
}
var file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.servers:type_name -> envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpBackend
	3, // 1: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpCluster.timeout:type_name -> google.protobuf.Duration
	1, // 2: envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpBackend.mcp_cluster:type_name -> envoy.extensions.clusters.mcp_multicluster.v3.ClusterConfig.McpCluster
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_init() }
func file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_init() {
	if File_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDesc), len(file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_msgTypes,
	}.Build()
	File_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto = out.File
	file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_goTypes = nil
	file_envoy_extensions_clusters_mcp_multicluster_v3_cluster_proto_depIdxs = nil
}
