// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/tracers/dynamic_modules/v3/dynamic_modules.proto

package dynamic_modulesv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/dynamic_modules/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the Dynamic Modules Tracer. This tracer allows loading shared object
// files via “dlopen“ to implement custom distributed tracing backends.
//
// A module can be loaded by multiple tracer configurations; the module is loaded only once
// and shared across multiple tracer instances.
//
// The tracer receives trace context from incoming requests and can inject trace context into
// outgoing requests for propagation. It supports the full span lifecycle: creation, tagging,
// logging, child spans, and reporting.
type DynamicModuleTracer struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies the shared-object level configuration. This field is required.
	DynamicModuleConfig *v3.DynamicModuleConfig `protobuf:"bytes,1,opt,name=dynamic_module_config,json=dynamicModuleConfig,proto3" json:"dynamic_module_config,omitempty"`
	// The name for this tracer configuration. If not specified, defaults to an empty string.
	//
	// This can be used to distinguish between different tracer implementations inside a dynamic
	// module. For example, a module can have completely different tracer implementations (e.g.,
	// Zipkin-compatible, OpenTelemetry-compatible). When Envoy receives this configuration, it
	// passes the “tracer_name“ to the dynamic module's tracer config init function together with
	// the “tracer_config“. That way a module can decide which in-module tracer implementation to
	// use based on the name at load time.
	TracerName string `protobuf:"bytes,2,opt,name=tracer_name,json=tracerName,proto3" json:"tracer_name,omitempty"`
	// The configuration for the tracer chosen by “tracer_name“. If not specified, an empty
	// configuration is passed to the module.
	//
	// This is passed to the module's tracer initialization function. Together with the
	// “tracer_name“, the module can decide which in-module tracer implementation to use and
	// fine-tune the behavior of the tracer.
	//
	// “google.protobuf.Struct“ is serialized as JSON before passing it to the module.
	// “google.protobuf.BytesValue“ and “google.protobuf.StringValue“ are passed directly
	// without the wrapper.
	//
	// .. code-block:: yaml
	//
	//	# Passing a JSON struct configuration
	//	tracer_config:
	//	  "@type": "type.googleapis.com/google.protobuf.Struct"
	//	  value:
	//	    endpoint: "http://tracing-backend:9411/api/v2/spans"
	//	    sample_rate: 0.1
	//
	//	# Passing a simple string configuration
	//	tracer_config:
	//	  "@type": "type.googleapis.com/google.protobuf.StringValue"
	//	  value: "http://tracing-backend:9411"
	TracerConfig  *anypb.Any `protobuf:"bytes,3,opt,name=tracer_config,json=tracerConfig,proto3" json:"tracer_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DynamicModuleTracer) Reset() {
	*x = DynamicModuleTracer{}
	mi := &file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DynamicModuleTracer) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DynamicModuleTracer) ProtoMessage() {}

func (x *DynamicModuleTracer) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DynamicModuleTracer.ProtoReflect.Descriptor instead.
func (*DynamicModuleTracer) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP(), []int{0}
}

func (x *DynamicModuleTracer) GetDynamicModuleConfig() *v3.DynamicModuleConfig {
	if x != nil {
		return x.DynamicModuleConfig
	}
	return nil
}

func (x *DynamicModuleTracer) GetTracerName() string {
	if x != nil {
		return x.TracerName
	}
	return ""
}

func (x *DynamicModuleTracer) GetTracerConfig() *anypb.Any {
	if x != nil {
		return x.TracerConfig
	}
	return nil
}

var File_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto protoreflect.FileDescriptor

const file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDesc = "" +
	"\n" +
	"Aenvoy/extensions/tracers/dynamic_modules/v3/dynamic_modules.proto\x12+envoy.extensions.tracers.dynamic_modules.v3\x1a9envoy/extensions/dynamic_modules/v3/dynamic_modules.proto\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe9\x01\n" +
	"\x13DynamicModuleTracer\x12v\n" +
	"\x15dynamic_module_config\x18\x01 \x01(\v28.envoy.extensions.dynamic_modules.v3.DynamicModuleConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x13dynamicModuleConfig\x12\x1f\n" +
	"\vtracer_name\x18\x02 \x01(\tR\n" +
	"tracerName\x129\n" +
	"\rtracer_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyR\ftracerConfigB\xc0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"9io.envoyproxy.envoy.extensions.tracers.dynamic_modules.v3B\x13DynamicModulesProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/tracers/dynamic_modules/v3;dynamic_modulesv3b\x06proto3"

var (
	file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce sync.Once
	file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescData []byte
)

func file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescGZIP() []byte {
	file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDesc)))
	})
	return file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDescData
}

var file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_goTypes = []any{
	(*DynamicModuleTracer)(nil),    // 0: envoy.extensions.tracers.dynamic_modules.v3.DynamicModuleTracer
	(*v3.DynamicModuleConfig)(nil), // 1: envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	(*anypb.Any)(nil),              // 2: google.protobuf.Any
}
var file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.tracers.dynamic_modules.v3.DynamicModuleTracer.dynamic_module_config:type_name -> envoy.extensions.dynamic_modules.v3.DynamicModuleConfig
	2, // 1: envoy.extensions.tracers.dynamic_modules.v3.DynamicModuleTracer.tracer_config:type_name -> google.protobuf.Any
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_init() }
func file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_init() {
	if File_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDesc), len(file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_msgTypes,
	}.Build()
	File_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto = out.File
	file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_goTypes = nil
	file_envoy_extensions_tracers_dynamic_modules_v3_dynamic_modules_proto_depIdxs = nil
}
