// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/compression/qatzip/compressor/v3alpha/qatzip.proto

package v3alpha

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Qatzip_HardwareBufferSize int32

const (
	Qatzip_DEFAULT Qatzip_HardwareBufferSize = 0
	Qatzip_SZ_4K   Qatzip_HardwareBufferSize = 1
	Qatzip_SZ_8K   Qatzip_HardwareBufferSize = 2
	Qatzip_SZ_32K  Qatzip_HardwareBufferSize = 3
	Qatzip_SZ_64K  Qatzip_HardwareBufferSize = 4
	Qatzip_SZ_128K Qatzip_HardwareBufferSize = 5
	Qatzip_SZ_512K Qatzip_HardwareBufferSize = 6
)

// Enum value maps for Qatzip_HardwareBufferSize.
var (
	Qatzip_HardwareBufferSize_name = map[int32]string{
		0: "DEFAULT",
		1: "SZ_4K",
		2: "SZ_8K",
		3: "SZ_32K",
		4: "SZ_64K",
		5: "SZ_128K",
		6: "SZ_512K",
	}
	Qatzip_HardwareBufferSize_value = map[string]int32{
		"DEFAULT": 0,
		"SZ_4K":   1,
		"SZ_8K":   2,
		"SZ_32K":  3,
		"SZ_64K":  4,
		"SZ_128K": 5,
		"SZ_512K": 6,
	}
)

func (x Qatzip_HardwareBufferSize) Enum() *Qatzip_HardwareBufferSize {
	p := new(Qatzip_HardwareBufferSize)
	*p = x
	return p
}

func (x Qatzip_HardwareBufferSize) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Qatzip_HardwareBufferSize) Descriptor() protoreflect.EnumDescriptor {
	return file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_enumTypes[0].Descriptor()
}

func (Qatzip_HardwareBufferSize) Type() protoreflect.EnumType {
	return &file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_enumTypes[0]
}

func (x Qatzip_HardwareBufferSize) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Qatzip_HardwareBufferSize.Descriptor instead.
func (Qatzip_HardwareBufferSize) EnumDescriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescGZIP(), []int{0, 0}
}

// [#next-free-field: 6]
type Qatzip struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Value from 1 to 9 that controls the main compression speed-density lever.
	// The higher quality, the slower compression. The default value is 1.
	CompressionLevel *wrapperspb.UInt32Value `protobuf:"bytes,1,opt,name=compression_level,json=compressionLevel,proto3" json:"compression_level,omitempty"`
	// A size of qat hardware buffer. This field will be set to "DEFAULT" if not specified.
	HardwareBufferSize Qatzip_HardwareBufferSize `protobuf:"varint,2,opt,name=hardware_buffer_size,json=hardwareBufferSize,proto3,enum=envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip_HardwareBufferSize" json:"hardware_buffer_size,omitempty"`
	// Threshold of compression service’s input size for software failover.
	// If the size of input request less than the threshold, qatzip will route the request to software
	// compressor. The default value is 1024. The maximum value is 512*1024.
	InputSizeThreshold *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=input_size_threshold,json=inputSizeThreshold,proto3" json:"input_size_threshold,omitempty"`
	// A size of stream buffer. The default value is 128 * 1024. The maximum value is 2*1024*1024 -
	// 5*1024
	StreamBufferSize *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=stream_buffer_size,json=streamBufferSize,proto3" json:"stream_buffer_size,omitempty"`
	// Value for compressor's next output buffer. If not set, defaults to 4096.
	ChunkSize     *wrapperspb.UInt32Value `protobuf:"bytes,5,opt,name=chunk_size,json=chunkSize,proto3" json:"chunk_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Qatzip) Reset() {
	*x = Qatzip{}
	mi := &file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Qatzip) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Qatzip) ProtoMessage() {}

func (x *Qatzip) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Qatzip.ProtoReflect.Descriptor instead.
func (*Qatzip) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescGZIP(), []int{0}
}

func (x *Qatzip) GetCompressionLevel() *wrapperspb.UInt32Value {
	if x != nil {
		return x.CompressionLevel
	}
	return nil
}

func (x *Qatzip) GetHardwareBufferSize() Qatzip_HardwareBufferSize {
	if x != nil {
		return x.HardwareBufferSize
	}
	return Qatzip_DEFAULT
}

func (x *Qatzip) GetInputSizeThreshold() *wrapperspb.UInt32Value {
	if x != nil {
		return x.InputSizeThreshold
	}
	return nil
}

func (x *Qatzip) GetStreamBufferSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.StreamBufferSize
	}
	return nil
}

func (x *Qatzip) GetChunkSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.ChunkSize
	}
	return nil
}

var File_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDesc = "" +
	"\n" +
	"Kcontrib/envoy/extensions/compression/qatzip/compressor/v3alpha/qatzip.proto\x126envoy.extensions.compression.qatzip.compressor.v3alpha\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xdc\x04\n" +
	"\x06Qatzip\x12T\n" +
	"\x11compression_level\x18\x01 \x01(\v2\x1c.google.protobuf.UInt32ValueB\t\xfaB\x06*\x04\x18\t(\x01R\x10compressionLevel\x12\x8d\x01\n" +
	"\x14hardware_buffer_size\x18\x02 \x01(\x0e2Q.envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.HardwareBufferSizeB\b\xfaB\x05\x82\x01\x02\x10\x01R\x12hardwareBufferSize\x12\\\n" +
	"\x14input_size_threshold\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x18\x80\x80 (\x80\x01R\x12inputSizeThreshold\x12X\n" +
	"\x12stream_buffer_size\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x18\x80\xd8\x7f(\x80\bR\x10streamBufferSize\x12I\n" +
	"\n" +
	"chunk_size\x18\x05 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x18\x80\x80\x04(\x80 R\tchunkSize\"i\n" +
	"\x12HardwareBufferSize\x12\v\n" +
	"\aDEFAULT\x10\x00\x12\t\n" +
	"\x05SZ_4K\x10\x01\x12\t\n" +
	"\x05SZ_8K\x10\x02\x12\n" +
	"\n" +
	"\x06SZ_32K\x10\x03\x12\n" +
	"\n" +
	"\x06SZ_64K\x10\x04\x12\v\n" +
	"\aSZ_128K\x10\x05\x12\v\n" +
	"\aSZ_512K\x10\x06B\xc4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Dio.envoyproxy.envoy.extensions.compression.qatzip.compressor.v3alphaB\vQatzipProtoP\x01Zegithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/compression/qatzip/compressor/v3alphab\x06proto3"

var (
	file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDesc), len(file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDescData
}

var file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_goTypes = []any{
	(Qatzip_HardwareBufferSize)(0), // 0: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.HardwareBufferSize
	(*Qatzip)(nil),                 // 1: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.compression_level:type_name -> google.protobuf.UInt32Value
	0, // 1: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.hardware_buffer_size:type_name -> envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.HardwareBufferSize
	2, // 2: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.input_size_threshold:type_name -> google.protobuf.UInt32Value
	2, // 3: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.stream_buffer_size:type_name -> google.protobuf.UInt32Value
	2, // 4: envoy.extensions.compression.qatzip.compressor.v3alpha.Qatzip.chunk_size:type_name -> google.protobuf.UInt32Value
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_init() }
func file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_init() {
	if File_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDesc), len(file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_depIdxs,
		EnumInfos:         file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_enumTypes,
		MessageInfos:      file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto = out.File
	file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_goTypes = nil
	file_contrib_envoy_extensions_compression_qatzip_compressor_v3alpha_qatzip_proto_depIdxs = nil
}
