// Copyright Istio Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//go:build !assert

package krt

// EnableAssertions = false disables krt assertions. These typically are violations of the krt collection requirements.
// Krt assertions can be enabled using build tags, `-tags=assert`
// This is const to allow go's dead code elimination to optimize the conditionals out of the krt core
const EnableAssertions = false
