/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha2

// InferenceModelRewriteSpecApplyConfiguration represents a declarative configuration of the InferenceModelRewriteSpec type for use
// with apply.
//
// InferenceModelRewriteSpec defines the desired state of InferenceModelRewrite.
type InferenceModelRewriteSpecApplyConfiguration struct {
	// PoolRef is a reference to the inference pool.
	PoolRef *PoolObjectReferenceApplyConfiguration `json:"poolRef,omitempty"`
	// --- Precedence and Conflict Resolution ---
	// If multiple InferenceModelRewrite resources target the same
	// InferencePool, the controller will merge them based on precedence.
	//
	// Across all rules specified on applicable rewrites, precedence MUST be
	// given to the match having an "Exact" model match over a generic match
	// (a rule with an empty `matches` array).
	//
	// If ties still exist across multiple InferenceModelRewrite resources (e.g.
	// two rewrites both have an exact match for the same model), matching
	// precedence MUST be determined by the oldest resource based on
	// creation timestamp.
	//
	// If ties still exist within a single InferenceModelRewrite resource, the
	// FIRST matching rule (in list order) is used.
	Rules []InferenceModelRewriteRuleApplyConfiguration `json:"rules,omitempty"`
}

// InferenceModelRewriteSpecApplyConfiguration constructs a declarative configuration of the InferenceModelRewriteSpec type for use with
// apply.
func InferenceModelRewriteSpec() *InferenceModelRewriteSpecApplyConfiguration {
	return &InferenceModelRewriteSpecApplyConfiguration{}
}

// WithPoolRef sets the PoolRef field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the PoolRef field is set to the value of the last call.
func (b *InferenceModelRewriteSpecApplyConfiguration) WithPoolRef(value *PoolObjectReferenceApplyConfiguration) *InferenceModelRewriteSpecApplyConfiguration {
	b.PoolRef = value
	return b
}

// WithRules adds the given value to the Rules field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Rules field.
func (b *InferenceModelRewriteSpecApplyConfiguration) WithRules(values ...*InferenceModelRewriteRuleApplyConfiguration) *InferenceModelRewriteSpecApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithRules")
		}
		b.Rules = append(b.Rules, *values[i])
	}
	return b
}
