// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/stat_sinks/graphite_statsd/v3/graphite_statsd.proto

package graphite_statsdv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GraphiteStatsdSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to StatsdSpecifier:
	//
	//	*GraphiteStatsdSink_Address
	StatsdSpecifier isGraphiteStatsdSink_StatsdSpecifier `protobuf_oneof:"statsd_specifier"`
	// Optional custom metric name prefix. See :ref:`StatsdSink's prefix field
	// <envoy_v3_api_field_config.metrics.v3.StatsdSink.prefix>` for more details.
	Prefix string `protobuf:"bytes,3,opt,name=prefix,proto3" json:"prefix,omitempty"`
	// Optional max datagram size to use when sending UDP messages. By default Envoy
	// will emit one metric per datagram. By specifying a max-size larger than a single
	// metric, Envoy will emit multiple, new-line separated metrics. The max datagram
	// size should not exceed your network's MTU.
	//
	// Note that this value may not be respected if smaller than a single metric.
	MaxBytesPerDatagram *wrapperspb.UInt64Value `protobuf:"bytes,4,opt,name=max_bytes_per_datagram,json=maxBytesPerDatagram,proto3" json:"max_bytes_per_datagram,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *GraphiteStatsdSink) Reset() {
	*x = GraphiteStatsdSink{}
	mi := &file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GraphiteStatsdSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GraphiteStatsdSink) ProtoMessage() {}

func (x *GraphiteStatsdSink) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GraphiteStatsdSink.ProtoReflect.Descriptor instead.
func (*GraphiteStatsdSink) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescGZIP(), []int{0}
}

func (x *GraphiteStatsdSink) GetStatsdSpecifier() isGraphiteStatsdSink_StatsdSpecifier {
	if x != nil {
		return x.StatsdSpecifier
	}
	return nil
}

func (x *GraphiteStatsdSink) GetAddress() *v3.Address {
	if x != nil {
		if x, ok := x.StatsdSpecifier.(*GraphiteStatsdSink_Address); ok {
			return x.Address
		}
	}
	return nil
}

func (x *GraphiteStatsdSink) GetPrefix() string {
	if x != nil {
		return x.Prefix
	}
	return ""
}

func (x *GraphiteStatsdSink) GetMaxBytesPerDatagram() *wrapperspb.UInt64Value {
	if x != nil {
		return x.MaxBytesPerDatagram
	}
	return nil
}

type isGraphiteStatsdSink_StatsdSpecifier interface {
	isGraphiteStatsdSink_StatsdSpecifier()
}

type GraphiteStatsdSink_Address struct {
	// The UDP address of a running Graphite-compliant listener. If specified,
	// statistics will be flushed to this address.
	Address *v3.Address `protobuf:"bytes,1,opt,name=address,proto3,oneof"`
}

func (*GraphiteStatsdSink_Address) isGraphiteStatsdSink_StatsdSpecifier() {}

var File_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto protoreflect.FileDescriptor

const file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDesc = "" +
	"\n" +
	"Denvoy/extensions/stat_sinks/graphite_statsd/v3/graphite_statsd.proto\x12.envoy.extensions.stat_sinks.graphite_statsd.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xdc\x01\n" +
	"\x12GraphiteStatsdSink\x129\n" +
	"\aaddress\x18\x01 \x01(\v2\x1d.envoy.config.core.v3.AddressH\x00R\aaddress\x12\x16\n" +
	"\x06prefix\x18\x03 \x01(\tR\x06prefix\x12Z\n" +
	"\x16max_bytes_per_datagram\x18\x04 \x01(\v2\x1c.google.protobuf.UInt64ValueB\a\xfaB\x042\x02 \x00R\x13maxBytesPerDatagramB\x17\n" +
	"\x10statsd_specifier\x12\x03\xf8B\x01B\xc6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"<io.envoyproxy.envoy.extensions.stat_sinks.graphite_statsd.v3B\x13GraphiteStatsdProtoP\x01Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/stat_sinks/graphite_statsd/v3;graphite_statsdv3b\x06proto3"

var (
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescOnce sync.Once
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescData []byte
)

func file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescGZIP() []byte {
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDesc), len(file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDesc)))
	})
	return file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDescData
}

var file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_goTypes = []any{
	(*GraphiteStatsdSink)(nil),     // 0: envoy.extensions.stat_sinks.graphite_statsd.v3.GraphiteStatsdSink
	(*v3.Address)(nil),             // 1: envoy.config.core.v3.Address
	(*wrapperspb.UInt64Value)(nil), // 2: google.protobuf.UInt64Value
}
var file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.stat_sinks.graphite_statsd.v3.GraphiteStatsdSink.address:type_name -> envoy.config.core.v3.Address
	2, // 1: envoy.extensions.stat_sinks.graphite_statsd.v3.GraphiteStatsdSink.max_bytes_per_datagram:type_name -> google.protobuf.UInt64Value
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_init() }
func file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_init() {
	if File_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto != nil {
		return
	}
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_msgTypes[0].OneofWrappers = []any{
		(*GraphiteStatsdSink_Address)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDesc), len(file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_msgTypes,
	}.Build()
	File_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto = out.File
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_goTypes = nil
	file_envoy_extensions_stat_sinks_graphite_statsd_v3_graphite_statsd_proto_depIdxs = nil
}
