// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/tls/cert_selectors/on_demand_secret/v3/config.proto

package on_demand_secretv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Fetches the secret on-demand while allowing the parent cluster or listener to accept connections
// without warming. During the handshake, a secret name is derived from the peer hello message, an
// SDS resource request starts, and the handshake is paused. Once an SDS response is received with a
// resource, the handshake is resumed with the provided certificate. If the SDS server indicates the
// resource removal, the handshake is failed, and the SDS subscription to the resource is stopped.
//
// Similar to the regular SDS, the certificate is configured using the outer common TLS context,
// e.g. by setting the FIPS compliance policy on the loaded certificate.
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Defines the configuration source of the secrets.
	ConfigSource *v3.ConfigSource `protobuf:"bytes,1,opt,name=config_source,json=configSource,proto3" json:"config_source,omitempty"`
	// Extension point to specify a function to compute the secret name. The extension is called
	// during the TLS handshake after receiving the *CLIENT HELLO* message from the client for the
	// downstream certificate selector, and using the transport socket options and *SERVER HELLO* for
	// the upstream certificate selector.
	// [#extension-category: envoy.tls.certificate_mappers,envoy.tls.upstream_certificate_mappers]
	CertificateMapper *v3.TypedExtensionConfig `protobuf:"bytes,2,opt,name=certificate_mapper,json=certificateMapper,proto3" json:"certificate_mapper,omitempty"`
	// A list of secret resource names to start fetching on configuration load (prior to receiving any
	// requests). The parent resource initializes immediately without waiting for the fetch to
	// complete.
	PrefetchSecretNames []string `protobuf:"bytes,3,rep,name=prefetch_secret_names,json=prefetchSecretNames,proto3" json:"prefetch_secret_names,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetConfigSource() *v3.ConfigSource {
	if x != nil {
		return x.ConfigSource
	}
	return nil
}

func (x *Config) GetCertificateMapper() *v3.TypedExtensionConfig {
	if x != nil {
		return x.CertificateMapper
	}
	return nil
}

func (x *Config) GetPrefetchSecretNames() []string {
	if x != nil {
		return x.PrefetchSecretNames
	}
	return nil
}

var File_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDesc = "" +
	"\n" +
	"Venvoy/extensions/transport_sockets/tls/cert_selectors/on_demand_secret/v3/config.proto\x12Ienvoy.extensions.transport_sockets.tls.cert_selectors.on_demand_secret.v3\x1a(envoy/config/core/v3/config_source.proto\x1a$envoy/config/core/v3/extension.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf4\x01\n" +
	"\x06Config\x12Q\n" +
	"\rconfig_source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fconfigSource\x12c\n" +
	"\x12certificate_mapper\x18\x02 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x11certificateMapper\x122\n" +
	"\x15prefetch_secret_names\x18\x03 \x03(\tR\x13prefetchSecretNamesB\xf6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Wio.envoyproxy.envoy.extensions.transport_sockets.tls.cert_selectors.on_demand_secret.v3B\vConfigProtoP\x01Z\x83\x01github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/cert_selectors/on_demand_secret/v3;on_demand_secretv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_goTypes = []any{
	(*Config)(nil),                  // 0: envoy.extensions.transport_sockets.tls.cert_selectors.on_demand_secret.v3.Config
	(*v3.ConfigSource)(nil),         // 1: envoy.config.core.v3.ConfigSource
	(*v3.TypedExtensionConfig)(nil), // 2: envoy.config.core.v3.TypedExtensionConfig
}
var file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.transport_sockets.tls.cert_selectors.on_demand_secret.v3.Config.config_source:type_name -> envoy.config.core.v3.ConfigSource
	2, // 1: envoy.extensions.transport_sockets.tls.cert_selectors.on_demand_secret.v3.Config.certificate_mapper:type_name -> envoy.config.core.v3.TypedExtensionConfig
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_init()
}
func file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_init() {
	if File_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDesc), len(file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto = out.File
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_tls_cert_selectors_on_demand_secret_v3_config_proto_depIdxs = nil
}
