/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha2

import (
	apixv1alpha2 "sigs.k8s.io/gateway-api-inference-extension/apix/v1alpha2"
)

// ModelMatchApplyConfiguration represents a declarative configuration of the ModelMatch type for use
// with apply.
//
// ModelMatch defines how to match against the model name in the request body.
type ModelMatchApplyConfiguration struct {
	// Type specifies the kind of string matching to use.
	// Supported value is "Exact". Defaults to "Exact".
	Type *apixv1alpha2.MatchValidationType `json:"type,omitempty"`
	// Value is the model name string to match against.
	Value *string `json:"value,omitempty"`
}

// ModelMatchApplyConfiguration constructs a declarative configuration of the ModelMatch type for use with
// apply.
func ModelMatch() *ModelMatchApplyConfiguration {
	return &ModelMatchApplyConfiguration{}
}

// WithType sets the Type field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Type field is set to the value of the last call.
func (b *ModelMatchApplyConfiguration) WithType(value apixv1alpha2.MatchValidationType) *ModelMatchApplyConfiguration {
	b.Type = &value
	return b
}

// WithValue sets the Value field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Value field is set to the value of the last call.
func (b *ModelMatchApplyConfiguration) WithValue(value string) *ModelMatchApplyConfiguration {
	b.Value = &value
	return b
}
