// Copyright Istio Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package krt

type TestingDummyContext struct{}

func (t TestingDummyContext) _internalHandler() {
}

func (t TestingDummyContext) registerDependency(d *dependency, s Syncer, f func(f erasedEventHandler) Syncer) {
}

func (t TestingDummyContext) name() string {
	return "testing"
}

func (t TestingDummyContext) DiscardResult() {
	panic("TestingDummyContext cannot DiscardResult")
}

var (
	_ registerDependency = TestingDummyContext{}
	_ HandlerContext     = TestingDummyContext{}
)
