// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/network/metadata_exchange/v3/metadata_exchange.proto

package config

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#protodoc-title: MetadataExchange protocol match and data transfer]
// MetadataExchange protocol match and data transfer
type MetadataExchange struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Protocol that Alpn should support on the server.
	// [#comment:TODO(GargNupur): Make it a list.]
	Protocol string `protobuf:"bytes,1,opt,name=protocol,proto3" json:"protocol,omitempty"`
	// If true, will attempt to use WDS in case the prefix peer metadata is not available.
	EnableDiscovery bool `protobuf:"varint,2,opt,name=enable_discovery,json=enableDiscovery,proto3" json:"enable_discovery,omitempty"`
	// Additional labels to be added to the peer metadata to help your understand the traffic.
	// e.g. “role“, “location“ etc.
	AdditionalLabels []string `protobuf:"bytes,3,rep,name=additional_labels,json=additionalLabels,proto3" json:"additional_labels,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *MetadataExchange) Reset() {
	*x = MetadataExchange{}
	mi := &file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetadataExchange) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetadataExchange) ProtoMessage() {}

func (x *MetadataExchange) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetadataExchange.ProtoReflect.Descriptor instead.
func (*MetadataExchange) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescGZIP(), []int{0}
}

func (x *MetadataExchange) GetProtocol() string {
	if x != nil {
		return x.Protocol
	}
	return ""
}

func (x *MetadataExchange) GetEnableDiscovery() bool {
	if x != nil {
		return x.EnableDiscovery
	}
	return false
}

func (x *MetadataExchange) GetAdditionalLabels() []string {
	if x != nil {
		return x.AdditionalLabels
	}
	return nil
}

var File_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDesc = "" +
	"\n" +
	"Ucontrib/envoy/extensions/filters/network/metadata_exchange/v3/metadata_exchange.proto\x12!envoy.tcp.metadataexchange.config\x1a\x1dudpa/annotations/status.proto\"\x86\x01\n" +
	"\x10MetadataExchange\x12\x1a\n" +
	"\bprotocol\x18\x01 \x01(\tR\bprotocol\x12)\n" +
	"\x10enable_discovery\x18\x02 \x01(\bR\x0fenableDiscovery\x12+\n" +
	"\x11additional_labels\x18\x03 \x03(\tR\x10additionalLabelsB\xa4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"/io.envoyproxy.envoy.tcp.metadataexchange.configB\x15MetadataExchangeProtoP\x01ZPgithub.com/envoyproxy/go-control-plane/contrib/envoy/tcp/metadataexchange/configb\x06proto3"

var (
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_goTypes = []any{
	(*MetadataExchange)(nil), // 0: envoy.tcp.metadataexchange.config.MetadataExchange
}
var file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_init()
}
func file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_init() {
	if File_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto = out.File
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_network_metadata_exchange_v3_metadata_exchange_proto_depIdxs = nil
}
