// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/quic/server_preferred_address/v3/fixed_server_preferred_address_config.proto

package server_preferred_addressv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for FixedServerPreferredAddressConfig.
type FixedServerPreferredAddressConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// String representation of IPv4 address, i.e. "127.0.0.2".
	// If not specified, none will be configured.
	Ipv4Address string `protobuf:"bytes,1,opt,name=ipv4_address,json=ipv4Address,proto3" json:"ipv4_address,omitempty"`
	// The IPv4 address to advertise to clients for Server Preferred Address.
	// This field takes precedence over
	// :ref:`ipv4_address <envoy_v3_api_field_extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.ipv4_address>`.
	Ipv4Config *FixedServerPreferredAddressConfig_AddressFamilyConfig `protobuf:"bytes,3,opt,name=ipv4_config,json=ipv4Config,proto3" json:"ipv4_config,omitempty"`
	// String representation of IPv6 address, i.e. "::1".
	// If not specified, none will be configured.
	Ipv6Address string `protobuf:"bytes,2,opt,name=ipv6_address,json=ipv6Address,proto3" json:"ipv6_address,omitempty"`
	// The IPv6 address to advertise to clients for Server Preferred Address.
	// This field takes precedence over
	// :ref:`ipv6_address <envoy_v3_api_field_extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.ipv6_address>`.
	Ipv6Config    *FixedServerPreferredAddressConfig_AddressFamilyConfig `protobuf:"bytes,4,opt,name=ipv6_config,json=ipv6Config,proto3" json:"ipv6_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FixedServerPreferredAddressConfig) Reset() {
	*x = FixedServerPreferredAddressConfig{}
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FixedServerPreferredAddressConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FixedServerPreferredAddressConfig) ProtoMessage() {}

func (x *FixedServerPreferredAddressConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FixedServerPreferredAddressConfig.ProtoReflect.Descriptor instead.
func (*FixedServerPreferredAddressConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescGZIP(), []int{0}
}

func (x *FixedServerPreferredAddressConfig) GetIpv4Address() string {
	if x != nil {
		return x.Ipv4Address
	}
	return ""
}

func (x *FixedServerPreferredAddressConfig) GetIpv4Config() *FixedServerPreferredAddressConfig_AddressFamilyConfig {
	if x != nil {
		return x.Ipv4Config
	}
	return nil
}

func (x *FixedServerPreferredAddressConfig) GetIpv6Address() string {
	if x != nil {
		return x.Ipv6Address
	}
	return ""
}

func (x *FixedServerPreferredAddressConfig) GetIpv6Config() *FixedServerPreferredAddressConfig_AddressFamilyConfig {
	if x != nil {
		return x.Ipv6Config
	}
	return nil
}

// Addresses for server preferred address for a single address family (IPv4 or IPv6).
type FixedServerPreferredAddressConfig_AddressFamilyConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The server preferred address sent to clients.
	//
	// .. note::
	//
	//	Envoy currently requires all packets for a QUIC connection to arrive on the same port. Therefore, unless a
	//	:ref:`dnat_address <envoy_v3_api_field_extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig.dnat_address>`
	//	is explicitly configured, the port specified here must be set to zero. In such cases, Envoy will automatically
	//	use the listener's port.
	Address *v3.SocketAddress `protobuf:"bytes,1,opt,name=address,proto3" json:"address,omitempty"`
	// If a DNAT exists between the client and Envoy, this is the address where Envoy will observe incoming server
	// preferred address packets. If unspecified, Envoy assumes there is no DNAT, and packets will be sent directly
	// to the address advertised to clients as the server preferred address.
	//
	// .. note::
	//
	//	Envoy currently requires all packets for a QUIC connection to arrive on the same port. Consequently, the
	//	port for this address must be set to zero, with Envoy defaulting to the listener's port instead.
	DnatAddress   *v3.SocketAddress `protobuf:"bytes,2,opt,name=dnat_address,json=dnatAddress,proto3" json:"dnat_address,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FixedServerPreferredAddressConfig_AddressFamilyConfig) Reset() {
	*x = FixedServerPreferredAddressConfig_AddressFamilyConfig{}
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FixedServerPreferredAddressConfig_AddressFamilyConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FixedServerPreferredAddressConfig_AddressFamilyConfig) ProtoMessage() {}

func (x *FixedServerPreferredAddressConfig_AddressFamilyConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FixedServerPreferredAddressConfig_AddressFamilyConfig.ProtoReflect.Descriptor instead.
func (*FixedServerPreferredAddressConfig_AddressFamilyConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescGZIP(), []int{0, 0}
}

func (x *FixedServerPreferredAddressConfig_AddressFamilyConfig) GetAddress() *v3.SocketAddress {
	if x != nil {
		return x.Address
	}
	return nil
}

func (x *FixedServerPreferredAddressConfig_AddressFamilyConfig) GetDnatAddress() *v3.SocketAddress {
	if x != nil {
		return x.DnatAddress
	}
	return nil
}

var File_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto protoreflect.FileDescriptor

const file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDesc = "" +
	"\n" +
	"]envoy/extensions/quic/server_preferred_address/v3/fixed_server_preferred_address_config.proto\x121envoy.extensions.quic.server_preferred_address.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1dudpa/annotations/status.proto\"\xa0\x04\n" +
	"!FixedServerPreferredAddressConfig\x12!\n" +
	"\fipv4_address\x18\x01 \x01(\tR\vipv4Address\x12\x89\x01\n" +
	"\vipv4_config\x18\x03 \x01(\v2h.envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfigR\n" +
	"ipv4Config\x12!\n" +
	"\fipv6_address\x18\x02 \x01(\tR\vipv6Address\x12\x89\x01\n" +
	"\vipv6_config\x18\x04 \x01(\v2h.envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfigR\n" +
	"ipv6Config\x1a\x9c\x01\n" +
	"\x13AddressFamilyConfig\x12=\n" +
	"\aaddress\x18\x01 \x01(\v2#.envoy.config.core.v3.SocketAddressR\aaddress\x12F\n" +
	"\fdnat_address\x18\x02 \x01(\v2#.envoy.config.core.v3.SocketAddressR\vdnatAddressB\xe8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"?io.envoyproxy.envoy.extensions.quic.server_preferred_address.v3B&FixedServerPreferredAddressConfigProtoP\x01Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/quic/server_preferred_address/v3;server_preferred_addressv3b\x06proto3"

var (
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescOnce sync.Once
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescData []byte
)

func file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescGZIP() []byte {
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDesc), len(file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDesc)))
	})
	return file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDescData
}

var file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_goTypes = []any{
	(*FixedServerPreferredAddressConfig)(nil),                     // 0: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig
	(*FixedServerPreferredAddressConfig_AddressFamilyConfig)(nil), // 1: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig
	(*v3.SocketAddress)(nil),                                      // 2: envoy.config.core.v3.SocketAddress
}
var file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.ipv4_config:type_name -> envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig
	1, // 1: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.ipv6_config:type_name -> envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig
	2, // 2: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig.address:type_name -> envoy.config.core.v3.SocketAddress
	2, // 3: envoy.extensions.quic.server_preferred_address.v3.FixedServerPreferredAddressConfig.AddressFamilyConfig.dnat_address:type_name -> envoy.config.core.v3.SocketAddress
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_init()
}
func file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_init() {
	if File_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDesc), len(file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_msgTypes,
	}.Build()
	File_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto = out.File
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_goTypes = nil
	file_envoy_extensions_quic_server_preferred_address_v3_fixed_server_preferred_address_config_proto_depIdxs = nil
}
