// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: contrib/envoy/extensions/filters/network/rocketmq_proxy/v3/rocketmq_proxy.proto

package rocketmq_proxyv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RocketmqProxy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The human readable prefix to use when emitting statistics.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// The route table for the connection manager is specified in this property.
	RouteConfig *RouteConfiguration `protobuf:"bytes,2,opt,name=route_config,json=routeConfig,proto3" json:"route_config,omitempty"`
	// The largest duration transient object expected to live, more than 10s is recommended.
	TransientObjectLifeSpan *durationpb.Duration `protobuf:"bytes,3,opt,name=transient_object_life_span,json=transientObjectLifeSpan,proto3" json:"transient_object_life_span,omitempty"`
	// If develop_mode is enabled, this proxy plugin may work without dedicated traffic intercepting
	// facility without considering backward compatibility of exiting RocketMQ client SDK.
	DevelopMode   bool `protobuf:"varint,4,opt,name=develop_mode,json=developMode,proto3" json:"develop_mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RocketmqProxy) Reset() {
	*x = RocketmqProxy{}
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RocketmqProxy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RocketmqProxy) ProtoMessage() {}

func (x *RocketmqProxy) ProtoReflect() protoreflect.Message {
	mi := &file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RocketmqProxy.ProtoReflect.Descriptor instead.
func (*RocketmqProxy) Descriptor() ([]byte, []int) {
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescGZIP(), []int{0}
}

func (x *RocketmqProxy) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *RocketmqProxy) GetRouteConfig() *RouteConfiguration {
	if x != nil {
		return x.RouteConfig
	}
	return nil
}

func (x *RocketmqProxy) GetTransientObjectLifeSpan() *durationpb.Duration {
	if x != nil {
		return x.TransientObjectLifeSpan
	}
	return nil
}

func (x *RocketmqProxy) GetDevelopMode() bool {
	if x != nil {
		return x.DevelopMode
	}
	return false
}

var File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto protoreflect.FileDescriptor

const file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDesc = "" +
	"\n" +
	"Ocontrib/envoy/extensions/filters/network/rocketmq_proxy/v3/rocketmq_proxy.proto\x122envoy.extensions.filters.network.rocketmq_proxy.v3\x1aFcontrib/envoy/extensions/filters/network/rocketmq_proxy/v3/route.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9f\x02\n" +
	"\rRocketmqProxy\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12i\n" +
	"\froute_config\x18\x02 \x01(\v2F.envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfigurationR\vrouteConfig\x12V\n" +
	"\x1atransient_object_life_span\x18\x03 \x01(\v2\x19.google.protobuf.DurationR\x17transientObjectLifeSpan\x12!\n" +
	"\fdevelop_mode\x18\x04 \x01(\bR\vdevelopModeB\xd4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.filters.network.rocketmq_proxy.v3B\x12RocketmqProxyProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/rocketmq_proxy/v3;rocketmq_proxyv3b\x06proto3"

var (
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescOnce sync.Once
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescData []byte
)

func file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescGZIP() []byte {
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescOnce.Do(func() {
		file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDesc)))
	})
	return file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDescData
}

var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_goTypes = []any{
	(*RocketmqProxy)(nil),       // 0: envoy.extensions.filters.network.rocketmq_proxy.v3.RocketmqProxy
	(*RouteConfiguration)(nil),  // 1: envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfiguration
	(*durationpb.Duration)(nil), // 2: google.protobuf.Duration
}
var file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.rocketmq_proxy.v3.RocketmqProxy.route_config:type_name -> envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfiguration
	2, // 1: envoy.extensions.filters.network.rocketmq_proxy.v3.RocketmqProxy.transient_object_life_span:type_name -> google.protobuf.Duration
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() {
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_init()
}
func file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_init() {
	if File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto != nil {
		return
	}
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_route_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDesc), len(file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_goTypes,
		DependencyIndexes: file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_depIdxs,
		MessageInfos:      file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_msgTypes,
	}.Build()
	File_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto = out.File
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_goTypes = nil
	file_contrib_envoy_extensions_filters_network_rocketmq_proxy_v3_rocketmq_proxy_proto_depIdxs = nil
}
